<?php

/**
 * Plugin Name: Builderius PRO
 * Plugin URI: https://builderius.io
 * Description: Professional site builder for WordPress. Create astonishing designs with easy to use drag and drop interface.
 * Version: 0.12.1
 * BuilderiusMinVersion: 0.12.0
 * Contributors: builderius
 * Author: 20script
 * Author URI: https://www.20script.ir
 * Domain Path: /languages/
 * Text Domain: builderius
 * Requires at least: 5.4
 * Tested up to: 6.3
 * Requires PHP: 7.3
 */

if (!defined('ABSPATH')) {
    exit;
    // Exit if accessed directly.
} elseif (strpos($_SERVER['PHP_SELF'], 'favicon.ico') !== false) {
    return;
}

require_once("vendor-prefixed/scoper-autoload.php");

if (!version_compare( PHP_VERSION, '7.2', '>=')) {
    add_action('admin_notices', 'builderius_pro_fail_php_version');
} elseif (!version_compare(get_bloginfo('version'), '5.4', '>=')) {
    add_action( 'admin_notices', 'builderius_pro_fail_wp_version' );
} else {
    include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
    $builderiusData = get_plugin_data( WP_PLUGIN_DIR.'/builderius/builderius.php', false, false);
    $builderiusProData = get_plugin_data( WP_PLUGIN_DIR.'/builderius-pro/builderius-pro.php', false, false);
    if (!is_plugin_active('builderius/builderius.php')) {
        add_action( 'admin_notices', 'builderius_pro_fail_no_builderius_plugin' );
        deactivate_plugins('builderius-pro/builderius-pro.php');
    } elseif (isset($builderiusProData['BuilderiusMinVersion'])) {
        if (!version_compare($builderiusData['Version'], $builderiusProData['BuilderiusMinVersion'], '>=')) {
            add_action( 'admin_notices', 'builderius_pro_fail_builderius_version' );
            deactivate_plugins('builderius-pro/builderius-pro.php');
        }
    } else {
        add_action("activated_plugin", function () {
            $wp_path_to_this_file = preg_replace('/(.*)plugins\/(.*)$/', WP_PLUGIN_DIR . "/$2", __FILE__);
            $this_plugin = plugin_basename(trim($wp_path_to_this_file));
            $active_plugins = get_option('active_plugins');
            $this_plugin_key = array_search($this_plugin, $active_plugins);
            $builderius_plugin_key = array_search('builderius/builderius.php', $active_plugins);
            if ($this_plugin_key > $builderius_plugin_key) {
                array_splice(
                    $active_plugins,
                    $builderius_plugin_key - 1,
                    0,
                    array_splice($active_plugins, $this_plugin_key, 1)
                );
                update_option('active_plugins', $active_plugins);
            }
        });
        add_filter('builderius_add_root_dir', function ($roots) {
            $roots[plugin_basename(__FILE__)][] = realpath(dirname(__FILE__)) . '/src';

            return $roots;
        });
    }
}

/**
 * PHP version check notice
 * @return void
 */
function builderius_pro_fail_php_version()
{
    $message = sprintf(
        esc_html__( 'Builderius PRO requires PHP version %s+. It is NOT ACTIVATED!', 'builderius' ), '7.2'
    );
    echo  builderius_pro_fail_notice_wrapper( $message ) ;
}

/**
 * WP version check notice
 * @return void
 */
function builderius_pro_fail_wp_version()
{
    $message = sprintf(
        esc_html__( 'Builderius PRO requires WordPress version %s+. It is NOT ACTIVATED!', 'builderius' ), '5.4'
    );
    echo  builderius_pro_fail_notice_wrapper( $message ) ;
}

/**
 * Builderius installation check notice
 * @return void
 */
function builderius_pro_fail_no_builderius_plugin()
{
    $message = 'The plugin <strong>"Builderius PRO"</strong> requires the plugin <strong>"Builderius"</strong> to be activated.';
    echo  builderius_pro_fail_notice_wrapper( $message ) ;
}

/**
 * Builderius installation check notice
 * @return void
 */
function builderius_pro_fail_builderius_version()
{
    $builderiusProData = get_plugin_data( WP_PLUGIN_DIR.'/builderius-pro/builderius-pro.php');
    $message = sprintf('The plugin <strong>"Builderius PRO"</strong> version %1$s requires the plugin <strong>"Builderius"</strong> version %1$s or higher.', $builderiusProData['Version']);
    echo  builderius_pro_fail_notice_wrapper( $message ) ;
}

/**
 * Fail notices wrapper. Returns sanitized string.
 * @param string
 * @return string
 */
function builderius_pro_fail_notice_wrapper($message)
{
    $html_message = sprintf('<div class="error">%s</div>', wpautop($message));
    return wp_kses_post( $html_message );
}

