<?php

namespace BuilderiusPro\Bundle\BuilderBundle\Hook;

use Builderius\MooMoo\Platform\Bundle\HookBundle\Model\AbstractAction;
use BuilderiusPro\Bundle\BuilderBundle\Helper\UserSwitchingHelper;

class UserSwitchingInitHook extends AbstractAction
{
    const APPLICATION = 'Builderius';

    /**
     * @inheritDoc
     */
    public function getFunction()
    {
        if (!isset($_REQUEST['action'])) {
            return;
        }

        $current_user = apply_filters('builderius_get_current_user', wp_get_current_user());

        switch ($_REQUEST['action']) {

            // We're attempting to switch to another user:
            case 'builderius_switch_to_user':
                if (isset($_REQUEST['user_id'])) {
                    $user_id = absint($_REQUEST['user_id']);
                } else {
                    $user_id = 0;
                }

                // Check authentication:
                if (!$current_user->has_cap('builderius-development')) {
                    wp_die(esc_html__('Could not switch users.', 'user-switching'), 403);
                }

                // Check intent:
                check_admin_referer("switch_to_user_{$user_id}");

                // Switch user:
                $oldUser = UserSwitchingHelper::getOldUser();
                if ($oldUser) {
                    $user = UserSwitchingHelper::switchToUser($user_id, UserSwitchingHelper::remember(), false, false);
                } else {
                    $user = UserSwitchingHelper::switchToUser($user_id, UserSwitchingHelper::remember(), true);
                }
                if ($user) {
                    update_user_meta($current_user->ID, 'builderius_dev_preview_as', sprintf('user_%d', $user_id));
                    wp_safe_redirect(add_query_arg([], UserSwitchingHelper::currentUrl()), 302, self::APPLICATION);
                    exit;
                } else {
                    wp_die(esc_html__('Could not switch users.', 'user-switching'), 404);
                }
                break;

            // We're attempting to switch back to the originating user:
            case 'builderius_switch_to_own_user':
                // Fetch the originating user data:
                $old_user = UserSwitchingHelper::getOldUser();
                if (!$old_user) {
                    wp_die(esc_html__('Could not switch users.', 'user-switching'), 400);
                }

                // Check authentication:
                if (!UserSwitchingHelper::authenticateOldUser($old_user)) {
                    wp_die(esc_html__('Could not switch users.', 'user-switching'), 403);
                }

                // Check intent:
                check_admin_referer("switch_to_own_user_{$old_user->ID}");

                // Switch user:
                if (UserSwitchingHelper::switchToUser($old_user->ID, UserSwitchingHelper::remember(), false, true)) {
                    update_user_meta($current_user->ID, 'builderius_dev_preview_as', 'own_user');
                    if (!empty($_REQUEST['interim-login']) && function_exists('login_header')) {
                        $GLOBALS['interim_login'] = 'success'; // @codingStandardsIgnoreLine
                        login_header('', '');
                        exit;
                    }

                    wp_safe_redirect(add_query_arg([], UserSwitchingHelper::currentUrl()), 302, self::APPLICATION);
                    exit;
                } else {
                    wp_die(esc_html__('Could not switch users.', 'user-switching'), 404);
                }
                break;

            // We're attempting to switch off the current user:
            case 'builderius_switch_to_unauthenticated':
                // Check authentication:
                if (!$current_user || !$current_user->has_cap('builderius-development')) {
                    /* Translators: "switch off" means to temporarily log out */
                    wp_die(esc_html__('Could not switch off.', 'user-switching'));
                }

                // Check intent:
                check_admin_referer("switch_to_unauthenticated_{$current_user->ID}");

                // Switch off:
                if (UserSwitchingHelper::switchOffUser()) {
                    update_user_meta($current_user->ID, 'builderius_dev_preview_as', 'unauthenticated');
                    wp_safe_redirect(add_query_arg([], UserSwitchingHelper::currentUrl()), 302, self::APPLICATION);
                    exit;
                } else {
                    /* Translators: "switch off" means to temporarily log out */
                    wp_die(esc_html__('Could not switch off.', 'user-switching'));
                }
                break;
        }
    }
}