<?php

namespace BuilderiusPro\Bundle\BuilderBundle\Hook;

use Builderius\MooMoo\Platform\Bundle\HookBundle\Model\AbstractAction;
use BuilderiusPro\Bundle\BuilderBundle\Helper\UserSwitchingHelper;

class UserSwitchingRestApiNonceChangeHook extends AbstractAction
{
    /**
     * @inheritDoc
     */
    public function getFunction()
    {
        $wp_scripts = wp_scripts();

        if (isset($wp_scripts->registered['wp-api-request'])) {
            $wp_scripts->registered['wp-api-request']->extra['data'] = null;
            wp_localize_script(
                'wp-api-request',
                'wpApiSettings',
                [
                    'root' => esc_url_raw(get_rest_url()),
                    'nonce' => (wp_installing() && !is_multisite()) ? '' : $this->wp_create_nonce('wp_rest'),
                    'versionString' => 'wp/v2/'
                ]
            );
        }
        if (isset($wp_scripts->registered['heartbeat'])) {
            $wp_scripts->registered['heartbeat']->extra['data'] = null;
            wp_localize_script(
                'heartbeat',
                'heartbeatSettings',
                [
                    'ajaxurl' => admin_url( 'admin-ajax.php', 'relative' ),
                    'nonce' => (wp_installing() && !is_multisite()) ? '' : $this->wp_create_nonce('heartbeat-nonce')
                ]
            );
        }
    }

    /**
     * @param int $action
     * @return false|string
     */
    private function wp_create_nonce($action = -1)
    {
        $user = apply_filters('builderius_get_current_user', wp_get_current_user());
        $uid = (int)$user->ID;
        if (!$uid) {
            /** This filter is documented in wp-includes/pluggable.php */
            $uid = apply_filters('nonce_user_logged_out', $uid, $action);
        }

        if ($user === wp_get_current_user()) {
            $token = wp_get_session_token();
        } else {
            $cookie = UserSwitchingHelper::getOldUserCookie();
            $cookie_elements = explode('|', $cookie);
            if (count($cookie_elements) !== 4) {
                return false;
            }

            list($username, $expiration, $token, $hmac) = $cookie_elements;

            $cookie = compact('username', 'expiration', 'token', 'hmac');
            $token = !empty($cookie['token']) ? $cookie['token'] : '';
        }
        $i = wp_nonce_tick();

        return substr(wp_hash($i . '|' . $action . '|' . $uid . '|' . $token, 'nonce'), -12, 10);
    }
}