<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\ModuleBundle\Model\ModuleAssetWithConditionInterface;
use Builderius\Bundle\TemplateBundle\EventListener\AbstractModulesAssetsRegistrationEventListener;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\AssetAwareInterface;
use BuilderiusPro\Bundle\ComponentBundle\Twig\TemplatePartDataVarsExtension;

abstract class AbstractTemplatePartModulesAssetsRegistrationEventListener extends AbstractModulesAssetsRegistrationEventListener
{
    /**
     * @var TemplatePartDataVarsExtension
     */
    protected $twigTemplatePartExtension;

    /**
     * @param string|array $moduleName
     * @param string $typeName
     * @param string $technologyName
     * @param array $assets
     * @return array
     * @throws \Exception
     */
    protected function processModuleAssets($moduleName, $typeName, $technologyName, array $assets)
    {
        $configVersions = $this->configVersion !== null ? $this->configVersion : [];
        if (is_string($moduleName)) {
            $module = $this->modulesProvider->getModule($moduleName, 'template_part', $technologyName, false, $configVersions);
            if ($module && $module instanceof AssetAwareInterface && !empty($module->getAssets())) {
                foreach ($module->getAssets() as $asset) {
                    if (!in_array($asset, $assets)) {
                        $assets[] = $asset;
                    }
                }
            }
        } elseif (is_array($moduleName) && isset($moduleName['name']) && isset($moduleName['visibilityCondition'])) {
            $condResult = $this->twigTemplatePartExtension->evaluateVisibilityCondition($typeName, $moduleName['visibilityCondition']);
            if (true === $condResult) {
                $module = $this->modulesProvider->getModule($moduleName['name'], 'template_part', $technologyName, false, $configVersions);
                if ($module && $module instanceof AssetAwareInterface && !empty($module->getAssets())) {
                    $moduleAssets = [];
                    foreach ($module->getAssets() as $asset) {
                        if ($asset instanceof ModuleAssetWithConditionInterface) {
                            $moduleAssets = $this->processModuleAssetWithCondition($asset, $moduleName, $moduleAssets);
                        } else {
                            if (!in_array($asset, $moduleAssets)) {
                                $moduleAssets[] = $asset;
                            }
                        }
                    }
                    if (null !== $this->configVersion) {
                        $moduleAssets = $this->processVersionRelatedModuleAssets($moduleAssets, $this->configVersion);
                    }
                    foreach ($moduleAssets as $moduleAsset) {
                        if (!in_array($moduleAsset, $assets)) {
                            $assets[] = $moduleAsset;
                        }
                    }
                }
            }
        }

        return $assets;
    }
}