<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\ModuleBundle\Model\ModuleInlineAssetWithConditionInterface;
use Builderius\Bundle\TemplateBundle\EventListener\AbstractModulesInlineAssetsRegistrationEventListener;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\AssetAwareInterface;
use BuilderiusPro\Bundle\ComponentBundle\Twig\TemplatePartDataVarsExtension;

abstract class AbstractTemplatePartModulesInlineAssetsRegistrationEventListener extends AbstractModulesInlineAssetsRegistrationEventListener
{
    /**
     * @var TemplatePartDataVarsExtension
     */
    protected $twigTemplatePartExtension;

    /**
     * @param string|array $moduleName
     * @param string $typeName
     * @param string $technologyName
     * @param string $assetType
     * @param array $assets
     * @return array
     * @throws \Exception
     */
    protected function processModuleAssets($moduleName, $typeName, $technologyName, $assetType, array $assets)
    {
        $configVersions = $this->configVersion !== null ? $this->configVersion : [];
        if (is_string($moduleName)) {
            $module = $this->modulesProvider->getModule($moduleName, 'template_part', $technologyName, false, $configVersions);
            if ($module && $module instanceof AssetAwareInterface && !empty($module->getInlineAssets())) {
                foreach ($module->getInlineAssets() as $asset) {
                    if ($assetType === $asset->getType() && !in_array($asset, $assets)) {
                        $assets[] = $asset;
                    }
                }
            }
        } elseif (is_array($moduleName) && isset($moduleName['name']) && isset($moduleName['visibilityCondition'])) {
            $condResult = $this->twigTemplatePartExtension->evaluateVisibilityCondition($typeName, $moduleName['visibilityCondition']);
            if (true === $condResult) {
                $module = $this->modulesProvider->getModule($moduleName['name'], 'template_part', $technologyName, false, $configVersions);
                if ($module && $module instanceof AssetAwareInterface && !empty($module->getInlineAssets())) {
                    $moduleAssets = [];
                    foreach ($module->getInlineAssets() as $asset) {
                        if ($assetType === $asset->getType()) {
                            if ($asset instanceof ModuleInlineAssetWithConditionInterface) {
                                $moduleAssets = $this->processModuleAssetWithCondition(clone($asset), $moduleName, $moduleAssets);
                            } else {
                                $content = $this->templatingEngine->render(
                                    $asset->getContentTemplate(),
                                    []
                                );
                                $asset->setContent($content);
                                if (!in_array($asset, $moduleAssets)) {
                                    $moduleAssets[] = $asset;
                                }
                            }
                        }
                    }
                    foreach ($moduleAssets as $moduleAsset) {
                        if (!in_array($moduleAsset, $assets)) {
                            $assets[] = $moduleAsset;
                        }
                    }
                }
            }
        }

        return $assets;
    }
}