<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Event\AssetRemovalEvent;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModuleProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use BuilderiusPro\Bundle\SettingBundle\EventListener\AbstractAssetRemovalCheckingEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateLibrariesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule\DeliverableTemplatePartSubModuleProviderInterface;

class AssetRemovalCheckingInFrontendModeEventListener extends AbstractAssetRemovalCheckingEventListener
{
    /**
     * @var DeliverableTemplateSubModuleProviderInterface
     */
    private $dtsmProvider;

    /**
     * @var DeliverableTemplatePartSubModuleProviderInterface
     */
    private $dtpsmProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigExtension;

    /**
     * @param DeliverableTemplateSubModuleProviderInterface $dtsmProvider
     * @param DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
     * @param TemplateDataVarsExtension $twigExtension
     */
    public function __construct(
        DeliverableTemplateSubModuleProviderInterface $dtsmProvider,
        DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider,
        TemplateDataVarsExtension $twigExtension
    ) {
        $this->dtsmProvider = $dtsmProvider;
        $this->dtpsmProvider = $dtpsmProvider;
        $this->twigExtension = $twigExtension;
    }

    /**
     * @param AssetRemovalEvent $event
     * @throws \Exception
     */
    public function beforeAssetRemoval(AssetRemovalEvent $event)
    {
        $templateSubModule = $this->dtsmProvider->getTemplateSubModule();
        if ($templateSubModule) {
            if ($templatePartsContent = $templateSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                    foreach ($templatePartsContent as $templatePartConfig) {
                        $this->processTemplatePartAssets($templatePartConfig, $event);
                    }
                }
            }
        }
    }

    /**
     * @param array $templatePartConfig
     * @param AssetRemovalEvent $event
     * @throws \Exception
     */
    private function processTemplatePartAssets(array $templatePartConfig, AssetRemovalEvent $event)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePartSubModule = $this->dtpsmProvider->getTemplatePartSubModule($templatePartConfig['type']);
                if ($templatePartSubModule) {
                    if ($templatePartsContent = $templatePartSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $this->processTemplatePartAssets($templatePartConfig, $event);
                            }
                        }
                    }
                    if ($libsContent = $templatePartSubModule->getContent(BuilderiusTemplateLibrariesContentProvider::CONTENT_TYPE)) {
                        $this->processAsset($event, $libsContent);
                    }
                }
            }
        }
    }
}