<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Event\AssetRemovalEvent;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplateProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use BuilderiusPro\Bundle\SettingBundle\EventListener\AbstractAssetRemovalCheckingEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateLibrariesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class AssetRemovalCheckingInPreviewModeEventListener extends AbstractAssetRemovalCheckingEventListener
{
    /**
     * @var BuilderiusTemplateProviderInterface
     */
    private $builderiusTemplateProvider;

    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigExtension;

    /**
     * @param BuilderiusTemplateProviderInterface $builderiusTemplateProvider
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     * @param TemplateDataVarsExtension $twigExtension
     */
    public function __construct(
        BuilderiusTemplateProviderInterface $builderiusTemplateProvider,
        BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider,
        TemplateDataVarsExtension $twigExtension
    ) {
        $this->builderiusTemplateProvider = $builderiusTemplateProvider;
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;
        $this->twigExtension = $twigExtension;
    }

    /**
     * @param AssetRemovalEvent $event
     * @throws \Exception
     */
    public function beforeAssetRemoval(AssetRemovalEvent $event)
    {
        $template = $this->builderiusTemplateProvider->getTemplate();
        if ($template) {
            if ($branch = $template->getActiveBranch()) {
                if ($templatePartsContent = $branch->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                    if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                        foreach ($templatePartsContent as $templatePartConfig) {
                            $this->processTemplatePartAssets($templatePartConfig, $event);
                        }
                    }
                } elseif ($commit = $branch->getActiveCommit()) {
                    if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $this->processTemplatePartAssets($templatePartConfig, $event);
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * @param array $templatePartConfig
     * @param AssetRemovalEvent $event
     */
    private function processTemplatePartAssets(array $templatePartConfig, AssetRemovalEvent $event)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePart = $this->builderiusTemplatePartProvider->getTemplatePart($templatePartConfig['type']);
                if ($templatePart) {
                    if ($branch = $templatePart->getActiveBranch()) {
                        if ($libsContent = $branch->getContent(BuilderiusTemplateLibrariesContentProvider::CONTENT_TYPE)) {
                            $this->processAsset($event, $libsContent);
                        } elseif ($commit = $branch->getActiveCommit()) {
                            if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                                if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                    foreach ($templatePartsContent as $templatePartConfig) {
                                        $this->processTemplatePartAssets($templatePartConfig, $event);
                                    }
                                }
                            }
                            if ($libsContent = $commit->getContent(BuilderiusTemplateLibrariesContentProvider::CONTENT_TYPE)) {
                                $this->processAsset($event, $libsContent);
                            }
                        }
                    }
                }
            }
        }
    }
}