<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use Builderius\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateAtRulesContentProvider;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModuleProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\EventDispatcher\ConditionAwareEventListener;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule\DeliverableTemplatePartSubModuleProviderInterface;
use BuilderiusPro\Bundle\ComponentBundle\Twig\TemplatePartDataVarsExtension;

class AtRulesRegistrationInFrontendModeEventListener extends ConditionAwareEventListener
{
    /**
     * @var DeliverableTemplateSubModuleProviderInterface
     */
    private $dtsmProvider;

    /**
     * @var DeliverableTemplatePartSubModuleProviderInterface
     */
    private $dtpsmProvider;
    
    /**
     * @var TemplateDataVarsExtension
     */
    private $twigTemplateExtension;

    /**
     * @var TemplatePartDataVarsExtension
     */
    private $twigTemplatePartExtension;

    /**
     * @param DeliverableTemplateSubModuleProviderInterface $dtsmProvider
     * @param DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
     * @param TemplateDataVarsExtension $twigTemplateExtension
     * @param TemplatePartDataVarsExtension $twigTemplatePartExtension
     */
    public function __construct(
        DeliverableTemplateSubModuleProviderInterface $dtsmProvider,
        DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider,
        TemplateDataVarsExtension $twigTemplateExtension,
        TemplatePartDataVarsExtension $twigTemplatePartExtension
    ) {
        $this->dtsmProvider = $dtsmProvider;
        $this->dtpsmProvider = $dtpsmProvider;
        $this->twigTemplateExtension = $twigTemplateExtension;
        $this->twigTemplatePartExtension = $twigTemplatePartExtension;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeAtRulesRegistration(ConfigContainingEvent $event)
    {
        $atRules = $event->getConfig();
        $templateSubModule = $this->dtsmProvider->getTemplateSubModule();
        if ($templateSubModule) {
            if ($templatePartsContent = $templateSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                    foreach ($templatePartsContent as $templatePartConfig) {
                        $atRules = $this->processTemplatePartAtRules($templatePartConfig, $atRules);
                    }
                }
            }
        }
        $event->setConfig($atRules);
    }

    /**
     * @param array $templatePartConfig
     * @param array $atRules
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartAtRules(array $templatePartConfig, array $atRules)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigTemplateExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePartSubModule = $this->dtpsmProvider->getTemplatePartSubModule($templatePartConfig['type']);
                if ($templatePartSubModule) {
                    if ($templatePartsContent = $templatePartSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $atRules = $this->processTemplatePartAtRules($templatePartConfig, $atRules);
                            }
                        }
                    }
                    if ($atRulesContent = $templatePartSubModule->getContent(BuilderiusTemplateAtRulesContentProvider::CONTENT_TYPE)) {
                        $atRulesContent = is_array($atRulesContent) ? $atRulesContent : [];
                        foreach($atRulesContent as $atRuleContent) {
                            if (null === $atRuleContent[BuilderiusTemplateAtRulesContentProvider::CONDITION]) {
                                if (!in_array($atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL], $atRules)) {
                                    $atRules[] = $atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL];
                                }
                            } else {
                                $condResult = $this->twigTemplatePartExtension->evaluateVisibilityCondition($templatePartConfig['type'], $atRuleContent[BuilderiusTemplateAtRulesContentProvider::CONDITION]);
                                if (true === $condResult && !in_array($atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL], $atRules)) {
                                    $atRules[] = $atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL];
                                }
                            }
                        }
                    }
                }
            }
        }

        return $atRules;
    }
}