<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use Builderius\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateAtRulesContentProvider;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModulesProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\EventDispatcher\ConditionAwareEventListener;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule\DeliverableTemplatePartSubModuleProviderInterface;
use BuilderiusPro\Bundle\ComponentBundle\Twig\TemplatePartDataVarsExtension;

class AtRulesRegistrationInHookTemplatesFrontendModeEventListener extends ConditionAwareEventListener
{
    /**
     * @var DeliverableTemplateSubModulesProviderInterface
     */
    private $dtsmProvider;

    /**
     * @var DeliverableTemplatePartSubModuleProviderInterface
     */
    private $dtpsmProvider;
    
    /**
     * @var TemplateDataVarsExtension
     */
    private $twigTemplateExtension;

    /**
     * @var TemplatePartDataVarsExtension
     */
    private $twigTemplatePartExtension;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    private $cache;

    /**
     * @param DeliverableTemplateSubModulesProviderInterface $dtsmProvider
     * @param DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
     * @param TemplateDataVarsExtension $twigTemplateExtension
     * @param TemplatePartDataVarsExtension $twigTemplatePartExtension
     * @param BuilderiusRuntimeObjectCache $cache
     */
    public function __construct(
        DeliverableTemplateSubModulesProviderInterface $dtsmProvider,
        DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider,
        TemplateDataVarsExtension $twigTemplateExtension,
        TemplatePartDataVarsExtension $twigTemplatePartExtension,
        BuilderiusRuntimeObjectCache $cache
    ) {
        $this->dtsmProvider = $dtsmProvider;
        $this->dtpsmProvider = $dtpsmProvider;
        $this->twigTemplateExtension = $twigTemplateExtension;
        $this->twigTemplatePartExtension = $twigTemplatePartExtension;
        $this->cache = $cache;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeAtRulesRegistration(ConfigContainingEvent $event)
    {
        $atRules = $event->getConfig();
        $templateSubModules = $this->dtsmProvider->getTemplateSubModules();
        if (!empty($templateSubModules)) {
            $cachedDtsmHookTemplate = $this->cache->get('builderius_dtsm_hook_template');
            foreach ($templateSubModules as $templateSubModule) {
                $this->cache->set('builderius_dtsm_hook_template', $templateSubModule);
                $cachedHookArgs = $this->cache->get(sprintf('dtsm_hook_template_args_%d', $templateSubModule->getId()));
                if (false !== $cachedHookArgs) {
                    $this->cache->set('hook_template_args', $cachedHookArgs);
                }
                if ($templatePartsContent = $templateSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                    if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                        foreach ($templatePartsContent as $templatePartConfig) {
                            $atRules = $this->processTemplatePartAtRules($templatePartConfig, $atRules);
                        }
                    }
                }
                $this->cache->delete('builderius_dtsm_hook_template');
            }
            if (false !== $cachedDtsmHookTemplate) {
                $this->cache->set('builderius_dtsm_hook_template', $cachedDtsmHookTemplate);
            }
        }
        $event->setConfig($atRules);
    }

    /**
     * @param array $templatePartConfig
     * @param array $atRules
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartAtRules(array $templatePartConfig, array $atRules)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigTemplateExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePartSubModule = $this->dtpsmProvider->getTemplatePartSubModule($templatePartConfig['type']);
                if ($templatePartSubModule) {
                    if ($templatePartsContent = $templatePartSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $atRules = $this->processTemplatePartAtRules($templatePartConfig, $atRules);
                            }
                        }
                    }
                    if ($atRulesContent = $templatePartSubModule->getContent(BuilderiusTemplateAtRulesContentProvider::CONTENT_TYPE)) {
                        $atRulesContent = is_array($atRulesContent) ? $atRulesContent : [];
                        foreach($atRulesContent as $atRuleContent) {
                            if (null === $atRuleContent[BuilderiusTemplateAtRulesContentProvider::CONDITION]) {
                                if (!in_array($atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL], $atRules)) {
                                    $atRules[] = $atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL];
                                }
                            } else {
                                $condResult = $this->twigTemplatePartExtension->evaluateVisibilityCondition($templatePartConfig['type'], $atRuleContent[BuilderiusTemplateAtRulesContentProvider::CONDITION]);
                                if (true === $condResult && !in_array($atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL], $atRules)) {
                                    $atRules[] = $atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL];
                                }
                            }
                        }
                    }
                }
            }
        }

        return $atRules;
    }
}