<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use Builderius\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateAtRulesContentProvider;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplatesProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\EventDispatcher\ConditionAwareEventListener;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;
use BuilderiusPro\Bundle\ComponentBundle\Twig\TemplatePartDataVarsExtension;

class AtRulesRegistrationInHookTemplatesPreviewModeEventListener extends ConditionAwareEventListener
{
    /**
     * @var BuilderiusTemplatesProviderInterface
     */
    private $builderiusHookTemplatesProvider;

    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigTemplateExtension;

    /**
     * @var TemplatePartDataVarsExtension
     */
    private $twigTemplatePartExtension;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    private $cache;

    /**
     * @param BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     * @param TemplateDataVarsExtension $twigTemplateExtension
     * @param TemplatePartDataVarsExtension $twigTemplatePartExtension
     * @param BuilderiusRuntimeObjectCache $cache
     */
    public function __construct(
        BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider,
        BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider,
        TemplateDataVarsExtension $twigTemplateExtension,
        TemplatePartDataVarsExtension $twigTemplatePartExtension,
        BuilderiusRuntimeObjectCache $cache
    ) {
        $this->builderiusHookTemplatesProvider = $builderiusHookTemplatesProvider;
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;
        $this->twigTemplateExtension = $twigTemplateExtension;
        $this->twigTemplatePartExtension = $twigTemplatePartExtension;
        $this->cache = $cache;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeAtRulesRegistration(ConfigContainingEvent $event)
    {
        $atRules = $event->getConfig();
        $templates = $this->builderiusHookTemplatesProvider->getTemplates();
        if (!empty($templates)) {
            $cachedHookTemplate = $this->cache->get('builderius_hook_template');
            foreach ($templates as $template) {
                $this->cache->set('builderius_hook_template', $template);
                $cachedHookArgs = $this->cache->get(sprintf('hook_template_args_%d', $template->getId()));
                if (false !== $cachedHookArgs) {
                    $this->cache->set('hook_template_args', $cachedHookArgs);
                }
                if ($branch = $template->getActiveBranch()) {
                    if ($templatePartsContent = $branch->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $atRules = $this->processTemplatePartAtRules($templatePartConfig, $atRules);
                            }
                        }
                    } elseif ($commit = $branch->getActiveCommit()) {
                        if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                            if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                foreach ($templatePartsContent as $templatePartConfig) {
                                    $atRules = $this->processTemplatePartAtRules($templatePartConfig, $atRules);
                                }
                            }
                        }
                    }
                }
                $this->cache->delete('builderius_hook_template');
            }
            if (false !== $cachedHookTemplate) {
                $this->cache->set('builderius_hook_template', $cachedHookTemplate);
                $cachedHookArgs = $this->cache->get(sprintf('hook_template_args_%d', $cachedHookTemplate->getId()));
                if (false !== $cachedHookArgs) {
                    $this->cache->set('hook_template_args', $cachedHookArgs);
                }
            }
        }
        $event->setConfig($atRules);
    }

    /**
     * @param array $templatePartConfig
     * @param array $atRules
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartAtRules(array $templatePartConfig, array $atRules)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigTemplateExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePart = $this->builderiusTemplatePartProvider->getTemplatePart($templatePartConfig['type']);
                if ($templatePart) {
                    if ($branch = $templatePart->getActiveBranch()) {
                        if ($atRulesContent = $branch->getContent(BuilderiusTemplateAtRulesContentProvider::CONTENT_TYPE)) {
                            $atRulesContent = is_array($atRulesContent) ? $atRulesContent : [];
                            foreach($atRulesContent as $atRuleContent) {
                                if (null === $atRuleContent[BuilderiusTemplateAtRulesContentProvider::CONDITION]) {
                                    if (!in_array($atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL], $atRules)) {
                                        $atRules[] = $atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL];
                                    }
                                } else {
                                    $condResult = $this->twigTemplatePartExtension->evaluateVisibilityCondition($templatePartConfig['type'], $atRuleContent[BuilderiusTemplateAtRulesContentProvider::CONDITION]);
                                    if (true === $condResult && !in_array($atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL], $atRules)) {
                                        $atRules[] = $atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL];
                                    }
                                }
                            }
                        } elseif ($commit = $branch->getActiveCommit()) {
                            if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                                if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                    foreach ($templatePartsContent as $templatePartConfig) {
                                        $atRules = $this->processTemplatePartAtRules($templatePartConfig, $atRules);
                                    }
                                }
                            }
                            if ($atRulesContent = $commit->getContent(BuilderiusTemplateAtRulesContentProvider::CONTENT_TYPE)) {
                                $atRulesContent = is_array($atRulesContent) ? $atRulesContent : [];
                                foreach($atRulesContent as $atRuleContent) {
                                    if (null === $atRuleContent[BuilderiusTemplateAtRulesContentProvider::CONDITION]) {
                                        if (!in_array($atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL], $atRules)) {
                                            $atRules[] = $atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL];
                                        }
                                    } else {
                                        $condResult = $this->twigTemplatePartExtension->evaluateVisibilityCondition($templatePartConfig['type'], $atRuleContent[BuilderiusTemplateAtRulesContentProvider::CONDITION]);
                                        if (true === $condResult && !in_array($atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL], $atRules)) {
                                            $atRules[] = $atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL];
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        return $atRules;
    }
}