<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use Builderius\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateAtRulesContentProvider;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplateProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\EventDispatcher\ConditionAwareEventListener;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;
use BuilderiusPro\Bundle\ComponentBundle\Twig\TemplatePartDataVarsExtension;

class AtRulesRegistrationInPreviewModeEventListener extends ConditionAwareEventListener
{
    /**
     * @var BuilderiusTemplateProviderInterface
     */
    private $builderiusTemplateProvider;

    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigTemplateExtension;

    /**
     * @var TemplatePartDataVarsExtension
     */
    private $twigTemplatePartExtension;

    /**
     * @param BuilderiusTemplateProviderInterface $builderiusTemplateProvider
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     * @param TemplateDataVarsExtension $twigTemplateExtension
     * @param TemplatePartDataVarsExtension $twigTemplatePartExtension
     */
    public function __construct(
        BuilderiusTemplateProviderInterface $builderiusTemplateProvider,
        BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider,
        TemplateDataVarsExtension $twigTemplateExtension,
        TemplatePartDataVarsExtension $twigTemplatePartExtension
    ) {
        $this->builderiusTemplateProvider = $builderiusTemplateProvider;
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;
        $this->twigTemplateExtension = $twigTemplateExtension;
        $this->twigTemplatePartExtension = $twigTemplatePartExtension;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeAtRulesRegistration(ConfigContainingEvent $event)
    {
        $atRules = $event->getConfig();
        $template = $this->builderiusTemplateProvider->getTemplate();
        if ($template) {
            if ($branch = $template->getActiveBranch()) {
                if ($templatePartsContent = $branch->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                    if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                        foreach ($templatePartsContent as $templatePartConfig) {
                            $atRules = $this->processTemplatePartAtRules($templatePartConfig, $atRules);
                        }
                    }
                } elseif ($commit = $branch->getActiveCommit()) {
                    if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $atRules = $this->processTemplatePartAtRules($templatePartConfig, $atRules);
                            }
                        }
                    }
                }
            }
        }
        $event->setConfig($atRules);
    }

    /**
     * @param array $templatePartConfig
     * @param array $atRules
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartAtRules(array $templatePartConfig, array $atRules)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigTemplateExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePart = $this->builderiusTemplatePartProvider->getTemplatePart($templatePartConfig['type']);
                if ($templatePart) {
                    if ($branch = $templatePart->getActiveBranch()) {
                        if ($atRulesContent = $branch->getContent(BuilderiusTemplateAtRulesContentProvider::CONTENT_TYPE)) {
                            $atRulesContent = is_array($atRulesContent) ? $atRulesContent : [];
                            foreach($atRulesContent as $atRuleContent) {
                                if (null === $atRuleContent[BuilderiusTemplateAtRulesContentProvider::CONDITION]) {
                                    if (!in_array($atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL], $atRules)) {
                                        $atRules[] = $atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL];
                                    }
                                } else {
                                    $condResult = $this->twigTemplatePartExtension->evaluateVisibilityCondition($templatePartConfig['type'], $atRuleContent[BuilderiusTemplateAtRulesContentProvider::CONDITION]);
                                    if (true === $condResult && !in_array($atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL], $atRules)) {
                                        $atRules[] = $atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL];
                                    }
                                }
                            }
                        } elseif ($commit = $branch->getActiveCommit()) {
                            if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                                if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                    foreach ($templatePartsContent as $templatePartConfig) {
                                        $atRules = $this->processTemplatePartAtRules($templatePartConfig, $atRules);
                                    }
                                }
                            }
                            if ($atRulesContent = $commit->getContent(BuilderiusTemplateAtRulesContentProvider::CONTENT_TYPE)) {
                                $atRulesContent = is_array($atRulesContent) ? $atRulesContent : [];
                                foreach($atRulesContent as $atRuleContent) {
                                    if (null === $atRuleContent[BuilderiusTemplateAtRulesContentProvider::CONDITION]) {
                                        if (!in_array($atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL], $atRules)) {
                                            $atRules[] = $atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL];
                                        }
                                    } else {
                                        $condResult = $this->twigTemplatePartExtension->evaluateVisibilityCondition($templatePartConfig['type'], $atRuleContent[BuilderiusTemplateAtRulesContentProvider::CONDITION]);
                                        if (true === $condResult && !in_array($atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL], $atRules)) {
                                            $atRules[] = $atRuleContent[BuilderiusTemplateAtRulesContentProvider::URL];
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        return $atRules;
    }
}