<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\DeliverableBundle\Event\DeliverableContainingEvent;
use Builderius\Bundle\GraphQLBundle\Executor\BuilderiusEntitiesGraphQLQueriesExecutorInterface;
use Builderius\Bundle\TemplateBundle\Converter\Version\BuilderiusTemplateConfigVersionConverterInterface;
use Builderius\Bundle\TemplateBundle\Model\BuilderiusTemplate;

class BuilderiusTemplatePartCreationOnDeliverableExtractListener
{
    /**
     * @var BuilderiusEntitiesGraphQLQueriesExecutorInterface
     */
    private $graphQLQueriesExecutor;

    /**
     * @var  BuilderiusTemplateConfigVersionConverterInterface
     */
    private $configVersionConverter;

    /**
     * @param BuilderiusEntitiesGraphQLQueriesExecutorInterface $graphQLQueriesExecutor
     * @param BuilderiusTemplateConfigVersionConverterInterface $configVersionConverter
     */
    public function __construct(
        BuilderiusEntitiesGraphQLQueriesExecutorInterface $graphQLQueriesExecutor,
        BuilderiusTemplateConfigVersionConverterInterface $configVersionConverter
    ) {
        $this->graphQLQueriesExecutor = $graphQLQueriesExecutor;
        $this->configVersionConverter = $configVersionConverter;
    }

    /**
     * @param DeliverableContainingEvent $event
     */
    public function onDeliverableExtraction(DeliverableContainingEvent $event)
    {
        $deliverable = $event->getDeliverable();
        foreach ($deliverable->getSubModules('template_part') as $dsm) {
            $attributes = $dsm->getAttributes();
            $this->createTemplatePost(
                $dsm->getName(),
                $dsm->getType(),
                $dsm->getTechnology(),
                $dsm->getContentConfig(),
                $attributes[BuilderiusTemplate::SORT_ORDER_FIELD],
                $attributes[BuilderiusTemplate::APPLY_RULES_CONFIG_FIELD],
            );
        }
    }

    /**
     * @param string $name
     * @param string $type
     * @param string $technology
     * @param array $config
     * @param int $sortOrder
     * @param array $applyRulesConfig
     * @throws \Exception
     */
    private function createTemplatePost($name, $type, $technology, $config, $sortOrder, $applyRulesConfig)
    {
        $name = str_replace('Template: ', '', $name);
        $name = explode('(tag', $name)[0];
        $serializedApplyRulesConfig = json_encode($applyRulesConfig, JSON_UNESCAPED_UNICODE|JSON_INVALID_UTF8_IGNORE);
        $serializedApplyRulesConfig = str_replace('\\', '\\\\', $serializedApplyRulesConfig);
        $serializedApplyRulesConfig = str_replace('"', '\\"', $serializedApplyRulesConfig);
        $mutations = [];
        $mutations[] = [
            'name' => 'createTemplate',
            'query' => 'mutation{
                        createTemplate(input: {
                            title: "' . $name . '",
                            type: "template_part",
                            sub_type: "' . $type . '",
                            technology: "' . $technology . '",
                            sort_order: ' . $sortOrder . ',
                            serialized_apply_rules_config: "' . $serializedApplyRulesConfig . '"
                        }){
                            template {
                                id
                                branches {
                                  id
                                }
                            }
                        }
                    }'
        ];
        $result = $this->graphQLQueriesExecutor->execute($mutations);
        $branchId = $result['createTemplate']['data']['createTemplate']['template']['branches'][0]['id'];

        $config = $this->configVersionConverter->convert($config);
        $serializedConfig = json_encode($config, JSON_UNESCAPED_UNICODE|JSON_INVALID_UTF8_IGNORE);
        $serializedConfig = str_replace('\\', '\\\\', $serializedConfig);
        $serializedConfig = str_replace('"', '\\"', $serializedConfig);

        $mutations[] =
            [
                'name' => 'createCommit',
                'query' => 'mutation {
                                            createCommit(input: {
                                                branch_id: ' . $branchId . ', 
                                                serialized_content_config: "' . $serializedConfig . '", 
                                                description: "Revision"
                                                }) {
                                                    commit {
                                                        id
                                                    }
                                                }    
                                            }'
            ];
        $this->graphQLQueriesExecutor->execute($mutations);
    }
}