<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateCssContentProvider;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModuleProviderInterface;
use Builderius\Bundle\TemplateBundle\Sanitizer\CssContentSanitizer;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\InlineAssetsContainingEvent;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\AssetDataItem;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\InlineAsset;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\EventDispatcher\ConditionAwareEventListener;
use Builderius\Symfony\Component\Templating\EngineInterface;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule\DeliverableTemplatePartSubModuleProviderInterface;

class CssRegistrationInFrontendModeEventListener extends ConditionAwareEventListener
{
    /**
     * @var DeliverableTemplateSubModuleProviderInterface
     */
    private $dtsmProvider;

    /**
     * @var DeliverableTemplatePartSubModuleProviderInterface
     */
    private $dtpsmProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigExtension;

    /**
     * @var EngineInterface
     */
    private $templatingEngine;

    /**
     * @param DeliverableTemplateSubModuleProviderInterface $dtsmProvider
     * @param DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
     * @param TemplateDataVarsExtension $twigExtension
     * @param EngineInterface $templatingEngine
     */
    public function __construct(
        DeliverableTemplateSubModuleProviderInterface $dtsmProvider,
        DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider,
        TemplateDataVarsExtension $twigExtension,
        EngineInterface $templatingEngine
    ) {
        $this->dtsmProvider = $dtsmProvider;
        $this->dtpsmProvider = $dtpsmProvider;
        $this->twigExtension = $twigExtension;
        $this->templatingEngine = $templatingEngine;
    }

    /**
     * @param InlineAssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(InlineAssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $templateSubModule = $this->dtsmProvider->getTemplateSubModule();
        if ($templateSubModule) {
            if ($templatePartsContent = $templateSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                    foreach ($templatePartsContent as $templatePartConfig) {
                        $assets = $this->processTemplatePartAssets($templatePartConfig, $assets);
                    }
                }
            }
        }
        $event->setAssets($assets);
    }

    /**
     * @param array $templatePartConfig
     * @param array $assets
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartAssets(array $templatePartConfig, array $assets)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePartSubModule = $this->dtpsmProvider->getTemplatePartSubModule($templatePartConfig['type']);
                if ($templatePartSubModule) {
                    if ($templatePartsContent = $templatePartSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $assets = $this->processTemplatePartAssets($templatePartConfig, $assets);
                            }
                        }
                    }
                    if ($cssContent = $templatePartSubModule->getContent(BuilderiusTemplateCssContentProvider::CONTENT_TYPE)) {
                        $cssContent = str_replace('builderius_data_var(', sprintf('builderius_tp_data_var(\'%s\', ', $templatePartConfig['type']), $cssContent);
                        $cssContent = str_replace('builderius_data_var_escaped(', sprintf('builderius_tp_data_var_escaped(\'%s\', ', $templatePartConfig['type']), $cssContent);
                        $cssContent = str_replace('builderius_visibility_condition(', sprintf('builderius_tp_visibility_condition(\'%s\', ', $templatePartConfig['type']), $cssContent);
                        $cssContent = $this->templatingEngine->render(
                            'BuilderiusTemplateBundle:templateDynamicContent.twig',
                            [
                                'dynamicContent' => $cssContent
                            ]
                        );
                        $cssContent = CssContentSanitizer::sanitize($cssContent);
                        if (trim($cssContent) !== '') {
                            $asset = new InlineAsset([
                                InlineAsset::TYPE_FIELD => 'style',
                                InlineAsset::CONTENT_FIELD => $cssContent
                            ]);
                            $assetDataItem = new AssetDataItem([
                                AssetDataItem::GROUP_FIELD => 'htmlAttributes',
                                AssetDataItem::KEY_FIELD => 'class',
                                AssetDataItem::VALUE_FIELD => 'builderius-css'
                            ]);
                            $asset->addAssetDataItem($assetDataItem);
                            if (!in_array($asset, $assets)) {
                                $assets[] = $asset;
                            }
                        }
                    }
                }
            }
        }

        return $assets;
    }
}