<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateCssContentProvider;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModulesProviderInterface;
use Builderius\Bundle\TemplateBundle\Sanitizer\CssContentSanitizer;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\InlineAssetsContainingEvent;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\AssetDataItem;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\InlineAsset;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\EventDispatcher\ConditionAwareEventListener;
use Builderius\Symfony\Component\Templating\EngineInterface;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule\DeliverableTemplatePartSubModuleProviderInterface;

class CssRegistrationInHookTemplatesFrontendModeEventListener extends ConditionAwareEventListener
{
    /**
     * @var DeliverableTemplateSubModulesProviderInterface
     */
    private $dtsmProvider;

    /**
     * @var DeliverableTemplatePartSubModuleProviderInterface
     */
    private $dtpsmProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigExtension;

    /**
     * @var EngineInterface
     */
    private $templatingEngine;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    private $cache;

    /**
     * @param DeliverableTemplateSubModulesProviderInterface $dtsmProvider
     * @param DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
     * @param TemplateDataVarsExtension $twigExtension
     * @param EngineInterface $templatingEngine
     * @param BuilderiusRuntimeObjectCache $cache
     */
    public function __construct(
        DeliverableTemplateSubModulesProviderInterface $dtsmProvider,
        DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider,
        TemplateDataVarsExtension $twigExtension,
        EngineInterface $templatingEngine,
        BuilderiusRuntimeObjectCache $cache
    ) {
        $this->dtsmProvider = $dtsmProvider;
        $this->dtpsmProvider = $dtpsmProvider;
        $this->twigExtension = $twigExtension;
        $this->templatingEngine = $templatingEngine;
        $this->cache = $cache;
    }

    /**
     * @param InlineAssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(InlineAssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $templateSubModules = $this->dtsmProvider->getTemplateSubModules();
        if (!empty($templateSubModules)) {
            $cachedDtsmHookTemplate = $this->cache->get('builderius_dtsm_hook_template');
            foreach ($templateSubModules as $templateSubModule) {
                $this->cache->set('builderius_dtsm_hook_template', $templateSubModule);
                $cachedHookArgs = $this->cache->get(sprintf('dtsm_hook_template_args_%d', $templateSubModule->getId()));
                if (false !== $cachedHookArgs) {
                    $this->cache->set('hook_template_args', $cachedHookArgs);
                }
                if ($templatePartsContent = $templateSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                    if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                        foreach ($templatePartsContent as $templatePartConfig) {
                            $assets = $this->processTemplatePartAssets($templatePartConfig, $assets);
                        }
                    }
                }
                $this->cache->delete('builderius_dtsm_hook_template');
            }
            if (false !== $cachedDtsmHookTemplate) {
                $this->cache->set('builderius_dtsm_hook_template', $cachedDtsmHookTemplate);
            }
        }
        $event->setAssets($assets);
    }

    /**
     * @param array $templatePartConfig
     * @param array $assets
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartAssets(array $templatePartConfig, array $assets)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePartSubModule = $this->dtpsmProvider->getTemplatePartSubModule($templatePartConfig['type']);
                if ($templatePartSubModule) {
                    if ($templatePartsContent = $templatePartSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $assets = $this->processTemplatePartAssets($templatePartConfig, $assets);
                            }
                        }
                    }
                    if ($cssContent = $templatePartSubModule->getContent(BuilderiusTemplateCssContentProvider::CONTENT_TYPE)) {
                        $cssContent = str_replace('builderius_data_var(', sprintf('builderius_tp_data_var(\'%s\', ', $templatePartConfig['type']), $cssContent);
                        $cssContent = str_replace('builderius_data_var_escaped(', sprintf('builderius_tp_data_var_escaped(\'%s\', ', $templatePartConfig['type']), $cssContent);
                        $cssContent = str_replace('builderius_visibility_condition(', sprintf('builderius_tp_visibility_condition(\'%s\', ', $templatePartConfig['type']), $cssContent);
                        $cssContent = $this->templatingEngine->render(
                            'BuilderiusTemplateBundle:templateDynamicContent.twig',
                            [
                                'dynamicContent' => $cssContent
                            ]
                        );
                        $cssContent = CssContentSanitizer::sanitize($cssContent);
                        if (trim($cssContent) !== '') {
                            $asset = new InlineAsset([
                                InlineAsset::TYPE_FIELD => 'style',
                                InlineAsset::CONTENT_FIELD => $cssContent
                            ]);
                            $assetDataItem = new AssetDataItem([
                                AssetDataItem::GROUP_FIELD => 'htmlAttributes',
                                AssetDataItem::KEY_FIELD => 'class',
                                AssetDataItem::VALUE_FIELD => 'builderius-css'
                            ]);
                            $asset->addAssetDataItem($assetDataItem);
                            if (!in_array($asset, $assets)) {
                                $assets[] = $asset;
                            }
                        }
                    }
                }
            }
        }

        return $assets;
    }
}