<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplatesProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\InlineAssetsContainingEvent;
use BuilderiusPro\Bundle\TemplateBundle\EventListener\AbstractAssetsRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateCustomCssContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class CustomCssRegistrationInHookTemplatesPreviewModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusTemplatesProviderInterface
     */
    private $builderiusHookTemplatesProvider;

    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigExtension;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    private $cache;

    /**
     * @param BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     * @param TemplateDataVarsExtension $twigExtension
     * @param BuilderiusRuntimeObjectCache $cache
     */
    public function __construct(
        BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider,
        BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider,
        TemplateDataVarsExtension $twigExtension,
        BuilderiusRuntimeObjectCache $cache
    ) {
        $this->builderiusHookTemplatesProvider = $builderiusHookTemplatesProvider;
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;
        $this->twigExtension = $twigExtension;
        $this->cache = $cache;
    }

    /**
     * @param InlineAssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(InlineAssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $templates = $this->builderiusHookTemplatesProvider->getTemplates();
        if (!empty($templates)) {
            $cachedHookTemplate = $this->cache->get('builderius_hook_template');
            foreach ($templates as $template) {
                $this->cache->set('builderius_hook_template', $template);
                $cachedHookArgs = $this->cache->get(sprintf('hook_template_args_%d', $template->getId()));
                if (false !== $cachedHookArgs) {
                    $this->cache->set('hook_template_args', $cachedHookArgs);
                }
                if ($branch = $template->getActiveBranch()) {
                    if ($templatePartsContent = $branch->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $assets = $this->processTemplatePartAssets($templatePartConfig, $assets);
                            }
                        }
                    } elseif ($commit = $branch->getActiveCommit()) {
                        if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                            if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                foreach ($templatePartsContent as $templatePartConfig) {
                                    $assets = $this->processTemplatePartAssets($templatePartConfig, $assets);
                                }
                            }
                        }
                    }
                }
                $this->cache->delete('builderius_hook_template');
            }
            if (false !== $cachedHookTemplate) {
                $this->cache->set('builderius_hook_template', $cachedHookTemplate);
                $cachedHookArgs = $this->cache->get(sprintf('hook_template_args_%d', $cachedHookTemplate->getId()));
                if (false !== $cachedHookArgs) {
                    $this->cache->set('hook_template_args', $cachedHookArgs);
                }
            }
        }
        $event->setAssets($assets);
    }

    /**
     * @param array $templatePartConfig
     * @param array $assets
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartAssets(array $templatePartConfig, array $assets)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePart = $this->builderiusTemplatePartProvider->getTemplatePart($templatePartConfig['type']);
                if ($templatePart) {
                    if ($branch = $templatePart->getActiveBranch()) {
                        if ($cssContent = $branch->getContent(BuilderiusTemplateCustomCssContentProvider::CONTENT_TYPE)) {
                            if (is_array($cssContent) && !empty($cssContent)) {
                                foreach ($cssContent as $cssContentItem) {
                                    $asset = $this->createAsset((array)$cssContentItem, 'style', 'builderius-css');
                                    if (!in_array($asset, $assets)) {
                                        $assets[] = $asset;
                                    }
                                }
                            }
                        } elseif ($commit = $branch->getActiveCommit()) {
                            if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                                if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                    foreach ($templatePartsContent as $templatePartConfig) {
                                        $assets = $this->processTemplatePartAssets($templatePartConfig, $assets);
                                    }
                                }
                            }
                            if ($cssContent = $commit->getContent(BuilderiusTemplateCustomCssContentProvider::CONTENT_TYPE)) {
                                if (is_array($cssContent) && !empty($cssContent)) {
                                    foreach ($cssContent as $cssContentItem) {
                                        $asset = $this->createAsset((array)$cssContentItem, 'style', 'builderius-css');
                                        if (!in_array($asset, $assets)) {
                                            $assets[] = $asset;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        return $assets;
    }
}