<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplateProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\InlineAssetsContainingEvent;
use BuilderiusPro\Bundle\TemplateBundle\EventListener\AbstractAssetsRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateJsContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class JsRegistrationInPreviewModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusTemplateProviderInterface
     */
    private $builderiusTemplateProvider;

    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigExtension;

    /**
     * @param BuilderiusTemplateProviderInterface $builderiusTemplateProvider
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     * @param TemplateDataVarsExtension $twigExtension
     */
    public function __construct(
        BuilderiusTemplateProviderInterface $builderiusTemplateProvider,
        BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider,
        TemplateDataVarsExtension $twigExtension
    ) {
        $this->builderiusTemplateProvider = $builderiusTemplateProvider;
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;
        $this->twigExtension = $twigExtension;
    }

    /**
     * @param InlineAssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(InlineAssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $template = $this->builderiusTemplateProvider->getTemplate();
        if ($template) {
            if ($branch = $template->getActiveBranch()) {
                if ($templatePartsContent = $branch->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                    if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                        foreach ($templatePartsContent as $templatePartConfig) {
                            $assets = $this->processTemplatePartAssets($templatePartConfig, $assets);
                        }
                    }
                } elseif ($commit = $branch->getActiveCommit()) {
                    if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $assets = $this->processTemplatePartAssets($templatePartConfig, $assets);
                            }
                        }
                    }
                }
            }
        }
        $event->setAssets($assets);
    }

    /**
     * @param array $templatePartConfig
     * @param array $assets
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartAssets(array $templatePartConfig, array $assets)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePart = $this->builderiusTemplatePartProvider->getTemplatePart($templatePartConfig['type']);
                if ($templatePart) {
                    if ($branch = $templatePart->getActiveBranch()) {
                        if ($jsContent = $branch->getContent(BuilderiusTemplateJsContentProvider::CONTENT_TYPE)) {
                            if (is_array($jsContent) && !empty($jsContent)) {
                                foreach ($jsContent as $jsContentItem) {
                                    $asset = $this->createAsset((array)$jsContentItem, 'script', 'builderius-js');
                                    if (!in_array($asset, $assets)) {
                                        $assets[] = $asset;
                                    }
                                }
                            }
                        } elseif ($commit = $branch->getActiveCommit()) {
                            if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                                if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                    foreach ($templatePartsContent as $templatePartConfig) {
                                        $assets = $this->processTemplatePartAssets($templatePartConfig, $assets);
                                    }
                                }
                            }
                            if ($jsContent = $commit->getContent(BuilderiusTemplateJsContentProvider::CONTENT_TYPE)) {
                                if (is_array($jsContent) && !empty($jsContent)) {
                                    foreach ($jsContent as $jsContentItem) {
                                        $asset = $this->createAsset((array)$jsContentItem, 'script', 'builderius-js');
                                        if (!in_array($asset, $assets)) {
                                            $assets[] = $asset;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        return $assets;
    }
}