<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModuleProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\AssetsContainingEvent;
use BuilderiusPro\Bundle\SettingBundle\EventListener\AbstractLibrariesRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateLibrariesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule\DeliverableTemplatePartSubModuleProviderInterface;

class LibrariesRegistrationInFrontendModeEventListener extends AbstractLibrariesRegistrationEventListener
{
    /**
     * @var DeliverableTemplateSubModuleProviderInterface
     */
    private $dtsmProvider;

    /**
     * @var DeliverableTemplatePartSubModuleProviderInterface
     */
    private $dtpsmProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigExtension;

    /**
     * @param DeliverableTemplateSubModuleProviderInterface $dtsmProvider
     * @param DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
     * @param TemplateDataVarsExtension $twigExtension
     */
    public function __construct(
        DeliverableTemplateSubModuleProviderInterface $dtsmProvider,
        DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider,
        TemplateDataVarsExtension $twigExtension
    ) {
        $this->dtsmProvider = $dtsmProvider;
        $this->dtpsmProvider = $dtpsmProvider;
        $this->twigExtension = $twigExtension;
    }

    /**
     * @param AssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(AssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $templateSubModule = $this->dtsmProvider->getTemplateSubModule();
        if ($templateSubModule) {
            if ($templatePartsContent = $templateSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                    foreach ($templatePartsContent as $templatePartConfig) {
                        $assets = $this->processTemplatePartAssets($templatePartConfig, $assets);
                    }
                }
            }
        }
        $event->setAssets($assets);
    }

    /**
     * @param array $templatePartConfig
     * @param array $assets
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartAssets(array $templatePartConfig, array $assets)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePartSubModule = $this->dtpsmProvider->getTemplatePartSubModule($templatePartConfig['type']);
                if ($templatePartSubModule) {
                    if ($templatePartsContent = $templatePartSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $assets = $this->processTemplatePartAssets($templatePartConfig, $assets);
                            }
                        }
                    }
                    if ($libsContent = $templatePartSubModule->getContent(BuilderiusTemplateLibrariesContentProvider::CONTENT_TYPE)) {
                        $assets = $this->processAssets($assets, $libsContent);
                    }
                }
            }
        }

        return $assets;
    }
}