<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\ExpressionLanguageBundle\ExpressionLanguage;
use Builderius\Bundle\ModuleBundle\Provider\BuilderiusModulesProviderInterface;
use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateModulesWithAssetsContentProvider;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModulesProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\AssetsContainingEvent;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule\DeliverableTemplatePartSubModuleProviderInterface;
use BuilderiusPro\Bundle\ComponentBundle\Twig\TemplatePartDataVarsExtension;

class ModulesAssetsRegistrationInHookTemplatesFrontendModeEventListener extends AbstractTemplatePartModulesAssetsRegistrationEventListener
{
    /**
     * @var DeliverableTemplateSubModulesProviderInterface
     */
    private $dtsmProvider;

    /**
     * @var DeliverableTemplatePartSubModuleProviderInterface
     */
    private $dtpsmProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigTemplateExtension;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    private $cache;

    /**
     * @param DeliverableTemplateSubModulesProviderInterface $dtsmProvider
     * @param DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
     * @param BuilderiusModulesProviderInterface $modulesProvider
     * @param TemplateDataVarsExtension $twigTemplateExtension
     * @param TemplatePartDataVarsExtension $twigTemplatePartExtension
     * @param BuilderiusRuntimeObjectCache $cache
     */
    public function __construct(
        DeliverableTemplateSubModulesProviderInterface $dtsmProvider,
        DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider,
        BuilderiusModulesProviderInterface $modulesProvider,
        TemplateDataVarsExtension $twigTemplateExtension,
        TemplatePartDataVarsExtension $twigTemplatePartExtension,
        ExpressionLanguage $expressionLanguage,
        BuilderiusRuntimeObjectCache $cache
    ) {
        $this->dtsmProvider = $dtsmProvider;
        $this->dtpsmProvider = $dtpsmProvider;
        $this->modulesProvider = $modulesProvider;
        $this->twigTemplateExtension = $twigTemplateExtension;
        $this->twigTemplatePartExtension = $twigTemplatePartExtension;
        $this->expressionLanguage = $expressionLanguage;
        $this->cache = $cache;
    }

    /**
     * @param AssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(AssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $templateSubModules = $this->dtsmProvider->getTemplateSubModules();
        if (!empty($templateSubModules)) {
            $cachedDtsmHookTemplate = $this->cache->get('builderius_dtsm_hook_template');
            foreach ($templateSubModules as $templateSubModule) {
                $this->cache->set('builderius_dtsm_hook_template', $templateSubModule);
                $cachedHookArgs = $this->cache->get(sprintf('dtsm_hook_template_args_%d', $templateSubModule->getId()));
                if (false !== $cachedHookArgs) {
                    $this->cache->set('hook_template_args', $cachedHookArgs);
                }
                if ($templatePartsContent = $templateSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                    if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                        foreach ($templatePartsContent as $templatePartConfig) {
                            $assets = $this->processTemplatePartModulesAssets($templatePartConfig, $assets);
                        }
                    }
                }
                $this->cache->delete('builderius_dtsm_hook_template');
            }
            if (false !== $cachedDtsmHookTemplate) {
                $this->cache->set('builderius_dtsm_hook_template', $cachedDtsmHookTemplate);
            }
        }
        $event->setAssets($this->processSsr($assets));
    }

    /**
     * @param array $templatePartConfig
     * @param array $assets
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartModulesAssets(array $templatePartConfig, array $assets)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigTemplateExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePartSubModule = $this->dtpsmProvider->getTemplatePartSubModule($templatePartConfig['type']);
                if ($templatePartSubModule) {
                    $typeName = $templatePartSubModule->getType();
                    $technologyName = $templatePartSubModule->getTechnology();
                    if ($templatePartsContent = $templatePartSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $assets = $this->processTemplatePartModulesAssets($templatePartConfig, $assets);
                            }
                        }
                    }
                    if ($modulesAssetsContent = $templatePartSubModule->getContent(BuilderiusTemplateModulesWithAssetsContentProvider::CONTENT_TYPE)) {
                        if (is_array($modulesAssetsContent) && !empty($modulesAssetsContent)) {
                            $config = $templatePartSubModule->getContentConfig();
                            $this->setConfigVersion($config['version']);
                            foreach ($modulesAssetsContent as $moduleName) {
                                $assets = $this->processModuleAssets($moduleName, $typeName, $technologyName, $assets);
                            }
                        }
                    }
                }
            }
        }

        return $assets;
    }
}