<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\ExpressionLanguageBundle\ExpressionLanguage;
use Builderius\Bundle\ModuleBundle\Provider\BuilderiusModulesProviderInterface;
use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateModulesWithInlineAssetsContentProvider;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplatesProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\InlineAssetsContainingEvent;
use Builderius\Symfony\Component\Templating\EngineInterface;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;
use BuilderiusPro\Bundle\ComponentBundle\Twig\TemplatePartDataVarsExtension;

class ModulesInlineAssetsRegistrationInHookTemplatesPreviewModeEventListener extends AbstractTemplatePartModulesInlineAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusTemplatesProviderInterface
     */
    private $builderiusHookTemplatesProvider;

    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;
    
    /**
     * @var TemplateDataVarsExtension
     */
    private $twigTemplateExtension;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    private $cache;

    /**
     * @var string
     */
    private $assetType;

    /**
     * @param BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     * @param BuilderiusModulesProviderInterface $modulesProvider
     * @param TemplateDataVarsExtension $twigTemplateExtension
     * @param TemplatePartDataVarsExtension $twigTemplatePartExtension
     * @param ExpressionLanguage $expressionLanguage
     * @param BuilderiusRuntimeObjectCache $cache
     * @param EngineInterface $templatingEngine
     * @param string $assetType
     */
    public function __construct(
        BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider,
        BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider,
        BuilderiusModulesProviderInterface $modulesProvider,
        TemplateDataVarsExtension $twigTemplateExtension,
        TemplatePartDataVarsExtension $twigTemplatePartExtension,
        ExpressionLanguage $expressionLanguage,
        BuilderiusRuntimeObjectCache $cache,
        EngineInterface $templatingEngine,
        $assetType
    ) {
        $this->builderiusHookTemplatesProvider = $builderiusHookTemplatesProvider;
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;
        $this->modulesProvider = $modulesProvider;
        $this->twigTemplateExtension = $twigTemplateExtension;
        $this->twigTemplatePartExtension = $twigTemplatePartExtension;
        $this->expressionLanguage = $expressionLanguage;
        $this->cache = $cache;
        $this->templatingEngine = $templatingEngine;
        $this->assetType = $assetType;
    }

    /**
     * @param InlineAssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(InlineAssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $templates = $this->builderiusHookTemplatesProvider->getTemplates();
        if (!empty($templates)) {
            $cachedHookTemplate = $this->cache->get('builderius_hook_template');
            foreach ($templates as $template) {
                $this->cache->set('builderius_hook_template', $template);
                $cachedHookArgs = $this->cache->get(sprintf('hook_template_args_%d', $template->getId()));
                if (false !== $cachedHookArgs) {
                    $this->cache->set('hook_template_args', $cachedHookArgs);
                }
                if ($branch = $template->getActiveBranch()) {
                    if ($templatePartsContent = $branch->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $assets = $this->processTemplatePartModulesAssets($templatePartConfig, $assets);
                            }
                        }
                    } elseif ($commit = $branch->getActiveCommit()) {
                        if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                            if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                foreach ($templatePartsContent as $templatePartConfig) {
                                    $assets = $this->processTemplatePartModulesAssets($templatePartConfig, $assets);
                                }
                            }
                        }
                    }
                }
                $this->cache->delete('builderius_hook_template');
            }
            if (false !== $cachedHookTemplate) {
                $this->cache->set('builderius_hook_template', $cachedHookTemplate);
                $cachedHookArgs = $this->cache->get(sprintf('hook_template_args_%d', $cachedHookTemplate->getId()));
                if (false !== $cachedHookArgs) {
                    $this->cache->set('hook_template_args', $cachedHookArgs);
                }
            }
        }
        $event->setAssets($assets);
    }

    /**
     * @param array $templatePartConfig
     * @param array $assets
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartModulesAssets(array $templatePartConfig, array $assets)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigTemplateExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePart = $this->builderiusTemplatePartProvider->getTemplatePart($templatePartConfig['type']);
                if ($templatePart) {
                    $typeName = $templatePart->getSubType();
                    $technologyName = $templatePart->getTechnology();
                    if ($branch = $templatePart->getActiveBranch()) {
                        if ($modulesAssetsContent = $branch->getContent(BuilderiusTemplateModulesWithInlineAssetsContentProvider::CONTENT_TYPE)) {
                            if (is_array($modulesAssetsContent) && !empty($modulesAssetsContent)) {
                                $config = $branch->getNotCommittedConfig();
                                $this->setConfigVersion($config['version']);
                                foreach ($modulesAssetsContent as $moduleName) {
                                    $assets = $this->processModuleAssets($moduleName, $typeName, $technologyName, $this->assetType, $assets);
                                }
                            }
                        } elseif ($commit = $branch->getActiveCommit()) {
                            if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                                if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                    foreach ($templatePartsContent as $templatePartConfig) {
                                        $assets = $this->processTemplatePartModulesAssets($templatePartConfig, $assets);
                                    }
                                }
                            }
                            if ($modulesAssetsContent = $commit->getContent(BuilderiusTemplateModulesWithInlineAssetsContentProvider::CONTENT_TYPE)) {
                                if (is_array($modulesAssetsContent) && !empty($modulesAssetsContent)) {
                                    $config = $commit->getContentConfig();
                                    $this->setConfigVersion($config['version']);
                                    foreach ($modulesAssetsContent as $moduleName) {
                                        $assets = $this->processModuleAssets($moduleName, $typeName, $technologyName, $this->assetType, $assets);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        return $assets;
    }
}