<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModulesProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use BuilderiusPro\Bundle\TemplateBundle\EventListener\AbstractAssetsRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateStringTranslationsContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule\DeliverableTemplatePartSubModuleProviderInterface;

class StringTranslationsRegistrationInHookTemplatesFrontendModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var DeliverableTemplateSubModulesProviderInterface
     */
    private $dtsmProvider;

    /**
     * @var DeliverableTemplatePartSubModuleProviderInterface
     */
    private $dtpsmProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigExtension;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    private $cache;

    /**
     * @param DeliverableTemplateSubModulesProviderInterface $dtsmProvider
     * @param DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
     * @param TemplateDataVarsExtension $twigExtension
     * @param BuilderiusRuntimeObjectCache $cache
     */
    public function __construct(
        DeliverableTemplateSubModulesProviderInterface $dtsmProvider,
        DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider,
        TemplateDataVarsExtension $twigExtension,
        BuilderiusRuntimeObjectCache $cache
    ) {
        $this->dtsmProvider = $dtsmProvider;
        $this->dtpsmProvider = $dtpsmProvider;
        $this->twigExtension = $twigExtension;
        $this->cache = $cache;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeStringTranslation(ConfigContainingEvent $event)
    {
        global $l10n;
        if (!isset($l10n['builderius_strings'])) {
            $templateSubModules = $this->dtsmProvider->getTemplateSubModules();
            if (!empty($templateSubModules)) {
                $config = $event->getConfig();
                $cachedDtsmHookTemplate = $this->cache->get('builderius_dtsm_hook_template');
                foreach ($templateSubModules as $templateSubModule) {
                    $this->cache->set('builderius_dtsm_hook_template', $templateSubModule);
                    $cachedHookArgs = $this->cache->get(sprintf('dtsm_hook_template_args_%d', $templateSubModule->getId()));
                    if (false !== $cachedHookArgs) {
                        $this->cache->set('hook_template_args', $cachedHookArgs);
                    }
                    $locale = determine_locale();
                    if ($templatePartsContent = $templateSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $config = $this->processTemplatePartTranslations($templatePartConfig, $config, $locale);
                            }
                        }
                    }
                    $this->cache->delete('builderius_dtsm_hook_template');
                }
                if (false !== $cachedDtsmHookTemplate) {
                    $this->cache->set('builderius_dtsm_hook_template', $cachedDtsmHookTemplate);
                }
                $event->setConfig($config);
            }
        }
    }

    /**
     * @param array $templatePartConfig
     * @param array $config
     * @param string $locale
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartTranslations(array $templatePartConfig, array $config, $locale)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePartSubModule = $this->dtpsmProvider->getTemplatePartSubModule($templatePartConfig['type']);
                if ($templatePartSubModule) {
                    if ($templatePartsContent = $templatePartSubModule->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $config = $this->processTemplatePartTranslations($templatePartConfig, $config, $locale);
                            }
                        }
                    }
                    if ($stContent = $templatePartSubModule->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                        if (is_array($stContent) && isset($stContent[$locale])) {
                            if (empty($config)) {
                                $config = $stContent[$locale];
                            } else {
                                foreach ($stContent[$locale] as $origString => $translatedString) {
                                    $config[$origString] = $translatedString;
                                }
                            }
                        }
                    }
                }
            }
        }

        return $config;
    }
}