<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplatesProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use BuilderiusPro\Bundle\TemplateBundle\EventListener\AbstractAssetsRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateStringTranslationsContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class StringTranslationsRegistrationInHookTemplatesPreviewModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusTemplatesProviderInterface
     */
    private $builderiusHookTemplatesProvider;

    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigExtension;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    private $cache;

    /**
     * @param BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     * @param TemplateDataVarsExtension $twigExtension
     * @param BuilderiusRuntimeObjectCache $cache
     */
    public function __construct(
        BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider,
        BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider,
        TemplateDataVarsExtension $twigExtension,
        BuilderiusRuntimeObjectCache $cache
    ) {
        $this->builderiusHookTemplatesProvider = $builderiusHookTemplatesProvider;
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;
        $this->twigExtension = $twigExtension;
        $this->cache = $cache;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeStringTranslation(ConfigContainingEvent $event)
    {
        global $l10n;
        if (!isset($l10n['builderius_strings'])) {
            $templates = $this->builderiusHookTemplatesProvider->getTemplates();
            if (!empty($templates)) {
                $config = $event->getConfig();
                $cachedHookTemplate = $this->cache->get('builderius_hook_template');
                foreach ($templates as $template) {
                    $this->cache->set('builderius_hook_template', $template);
                    $cachedHookArgs = $this->cache->get(sprintf('hook_template_args_%d', $template->getId()));
                    if (false !== $cachedHookArgs) {
                        $this->cache->set('hook_template_args', $cachedHookArgs);
                    }
                    $locale = determine_locale();
                    if ($branch = $template->getActiveBranch()) {
                        if ($templatePartsContent = $branch->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                            if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                foreach ($templatePartsContent as $templatePartConfig) {
                                    $config = $this->processTemplatePartTranslations($templatePartConfig, $config, $locale);
                                }
                            }
                        } elseif ($commit = $branch->getActiveCommit()) {
                            if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                                if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                    foreach ($templatePartsContent as $templatePartConfig) {
                                        $config = $this->processTemplatePartTranslations($templatePartConfig, $config, $locale);
                                    }
                                }
                            }
                        }
                    }
                    $this->cache->delete('builderius_hook_template');
                }
                if (false !== $cachedHookTemplate) {
                    $this->cache->set('builderius_hook_template', $cachedHookTemplate);
                    $cachedHookArgs = $this->cache->get(sprintf('hook_template_args_%d', $cachedHookTemplate->getId()));
                    if (false !== $cachedHookArgs) {
                        $this->cache->set('hook_template_args', $cachedHookArgs);
                    }
                }
                $event->setConfig($config);
            }
        }
    }

    /**
     * @param array $templatePartConfig
     * @param array $config
     * @param string $locale
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartTranslations(array $templatePartConfig, array $config, $locale)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePart = $this->builderiusTemplatePartProvider->getTemplatePart($templatePartConfig['type']);
                if ($templatePart) {
                    if ($branch = $templatePart->getActiveBranch()) {
                        if ($stContent = $branch->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                            if (is_array($stContent) && isset($stContent[$locale])) {
                                if (empty($config)) {
                                    $config = $stContent[$locale];
                                } else {
                                    foreach ($stContent[$locale] as $origString => $translatedString) {
                                        $config[$origString] = $translatedString;
                                    }
                                }
                            }
                        } else {
                            $commit = $branch->getActiveCommit();
                            if ($commit) {
                                if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                                    if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                        foreach ($templatePartsContent as $templatePartConfig) {
                                            $config = $this->processTemplatePartTranslations($templatePartConfig, $config, $locale);
                                        }
                                    }
                                }
                                if ($stContent = $commit->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                                    if (is_array($stContent) && isset($stContent[$locale])) {
                                        if (empty($config)) {
                                            $config = $stContent[$locale];
                                        } else {
                                            foreach ($stContent[$locale] as $origString => $translatedString) {
                                                $config[$origString] = $translatedString;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        return $config;
    }
}