<?php

namespace BuilderiusPro\Bundle\ComponentBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplateProviderInterface;
use Builderius\Bundle\TemplateBundle\Twig\TemplateDataVarsExtension;
use BuilderiusPro\Bundle\TemplateBundle\EventListener\AbstractAssetsRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateStringTranslationsContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template\BuilderiusTemplateTemplatePartTypesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class StringTranslationsRegistrationInPreviewModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusTemplateProviderInterface
     */
    private $builderiusTemplateProvider;

    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;

    /**
     * @var TemplateDataVarsExtension
     */
    private $twigExtension;

    /**
     * @param BuilderiusTemplateProviderInterface $builderiusTemplateProvider
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     * @param TemplateDataVarsExtension $twigExtension
     */
    public function __construct(
        BuilderiusTemplateProviderInterface $builderiusTemplateProvider,
        BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider,
        TemplateDataVarsExtension $twigExtension
    ) {
        $this->builderiusTemplateProvider = $builderiusTemplateProvider;
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;
        $this->twigExtension = $twigExtension;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeStringTranslation(ConfigContainingEvent $event)
    {
        global $l10n;
        if (!isset($l10n['builderius_strings'])) {
            $template = $this->builderiusTemplateProvider->getTemplate();
            if ($template) {
                $config = $event->getConfig();
                $locale = determine_locale();
                if ($branch = $template->getActiveBranch()) {
                    if ($templatePartsContent = $branch->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                        if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                            foreach ($templatePartsContent as $templatePartConfig) {
                                $config = $this->processTemplatePartTranslations($templatePartConfig, $config, $locale);
                            }
                        }
                    } elseif ($commit = $branch->getActiveCommit()) {
                        if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                            if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                foreach ($templatePartsContent as $templatePartConfig) {
                                    $config = $this->processTemplatePartTranslations($templatePartConfig, $config, $locale);
                                }
                            }
                        }
                    }
                }
                $event->setConfig($config);
            }
        }
    }

    /**
     * @param array $templatePartConfig
     * @param array $config
     * @param string $locale
     * @return array
     * @throws \Exception
     */
    private function processTemplatePartTranslations(array $templatePartConfig, array $config, $locale)
    {
        if (is_array($templatePartConfig) && isset($templatePartConfig['type']) && isset($templatePartConfig['visibilityCondition'])) {
            $condResult = $this->twigExtension->evaluateVisibilityCondition($templatePartConfig['visibilityCondition']);
            if (true === $condResult) {
                $templatePart = $this->builderiusTemplatePartProvider->getTemplatePart($templatePartConfig['type']);
                if ($templatePart) {
                    if ($branch = $templatePart->getActiveBranch()) {
                        if ($stContent = $branch->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                            if (is_array($stContent) && isset($stContent[$locale])) {
                                if (empty($config)) {
                                    $config = $stContent[$locale];
                                } else {
                                    foreach ($stContent[$locale] as $origString => $translatedString) {
                                        $config[$origString] = $translatedString;
                                    }
                                }
                            }
                        } else {
                            $commit = $branch->getActiveCommit();
                            if ($commit) {
                                if ($templatePartsContent = $commit->getContent(BuilderiusTemplateTemplatePartTypesContentProvider::CONTENT_TYPE)) {
                                    if (is_array($templatePartsContent) && !empty($templatePartsContent)) {
                                        foreach ($templatePartsContent as $templatePartConfig) {
                                            $config = $this->processTemplatePartTranslations($templatePartConfig, $config, $locale);
                                        }
                                    }
                                }
                                if ($stContent = $commit->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                                    if (is_array($stContent) && isset($stContent[$locale])) {
                                        if (empty($config)) {
                                            $config = $stContent[$locale];
                                        } else {
                                            foreach ($stContent[$locale] as $origString => $translatedString) {
                                                $config[$origString] = $translatedString;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        return $config;
    }
}