<?php

namespace BuilderiusPro\Bundle\ComponentBundle\Hook;

use Builderius\Bundle\SettingBundle\Factory\BuilderiusGlobalSettingsSetFromPostFactory;
use Builderius\Bundle\TemplateBundle\Cache\BuilderiusPersistentObjectCache;
use Builderius\Bundle\TemplateBundle\Hook\AbstractApplicantDataAction;
use Builderius\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateDataVarsContentProvider;
use Builderius\Bundle\TemplateBundle\Provider\DataVar\DataVarValueGeneratorInterface;
use Builderius\Symfony\Component\Cache\CacheItem;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class ApplicantTemplatePartDataProvidingHook extends AbstractApplicantDataAction
{
    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $templatePartProvider;

    /**
     * @var BuilderiusGlobalSettingsSetFromPostFactory
     */
    protected $globalSettingsSetFromPostFactory;

    /**
     * @var DataVarValueGeneratorInterface
     */
    private $dataVarValueGenerator;

    /**
     * @var BuilderiusPersistentObjectCache
     */
    private $persistentCache;

    /**
     * @var BuilderiusTemplateDataVarsContentProvider
     */
    private $dataVarsContentProvider;

    /**
     * @param BuilderiusTemplatePartProviderInterface $templatePartProvider
     * @return $this
     */
    public function setTemplatePartProvider(
        BuilderiusTemplatePartProviderInterface $templatePartProvider
    ) {
        $this->templatePartProvider = $templatePartProvider;

        return $this;
    }

    /**
     * @param BuilderiusGlobalSettingsSetFromPostFactory $globalSettingsSetFromPostFactory
     * @return $this
     */
    public function setGlobalSettingsSetFromPostFactory(BuilderiusGlobalSettingsSetFromPostFactory $globalSettingsSetFromPostFactory)
    {
        $this->globalSettingsSetFromPostFactory = $globalSettingsSetFromPostFactory;

        return $this;
    }

    /**
     * @param DataVarValueGeneratorInterface $dataVarValueGenerator
     * @return $this
     */
    public function setDataVarValueGenerator(DataVarValueGeneratorInterface $dataVarValueGenerator)
    {
        $this->dataVarValueGenerator = $dataVarValueGenerator;

        return $this;
    }

    /**
     * @param BuilderiusPersistentObjectCache $persistentCache
     * @return $this
     */
    public function setPersistentCache(BuilderiusPersistentObjectCache $persistentCache)
    {
        $this->persistentCache = $persistentCache;

        return $this;
    }

    /**
     * @param BuilderiusTemplateDataVarsContentProvider $dataVarsContentProvider
     * @return $this
     */
    public function setDataVarsContentProvider(BuilderiusTemplateDataVarsContentProvider $dataVarsContentProvider)
    {
        $this->dataVarsContentProvider = $dataVarsContentProvider;

        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getFunction()
    {
        $user = apply_filters('builderius_get_current_user', $this->getUser());
        if (
            isset($_POST['builderius-applicant-template-part-data']) &&
            isset($_POST['uniqid']) &&
            user_can($user, 'builderius-development')
        ) {
            /** @var CacheItem $typesCacheItem */
            $typesCacheItem = $this->persistentCache->getItem(sprintf('applicant_template_part_types_%s', $_POST['uniqid']));
            $templatePartTypes = $typesCacheItem->get();
            $this->persistentCache->delete(sprintf('applicant_template_part_types_%s', $_POST['uniqid']));
            $result = [];
            foreach ($templatePartTypes as $templatePartType) {
                $templatePart = $this->templatePartProvider->getTemplatePart($templatePartType);
                $data = [];
                if (null !== $templatePart) {
                    $commit = $templatePart->getActiveBranch()->getActiveCommit();
                    if (null !== $commit) {
                        $data['module'] = 'TmplPart';
                        $data['value'] = $templatePartType;
                        $data['builder_mode_link'] = $templatePart->getBuilderModeLink();
                        $data['config'] = $this->getFinalConfig($commit->getContentConfig());
                        $dataVarsContent = $this->dataVarsContentProvider->getContent('html', $data['config']);
                        $finalDataVarsConfig = $this->getDataVarsFinalConfigs($dataVarsContent);
                        $dataVarsValues = [];
                        foreach (array_keys($finalDataVarsConfig) as $name) {
                            $dataVarsValues = $this->dataVarValueGenerator->generateValue(
                                $templatePart->getType(),
                                $name,
                                $finalDataVarsConfig,
                                $dataVarsValues
                            );
                        }
                        $data['data'] = $dataVarsValues;
                    }
                    $result[] = $data;
                }
            }

            /** @var CacheItem $dataCacheItem */
            $dataCacheItem = $this->persistentCache->getItem(sprintf('applicant_template_parts_data_%s', $_POST['uniqid']));
            $dataCacheItem->set($result);
            $this->persistentCache->save($dataCacheItem);
        }
    }

    /**
     * @param array $config
     * @return array
     * @throws \Exception
     */
    public function getFinalConfig(array $config)
    {
        $innerTemplatePartTypes = [];
        foreach ($config['modules'] as $midx => $moduleConfig) {
            if ($moduleConfig['name'] === 'TmplPart') {
                $visibilityCondition = null;
                foreach ($moduleConfig['settings'] as $settingConfig) {
                    if ($settingConfig['name'] === 'visibilityCondition' && !empty($settingConfig['value']['a1'])) {
                        $visibilityCondition = $settingConfig['value']['a1'];
                        break;
                    }
                }
                foreach ($moduleConfig['settings'] as $settingConfig) {
                    if ($settingConfig['name'] === 'dataTmplPart') {
                        if (!in_array($settingConfig['value']['a1'], $innerTemplatePartTypes)) {
                            $innerTemplatePartTypes[] = $settingConfig['value']['a1'];
                            $templatePart = $this->templatePartProvider->getTemplatePart($settingConfig['value']['a1']);
                            if (null !== $templatePart) {
                                $commit = $templatePart->getActiveBranch()->getActiveCommit();
                                if (null !== $commit) {
                                    $subconfig = $this->getFinalConfig($commit->getContentConfig());
                                    $config = $this->processModules($midx, $config, $subconfig, $visibilityCondition);
                                    $config = $this->processSingularSetting('dataVars', 'b1',  $config, $subconfig);
                                    $config = $this->processSingularSetting('stringTranslations', 'a1',  $config, $subconfig);
                                    $config = $this->processMultiSetting('customJs', $config, $subconfig);
                                    $config = $this->processMultiSetting('customCss', $config, $subconfig);
                                    $config = $this->processMultiSetting('jsLibraries', $config, $subconfig);
                                    $config = $this->processMultiSetting('cssLibraries', $config, $subconfig);
                                }
                            }
                        }
                    }
                }
                $config = $this->getFinalConfig($config);
                break;
            }
        }

        return $config;
    }

    /**
     * @param int $midx
     * @param array $config
     * @param array $subconfig
     * @return array
     */
    private function processModules($midx, array $config, array $subconfig, $tpVisibilityCondition)
    {
        if (isset($subconfig['modules'])) {
            unset($config['modules'][$midx]);
            foreach ($subconfig['modules'] as $id => $modConf) {
                if (null !== $tpVisibilityCondition) {
                    $hasVisibilityConditions = false;
                    foreach ($modConf['settings'] as $k => $settingConfig) {
                        if ($settingConfig['name'] === 'visibilityCondition') {
                            $hasVisibilityConditions = true;
                            if (!empty($settingConfig['value']['a1']) && $settingConfig['value']['a1'] !== $tpVisibilityCondition) {
                                $modConf['settings'][$k]['value']['a1'] = sprintf(
                                    '(%s) && (%s)',
                                    $settingConfig['value']['a1'],
                                    $tpVisibilityCondition
                                );
                            } else {
                                $modConf['settings'][$k]['value']['a1'] = $tpVisibilityCondition;
                            }
                            break;
                        }
                    }
                    if (false === $hasVisibilityConditions) {
                        $modConf['settings'][] = [
                            'name' => 'visibilityCondition',
                            'value' => [
                                'a1' => $tpVisibilityCondition
                            ]
                        ];
                    }
                }
                $config['modules'][$id] = $modConf;
            }
        }
        if (isset($subconfig['indexes'])) {
            foreach ($subconfig['indexes'] as $pidx => $cidx) {
                if ('root' !== $pidx) {
                    $config['indexes'][$pidx] = $cidx;
                } else {
                    foreach ($config['indexes'] as $idx => $cld) {
                        $newCld = [];
                        foreach ($cld as $n) {
                            if ($n === $midx) {
                                foreach ($cidx as $nn) {
                                    $newCld[] = $nn;
                                }
                            } else {
                                $newCld[] = $n;
                            }
                        }
                        $config['indexes'][$idx] = $newCld;
                    }
                }
            }
        }

        return $config;
    }

    /**
     * @param array $config
     * @param array $subconfig
     * @return array
     */
    private function processSingularSetting($settingName, $nameKey, array $config, array $subconfig)
    {
        $configTmplSettings = $config['template']['settings'];
        $subConfigTmplSettings = $subconfig['template']['settings'];
        $confDataVarIndex = $this->getTemplateSettingIndex($configTmplSettings, $settingName);
        $subconfDataVarIndex = $this->getTemplateSettingIndex($subConfigTmplSettings, $settingName);
        if (null === $subconfDataVarIndex) {
            return $config;
        }
        $configDatVars = [
            'name' => $settingName,
            'value' => [
                'i1' => []
            ]
        ];
        if (null !== $confDataVarIndex) {
            $configDatVars = $configTmplSettings[$confDataVarIndex];
        }
        foreach ($subConfigTmplSettings[$subconfDataVarIndex]['value']['i1'] as $sDataVar)
        {
            $sDataVarName = $sDataVar[$nameKey];
            $foundSameName = false;
            foreach ($configDatVars['value']['i1'] as $dataVar)
            {
                $dataVarName = $dataVar[$nameKey];
                if ($sDataVarName === $dataVarName) {
                    $foundSameName = true;
                }
            }
            if (false === $foundSameName) {
                $configDatVars['value']['i1'][] = $sDataVar;
            }
        }
        if (null !== $confDataVarIndex) {
            $config['template']['settings'][$confDataVarIndex] = $configDatVars;
        } else {
            $config['template']['settings'][] = $configDatVars;
        }

        return $config;
    }

    /**
     * @param array $config
     * @param array $subconfig
     * @return array
     */
    private function processMultiSetting($settingName, array $config, array $subconfig)
    {
        $configTmplSettings = $config['template']['settings'];
        $subConfigTmplSettings = $subconfig['template']['settings'];
        $confSettValIndex = $this->getTemplateSettingIndex($configTmplSettings, $settingName);
        $subconfSettValIndex = $this->getTemplateSettingIndex($subConfigTmplSettings, $settingName);
        if (null === $subconfSettValIndex) {
            return $config;
        }
        $configDatVars = [
            'name' => $settingName,
            'value' => [
                'i1' => []
            ]
        ];
        if (null !== $confSettValIndex) {
            $configDatVars = $configTmplSettings[$confSettValIndex];
        }
        foreach ($subConfigTmplSettings[$subconfSettValIndex]['value']['i1'] as $sSettVal)
        {
            array_unshift($configDatVars['value']['i1'], $sSettVal);
        }
        if (null !== $confSettValIndex) {
            $config['template']['settings'][$confSettValIndex] = $configDatVars;
        } else {
            $config['template']['settings'][] = $configDatVars;
        }

        return $config;
    }

    /**
     * @param array $settingsConfig
     * @param string $settingName
     * @return int|null
     */
    private function getTemplateSettingIndex(array $settingsConfig, $settingName)
    {
        foreach ($settingsConfig as $i => $settingConfig)
        {
            if ($settingConfig['name'] === $settingName) {
                return $i;
            }
        }

        return null;
    }

    /**
     * @param array $templatePartDataVarsContent
     * @return array
     */
    public function getDataVarsFinalConfigs(array $templatePartDataVarsContent)
    {
        $dataVarsConfigs = [];
        if (is_array($templatePartDataVarsContent)) {
            foreach ($templatePartDataVarsContent as $templatePartDataVarsItem) {
                $dataVarsConfigs[$templatePartDataVarsItem['name']] = $templatePartDataVarsItem;
            }
        }

        return $dataVarsConfigs;
    }
}