<?php

namespace BuilderiusPro\Bundle\ComponentBundle\Hook;

use Builderius\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateHtmlContentProvider;
use Builderius\MooMoo\Platform\Bundle\HookBundle\Model\AbstractFilter;
use Builderius\Symfony\Component\Templating\EngineInterface;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class HtmlInsertionInPreviewModeHook extends AbstractFilter
{
    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;

    /**
     * @var EngineInterface
     */
    private $templatingEngine;

    /**
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     * @return $this
     */
    public function setBuilderiusTemplatePartProvider(BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider)
    {
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;

        return $this;
    }

    /**
     * @param EngineInterface $templatingEngine
     * @return $this
     */
    public function setTemplatingEngine(EngineInterface $templatingEngine)
    {
        $this->templatingEngine = $templatingEngine;
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getFunction()
    {
        $type = func_get_arg(0);
        $template = $this->builderiusTemplatePartProvider->getTemplatePart($type);
        if ($template && $branch = $template->getActiveBranch()) {
            if ($html = $branch->getContent(BuilderiusTemplateHtmlContentProvider::CONTENT_TYPE)) {
                $html = str_replace('builderius_data_var(', sprintf('builderius_tp_data_var(\'%s\', ', $template->getSubType()), $html);
                $html = str_replace('builderius_data_var_escaped(', sprintf('builderius_tp_data_var_escaped(\'%s\', ', $template->getSubType()), $html);
                $html = str_replace('builderius_visibility_condition(', sprintf('builderius_tp_visibility_condition(\'%s\', ', $template->getSubType()), $html);

                return do_blocks(do_shortcode($this->templatingEngine->render(
                    'BuilderiusTemplateBundle:templateDynamicContent.twig',
                    [
                        'dynamicContent' => $html
                    ]
                )));
            } elseif ($commit = $branch->getActiveCommit()) {
                if ($html = $commit->getContent(BuilderiusTemplateHtmlContentProvider::CONTENT_TYPE)) {
                    $html = str_replace('builderius_data_var(', sprintf('builderius_tp_data_var(\'%s\', ', $template->getSubType()), $html);
                    $html = str_replace('builderius_data_var_escaped(', sprintf('builderius_tp_data_var_escaped(\'%s\', ', $template->getSubType()), $html);
                    $html = str_replace('builderius_visibility_condition(', sprintf('builderius_tp_visibility_condition(\'%s\', ', $template->getSubType()), $html);

                    return do_blocks(do_shortcode($this->templatingEngine->render(
                        'BuilderiusTemplateBundle:templateDynamicContent.twig',
                        [
                            'dynamicContent' => $html
                        ]
                    )));
                }
            }
        }

        return '';
    }
}
