<?php

namespace BuilderiusPro\Bundle\ComponentBundle\Provider\Content\Template;

use Builderius\Bundle\ModuleBundle\Provider\BuilderiusModulesProviderInterface;
use Builderius\Bundle\SettingBundle\Factory\SettingValue\BuilderiusSettingValueFactoryInterface;
use Builderius\Bundle\SettingBundle\Generator\FinalSettingValue\FinalSettingValueGeneratorInterface;
use Builderius\Bundle\SettingBundle\Model\BuilderiusSettingCssAwareInterface;
use Builderius\Bundle\SettingBundle\Model\BuilderiusSettingValue;
use Builderius\Bundle\TemplateBundle\Converter\ConfigToHierarchicalConfigConverter;
use Builderius\Bundle\TemplateBundle\Provider\Content\Template\AbstractBuilderiusTemplateContentProvider;

class BuilderiusTemplateTemplatePartTypesContentProvider extends AbstractBuilderiusTemplateContentProvider
{
    const CONTENT_TYPE = 'templatePartTypes';

    /**
     * @var BuilderiusModulesProviderInterface
     */
    private $modulesProvider;

    /**
     * @var BuilderiusSettingValueFactoryInterface
     */
    private $settingValueFactory;

    /**
     * @var FinalSettingValueGeneratorInterface
     */
    private $finalSettingValueGenerator;

    /**
     * @param BuilderiusModulesProviderInterface $modulesProvider
     * @param BuilderiusSettingValueFactoryInterface $settingValueFactory
     * @param FinalSettingValueGeneratorInterface $finalSettingValueGenerator
     */
    public function __construct(
        BuilderiusModulesProviderInterface $modulesProvider,
        BuilderiusSettingValueFactoryInterface $settingValueFactory,
        FinalSettingValueGeneratorInterface $finalSettingValueGenerator
    ) {
        $this->modulesProvider = $modulesProvider;
        $this->settingValueFactory = $settingValueFactory;
        $this->finalSettingValueGenerator = $finalSettingValueGenerator;
    }

    /**
     * @inheritDoc
     */
    public function getContentType()
    {
        return self::CONTENT_TYPE;
    }

    /**
     * @inheritDoc
     */
    public function getContent($technology, array $contentConfig)
    {
        if (!in_array($technology, $this->technologies)) {
            return null;
        }
        $templateType = $contentConfig['template']['type'];
        $templatePartTypes = [];
        if (isset($contentConfig[ConfigToHierarchicalConfigConverter::CONFIG_MODULES_KEY])) {
            foreach ($contentConfig[ConfigToHierarchicalConfigConverter::CONFIG_MODULES_KEY] as $moduleConfig) {
                if (isset($moduleConfig['name'])) {
                    $module = $this->modulesProvider->getModule($moduleConfig['name'], $templateType, $technology);
                    if ($moduleConfig['name'] === 'TmplPart') {
                        $data = [];
                        $parentVisibilityCondition = null;
                        if (!empty($moduleConfig['parent'])) {
                            $parentVisibilityCondition = $this->getParentVisibilityCondition($contentConfig, $moduleConfig['parent']);
                        }
                        foreach ($moduleConfig['settings'] as $settingData) {
                            $setting = $module->getSetting($settingData['name']);
                            if ($setting) {
                                $setting->resetValues();
                                if ($settingData['name'] === 'visibilityCondition') {
                                    $settingValue = $this->settingValueFactory->create([
                                        BuilderiusSettingCssAwareInterface::CSS_FIELD => false,
                                        BuilderiusSettingValue::VALUE_FIELD => $settingData['value']
                                    ]);
                                    $setting->addValue($settingValue);
                                    $settingValues = $setting->getValues();
                                    $settingValue = reset($settingValues);
                                    $valueExpressions = $setting->getValueExpressions();
                                    $valueExpression = reset($valueExpressions);
                                    $data['visibilityCondition'] = $this->finalSettingValueGenerator->generateFinalSettingValue(
                                        $settingValue,
                                        $valueExpression,
                                        $setting->getValueSchema()
                                    );
                                    $setting->resetValues();
                                } elseif ($settingData['name'] === 'dataTmplPart') {
                                    $data['type'] = $settingData['value']['a1'];
                                }
                                $setting->resetValues();
                            }
                        }
                        if (!isset($data['visibilityCondition']) && empty($parentVisibilityCondition)) {
                            $data['visibilityCondition'] = 'true==true';
                        }
                        if (!empty($parentVisibilityCondition)) {
                            if (isset($data['visibilityCondition'])) {
                                $data['visibilityCondition'] = sprintf('(%s) && (%s)', $parentVisibilityCondition, $data['visibilityCondition']);
                            } else {
                                $data['visibilityCondition'] = $parentVisibilityCondition;
                            }
                        }

                        if (!in_array($data, $templatePartTypes)) {
                            $templatePartTypes[] = $data;
                        }
                    }
                }
            }
        }

        return $templatePartTypes;
    }

    /**
     * @param array $contentConfig
     * @param $moduleId
     * @return string
     */
    private function getParentVisibilityCondition(array $contentConfig, $moduleId)
    {
        $visibilityCondition = null;
        $parentVisibilityCondition = null;
        if (
            isset($contentConfig[ConfigToHierarchicalConfigConverter::CONFIG_MODULES_KEY]) &&
            isset($contentConfig[ConfigToHierarchicalConfigConverter::CONFIG_MODULES_KEY][$moduleId])
        ) {
            $moduleConfig = $contentConfig[ConfigToHierarchicalConfigConverter::CONFIG_MODULES_KEY][$moduleId];
            foreach ($moduleConfig['settings'] as $settingData) {
                if ($settingData['name'] === 'visibilityCondition') {
                    $visibilityCondition = $settingData['value']['a1'];
                    break;
                }
            }
            if (!empty($moduleConfig['parent'])) {
                $parentVisibilityCondition = $this->getParentVisibilityCondition($contentConfig, $moduleConfig['parent']);
            }
        }
        if (!empty($visibilityCondition) && !empty($parentVisibilityCondition)) {
            if ($visibilityCondition !== $parentVisibilityCondition) {
                return sprintf('(%s) && (%s)', $parentVisibilityCondition, $visibilityCondition);
            } else {
                return $visibilityCondition;
            }
        } elseif (empty($visibilityCondition)) {
            return $parentVisibilityCondition;
        } else {
            return $visibilityCondition;
        }
    }
}