<?php

namespace BuilderiusPro\Bundle\ComponentBundle\Provider\DataVar;

use Builderius\Bundle\DeliverableBundle\Provider\BuilderiusDeliverableProviderInterface;
use Builderius\Bundle\SettingBundle\Factory\BuilderiusGlobalSettingsSetFromPostFactory;
use Builderius\Bundle\SettingBundle\Registration\BuilderiusGlobalSettingsSetPostType;
use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateDataVarsContentProvider;
use Builderius\Bundle\TemplateBundle\Provider\DataVar\DataVarValueGeneratorInterface;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModuleProviderInterface;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplateProviderInterface;
use Builderius\MooMoo\Platform\Bundle\ConditionBundle\Model\ConditionInterface;
use BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule\DeliverableTemplatePartSubModuleProviderInterface;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class DataVarsFinalValuesProvider implements DataVarsFinalValuesProviderInterface
{
    /**
     * @var DataVarValueGeneratorInterface
     */
    private $dataVarValueGenerator;

    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $templatePartProvider;

    /**
     * @var BuilderiusTemplateProviderInterface
     */
    private $templateProvider;
    /**
     * @var BuilderiusDeliverableProviderInterface
     */
    private $deliverableProvider;

    /**
     * @var DeliverableTemplatePartSubModuleProviderInterface
     */
    private $dtpsmProvider;

    /**
     * @var DeliverableTemplateSubModuleProviderInterface
     */
    private $dtsmProvider;

    /**
     * @var ConditionInterface[]
     */
    private $previewConditions;

    /**
     * @var ConditionInterface[]
     */
    private $frontendConditions;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    private $cache;

    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @var BuilderiusGlobalSettingsSetFromPostFactory
     */
    protected $globalSettingsSetFromPostFactory;

    /**
     * @param DataVarValueGeneratorInterface $dataVarValueGenerator
     * @param BuilderiusTemplatePartProviderInterface $templatePartProvider
     * @param BuilderiusTemplateProviderInterface $templateProvider
     * @param BuilderiusDeliverableProviderInterface $deliverableProvider
     * @param DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
     * @param DeliverableTemplateSubModuleProviderInterface $dtsmProvider
     * @param ConditionInterface[] $previewConditions
     * @param ConditionInterface[] $frontendConditions
     * @param BuilderiusRuntimeObjectCache $cache
     * @param \WP_Query $wpQuery
     * @param BuilderiusGlobalSettingsSetFromPostFactory $globalSettingsSetFromPostFactory
     */
    public function __construct(
        DataVarValueGeneratorInterface $dataVarValueGenerator,
        BuilderiusTemplatePartProviderInterface $templatePartProvider,
        BuilderiusTemplateProviderInterface $templateProvider,
        BuilderiusDeliverableProviderInterface $deliverableProvider,
        DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider,
        DeliverableTemplateSubModuleProviderInterface $dtsmProvider,
        array $previewConditions,
        array $frontendConditions,
        BuilderiusRuntimeObjectCache $cache,
        \WP_Query $wpQuery,
        BuilderiusGlobalSettingsSetFromPostFactory $globalSettingsSetFromPostFactory
    ) {
        $this->dataVarValueGenerator = $dataVarValueGenerator;
        $this->templatePartProvider = $templatePartProvider;
        $this->templateProvider = $templateProvider;
        $this->deliverableProvider = $deliverableProvider;
        $this->dtpsmProvider = $dtpsmProvider;
        $this->dtsmProvider = $dtsmProvider;
        $this->previewConditions = $previewConditions;
        $this->frontendConditions = $frontendConditions;
        $this->cache = $cache;
        $this->wpQuery = $wpQuery;
        $this->globalSettingsSetFromPostFactory = $globalSettingsSetFromPostFactory;
    }

    /**
     * @return bool
     */
    private function isPreview()
    {
        foreach ($this->previewConditions as $previewCondition) {
            if (!$previewCondition->evaluate()) {
                return false;
            }
        }

        return true;
    }

    /**
     * @return bool
     */
    private function isFrontend()
    {
        foreach ($this->frontendConditions as $frontendCondition) {
            if (!$frontendCondition->evaluate()) {
                return false;
            }
        }

        return true;
    }
    /**
     * @param string $type
     * @return array
     */
    public function getDataVarsFinalConfigs($type)
    {
        $dataVarsConfigs = [];
        if ($this->isPreview()) {
            $dataVarsConfigs = $this->cache->get(sprintf('builderius_data_tp_%s_vars_final_configs', $type));
            if (false === $dataVarsConfigs) {
                $templatePart = $this->templatePartProvider->getTemplatePart($type);
                if ($templatePart) {
                    $technologyName = $templatePart->getTechnology();
                    $posts = $this->cache->get(sprintf('builderius_gss_posts_%s', $technologyName));
                    if (false == $posts) {
                        $queryArgs = [
                            'post_type' => BuilderiusGlobalSettingsSetPostType::POST_TYPE,
                            'post_status' => get_post_stati(),
                            'name' => $technologyName,
                            'posts_per_page' => -1,
                            'no_found_rows' => true,
                            'orderby' => 'name',
                            'order' => 'ASC'
                        ];
                        $posts = $this->wpQuery->query($queryArgs);
                        $this->cache->set(sprintf('builderius_gss_posts_%s', $technologyName), $posts);
                    }
                    foreach ($posts as $post) {
                        $globalSettingsSet = $this->globalSettingsSetFromPostFactory->createGlobalSettingsSet($post);
                        $branch = $globalSettingsSet->getActiveBranch();
                        $dataVarsContent = [];
                        if ($branch) {
                            if ($branch->getContent(BuilderiusTemplateDataVarsContentProvider::CONTENT_TYPE)) {
                                $dataVarsContent = $branch->getContent(BuilderiusTemplateDataVarsContentProvider::CONTENT_TYPE);
                            } else {
                                $commit = $branch->getActiveCommit();
                                if ($commit) {
                                    if ($commit->getContent(BuilderiusTemplateDataVarsContentProvider::CONTENT_TYPE)) {
                                        $dataVarsContent = $commit->getContent(BuilderiusTemplateDataVarsContentProvider::CONTENT_TYPE);
                                    }
                                }
                            }
                        }
                        if (is_array($dataVarsContent)) {
                            foreach ($dataVarsContent as $dataVarsContentItem) {
                                $dataVarsConfigs[$dataVarsContentItem['name']] = $dataVarsContentItem;
                            }
                        }
                    }
                    $templatePartDataVars = [];
                    $branch = $templatePart->getActiveBranch();
                    if ($branch) {
                        $commit = $branch->getActiveCommit();
                        if ($commit) {
                            $templatePartDataVars = $commit->getContent(BuilderiusTemplateDataVarsContentProvider::CONTENT_TYPE);
                        } else {
                            $templatePartDataVars = $branch->getContent(BuilderiusTemplateDataVarsContentProvider::CONTENT_TYPE);
                        }
                    }
                    if (is_array($templatePartDataVars)) {
                        foreach ($templatePartDataVars as $templatePartDataVarsItem) {
                            $dataVarsConfigs[$templatePartDataVarsItem['name']] = $templatePartDataVarsItem;
                        }
                    }
                    $template = $this->templateProvider->getTemplate();
                    if (!$template) {
                        $template = $this->cache->get('builderius_hook_template');
                    }
                    if ($template) {
                        $templateDataVars = [];
                        $branch = $template->getActiveBranch();
                        if ($branch) {
                            $commit = $branch->getActiveCommit();
                            if ($commit) {
                                $templateDataVars = $commit->getContent(BuilderiusTemplateDataVarsContentProvider::CONTENT_TYPE);
                            } else {
                                $templateDataVars = $branch->getContent(BuilderiusTemplateDataVarsContentProvider::CONTENT_TYPE);
                            }
                        }
                        if (is_array($templateDataVars)) {
                            foreach ($templateDataVars as $templateDataVarsItem) {
                                if ($templateDataVarsItem['name'] !== 'wp') {
                                    $dataVarsConfigs[$templateDataVarsItem['name']] = $templateDataVarsItem;
                                }
                            }
                        }
                    }
                }
            }
        }
        if ($this->isFrontend()) {
            $dataVarsConfigs = $this->cache->get(sprintf('builderius_data_tp_%s_vars_final_configs', $type));
            if (false === $dataVarsConfigs) {
                $dataVarsConfigs = [];
                $templatePartSubModule = $this->dtpsmProvider->getTemplatePartSubModule($type);
                if ($templatePartSubModule) {
                    $technology = $templatePartSubModule->getTechnology();
                    $deliverable = $this->deliverableProvider->getDeliverable();
                    $gssAll = $deliverable->getSubModules('global_settings_set', $technology);
                    if (!empty($gssAll)) {
                        $gssAll = reset($gssAll);
                        $gssAllDataVars = $gssAll->getContent(BuilderiusTemplateDataVarsContentProvider::CONTENT_TYPE) ? : [];
                        foreach ($gssAllDataVars as $gssAllDataVarsItem) {
                            $dataVarsConfigs[$gssAllDataVarsItem['name']] = $gssAllDataVarsItem;
                        }
                    }
                    $templatePartDataVars = $templatePartSubModule->getContent(BuilderiusTemplateDataVarsContentProvider::CONTENT_TYPE) ? : [];
                    foreach ($templatePartDataVars as $templatePartDataVarsItem) {
                        $dataVarsConfigs[$templatePartDataVarsItem['name']] = $templatePartDataVarsItem;
                    }
                    $templateSubModule = $this->dtsmProvider->getTemplateSubModule();
                    if (!$templateSubModule) {
                        $templateSubModule = $this->cache->get('builderius_dtsm_hook_template');
                    }
                    if ($templateSubModule) {
                        $templateDataVars = $templateSubModule->getContent(BuilderiusTemplateDataVarsContentProvider::CONTENT_TYPE) ? : [];
                        foreach ($templateDataVars as $templateDataVarsItem) {
                            if ($templateDataVarsItem['name'] !== 'wp') {
                                $dataVarsConfigs[$templateDataVarsItem['name']] = $templateDataVarsItem;
                            }
                        }
                    }
                }
            }
        }
        $this->cache->set(sprintf('builderius_data_tp_%s_vars_final_configs', $type), $dataVarsConfigs);

        return $dataVarsConfigs;
    }

    /**
     * @inheritDoc
     */
    public function getDataVarsFinalValues($type)
    {
        $gettingHookArgsBeforeHook = $this->cache->get('getting_hook_args_before_hook');
        if (false === $gettingHookArgsBeforeHook) {
            $dataVarsValues = $this->cache->get(sprintf('builderius_data_tp_%s_vars_values', $type));
        } else {
            $dataVarsValues = $this->cache->get(sprintf('builderius_data_tp_%_vars_values_with_hook_args_before_hook', $type));
        }
        $dataVarsConfigs = $this->getDataVarsFinalConfigs($type);
        if (false === $dataVarsValues || (is_array($dataVarsValues) && count($dataVarsValues) !== count($dataVarsValues))) {
            if (false === $dataVarsValues) {
                $dataVarsValues = [];
            }
            foreach (array_keys($dataVarsConfigs) as $name) {
                $dataVarsValues = $this->dataVarValueGenerator->generateValue(
                    'template_part',
                    $name,
                    $dataVarsConfigs,
                    $dataVarsValues
                );
            }
            $gettingHookArgsBeforeHook = $this->cache->get('getting_hook_args_before_hook');
            if (false === $gettingHookArgsBeforeHook) {
                $dataVarsValues = $this->cache->set(sprintf('builderius_data_tp_%s_vars_values', $type), $dataVarsValues);
            } else {
                $dataVarsValues = $this->cache->set(sprintf('builderius_data_tp_%_vars_values_with_hook_args_before_hook', $type), $dataVarsValues);
            }
        }

        return $dataVarsValues;
    }

    /**
     * @inheritDoc
     */
    public function getDataVarFinalValue($type, $dataVarName)
    {
        $dataVarsValues = $this->cache->get(sprintf('builderius_data_tp_%s_vars_values', $type));
        if (false === $dataVarsValues || !in_array($dataVarName, array_keys($dataVarsValues))) {
            if (false === $dataVarsValues) {
                $dataVarsValues = [];
            }
            $dataVarsValues = $this->dataVarValueGenerator->generateValue(
                'template_part',
                $dataVarName,
                $this->getDataVarsFinalConfigs($type),
                $dataVarsValues
            );
            $this->cache->set(sprintf('builderius_data_tp_%s_vars_values', $type), $dataVarsValues);
        }

        return array_key_exists($dataVarName, $dataVarsValues) ? $dataVarsValues[$dataVarName] : null;
    }
}