<?php

namespace BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule;

use Builderius\Bundle\DeliverableBundle\Model\BuilderiusDeliverableSubModuleInterface;
use Builderius\Bundle\DeliverableBundle\Provider\BuilderiusDeliverableProviderInterface;
use Builderius\Bundle\TemplateBundle\ApplyRule\Checker\BuilderiusTemplateApplyRulesChecker;
use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\TemplateBundle\Model\BuilderiusTemplate;

class DeliverableTemplatePartSubModuleProvider implements DeliverableTemplatePartSubModuleProviderInterface
{
    /**
     * @var BuilderiusDeliverableProviderInterface
     */
    private $deliverableProvider;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    protected $cache;

    /**
     * @var BuilderiusTemplateApplyRulesChecker
     */
    private $applyRuleChecker;

    /**
     * @param BuilderiusDeliverableProviderInterface $deliverableProvider
     * @param BuilderiusRuntimeObjectCache $cache
     * @param BuilderiusTemplateApplyRulesChecker $applyRuleChecker
     */
    public function __construct(
        BuilderiusDeliverableProviderInterface $deliverableProvider,
        BuilderiusRuntimeObjectCache $cache,
        BuilderiusTemplateApplyRulesChecker $applyRuleChecker
    ) {
        $this->deliverableProvider = $deliverableProvider;
        $this->cache = $cache;
        $this->applyRuleChecker = $applyRuleChecker;
    }

    /**
     * @inheritDoc
     */
    public function getTemplatePartSubModule($type)
    {
        $cacheKey = sprintf('dtpsm_%s', $type);
        $templatePartSubModule = $this->cache->get($cacheKey);
        if (false === $templatePartSubModule) {
            $templatePartSubModule = null;
            $deliverable = $this->deliverableProvider->getDeliverable();
            if ($deliverable) {
                $applicableTmpltPartSubModules = [];
                $tmpltPartSubModules = $deliverable->getSubModules('template_part', null, $type);

                foreach ($tmpltPartSubModules as $tmpltPartSubModule) {
                    $applyRuleConfig = $tmpltPartSubModule->getAttribute(BuilderiusTemplate::APPLY_RULES_CONFIG_FIELD) ?: [];
                    if ($this->applyRuleChecker->checkApplyRule($applyRuleConfig)) {
                        $applicableTmpltPartSubModules[] = $tmpltPartSubModule;
                    }
                }
                if (count($applicableTmpltPartSubModules) > 1) {
                    usort($applicableTmpltPartSubModules, function (BuilderiusDeliverableSubModuleInterface $a, BuilderiusDeliverableSubModuleInterface $b) {
                        $aSortOrder = (int)$a->getAttribute(BuilderiusTemplate::SORT_ORDER_FIELD) ?: 10;
                        $bSortOrder = (int)$b->getAttribute(BuilderiusTemplate::SORT_ORDER_FIELD) ?: 10;
                        if ($aSortOrder < $bSortOrder) {
                            return -1;
                        } elseif ($aSortOrder > $bSortOrder) {
                            return 1;
                        } else {
                            return 0;
                        }
                    });
                }
                $templatePartSubModule = reset($applicableTmpltPartSubModules);
                $this->cache->set($cacheKey, $templatePartSubModule);
            }
        }

        return $templatePartSubModule;
    }
}