<?php

namespace BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart;

use Builderius\Bundle\TemplateBundle\ApplyRule\Checker\BuilderiusTemplateApplyRulesChecker;
use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\TemplateBundle\Factory\BuilderiusTemplateFromPostFactory;
use Builderius\Bundle\TemplateBundle\Model\BuilderiusTemplate;
use Builderius\Bundle\TemplateBundle\Provider\TemplateType\BuilderiusTemplateTypesProviderInterface;
use Builderius\Bundle\TemplateBundle\Registration\BuilderiusTemplatePostType;
use Builderius\Bundle\TemplateBundle\Registration\BuilderiusTemplateTechnologyTaxonomy;
use Builderius\Bundle\TemplateBundle\Registration\BuilderiusTemplateTypeTaxonomy;
use Builderius\MooMoo\Platform\Bundle\ConditionBundle\Registry\ConditionsRegistryInterface;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePartPosts\BuilderiusTemplatePartPostsProviderInterface;

class BuilderiusTemplatePartProvider implements BuilderiusTemplatePartProviderInterface
{
    /**
     * @var BuilderiusTemplateFromPostFactory
     */
    protected $builderiusTemplateFromPostFactory;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    protected $cache;

    /**
     * @var BuilderiusTemplateApplyRulesChecker
     */
    private $applyRuleChecker;

    /**
     * @var BuilderiusTemplateTypesProviderInterface
     */
    private $templateTypesProvider;

    /**
     * @var BuilderiusTemplatePartPostsProviderInterface
     */
    private $templatePostsProvider;

    /**
     * @var ConditionsRegistryInterface
     */
    private $conditionsRegistry;

    /**
     * @param BuilderiusTemplateFromPostFactory $builderiusTemplateFromPostFactory
     * @param BuilderiusTemplateApplyRulesChecker $applyRuleChecker
     * @param BuilderiusTemplateTypesProviderInterface $templateTypesProvider
     * @param BuilderiusTemplatePartPostsProviderInterface $templatePostsProvider
     * @param BuilderiusRuntimeObjectCache $cache
     * @param ConditionsRegistryInterface $conditionsRegistry
     */
    public function __construct(
        BuilderiusTemplateFromPostFactory $builderiusTemplateFromPostFactory,
        BuilderiusTemplateApplyRulesChecker $applyRuleChecker,
        BuilderiusTemplateTypesProviderInterface $templateTypesProvider,
        BuilderiusTemplatePartPostsProviderInterface $templatePostsProvider,
        BuilderiusRuntimeObjectCache $cache,
        ConditionsRegistryInterface $conditionsRegistry
    ) {
        $this->builderiusTemplateFromPostFactory = $builderiusTemplateFromPostFactory;
        $this->cache = $cache;
        $this->applyRuleChecker = $applyRuleChecker;
        $this->templateTypesProvider = $templateTypesProvider;
        $this->templatePostsProvider = $templatePostsProvider;
        $this->conditionsRegistry = $conditionsRegistry;
    }

    /**
     * @inheritDoc
     */
    public function getTemplatePartPost($type = null)
    {
        if (null !== $type) {
            $templatePartPost = $this->cache->get(sprintf('builderius_template_part_post_%s', $type));
        } else {
            $templatePartPost = $this->cache->get('builderius_template_part_post');
        }
        if ( false === $templatePartPost) {
            $types = ['template_part'];
            $technologies = [];
            foreach ($this->templateTypesProvider->getType('template_part')->getTechnologies() as $technology) {
                $technologies[] = $technology->getName();
            }
            /** @var \WP_Post $post */
            $post = get_post();
            if ($post && $post->post_type === BuilderiusTemplatePostType::POST_TYPE && !is_404()) {
                $typeTerms = get_the_terms($post->ID, BuilderiusTemplateTypeTaxonomy::NAME);
                $type = !empty($typeTerms) ? reset($typeTerms)->slug : null;
                $technologyTerms = get_the_terms($post->ID, BuilderiusTemplateTechnologyTaxonomy::NAME);
                $technology = !empty($technologyTerms) ? reset($technologyTerms)->slug : null;
                $isBuilderMode = $this->conditionsRegistry->getCondition('is_builderius_builder_mode')->evaluate();

                if ($type === 'template_part' && in_array($type, $types) && in_array($technology, $technologies) && $isBuilderMode) {
                    $templatePartPost = $post;
                }
            } else {
                $finalType = $type;
                if ($finalType === null) {
                    if(!isset($_GET['builderius-template-part-type'])) {
                        return null;
                    } else {
                        $finalType = $_GET['builderius-template-part-type'];
                    }
                }
                $applicableTemplatePosts = [];
                $templatePartPosts = $this->templatePostsProvider->getTemplatePartPosts($finalType);
                foreach ($templatePartPosts as $templPost) {
                    $applyRuleConfig = json_decode(
                        $templPost->__get(BuilderiusTemplate::APPLY_RULES_CONFIG_FIELD),
                        true
                    ) ? : [];
                    if ($this->applyRuleChecker->checkApplyRule($applyRuleConfig)) {
                        $applicableTemplatePosts[] = $templPost;
                    }
                }
                if (count($applicableTemplatePosts) > 1) {
                    usort($applicableTemplatePosts, function (\WP_Post $a, \WP_Post $b) {
                        $aSortOrder = (int)$a->__get(BuilderiusTemplate::SORT_ORDER_FIELD);
                        $bSortOrder = (int)$b->__get(BuilderiusTemplate::SORT_ORDER_FIELD);
                        if ($aSortOrder < $bSortOrder) {
                            return -1;
                        } elseif ($aSortOrder > $bSortOrder) {
                            return 1;
                        } elseif ($a->ID < $b->ID) {
                            return -1;
                        } elseif ($a->ID > $b->ID) {
                            return 1;
                        } else {
                            return 0;
                        }
                    });
                }
                $templatePartPost = reset($applicableTemplatePosts);
            }
            if ( false !== $templatePartPost ) {
                if (null !== $type) {
                    $this->cache->set(sprintf('builderius_template_part_post_%s', $type), $templatePartPost);
                } else {
                    $this->cache->set('builderius_template_part_post', $templatePartPost);
                }
            } else {
                if (null !== $type) {
                    $this->cache->set(sprintf('builderius_template_part_post_%s', $type), null);
                } else {
                    $this->cache->set('builderius_template_part_post', null);
                }
            }
        }

        return $templatePartPost;
    }

    /**
     * @inheritDoc
     */
    public function getTemplatePart($type = null)
    {
        if (null !== $type) {
            $templatePart = $this->cache->get(sprintf('builderius_template_part_%s', $type));
        } else {
            $templatePart = $this->cache->get('builderius_template_part');
        }
        if (false === $templatePart) {
            $templatePart = null;
            if ($templatePost = $this->getTemplatePartPost($type)) {
                $templatePart = $this->builderiusTemplateFromPostFactory
                    ->createTemplate($templatePost);
            }
            if (null !== $type) {
                $this->cache->set(sprintf('builderius_template_part_%s', $type), $templatePart);
            } else {
                $this->cache->set('builderius_template_part', $templatePart);
            }
        }

        return $templatePart;
    }
}
