<?php

namespace BuilderiusPro\Bundle\ComponentBundle\Registration;

use Builderius\Bundle\BuilderBundle\Registration\AbstractBuilderiusBuilderScriptLocalization;
use Builderius\Bundle\GraphQLBundle\Executor\BuilderiusEntitiesGraphQLQueriesExecutorInterface;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class BuilderiusSavedFragmentsScriptLocalization extends AbstractBuilderiusBuilderScriptLocalization
{
    const PROPERTY_NAME = 'savedFragments';

    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;

    /**
     * @var BuilderiusEntitiesGraphQLQueriesExecutorInterface
     */
    private $graphQLQueriesExecutor;

    /**
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     * @param BuilderiusEntitiesGraphQLQueriesExecutorInterface $graphQLQueriesExecutor
     */
    public function __construct(
        BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider,
        BuilderiusEntitiesGraphQLQueriesExecutorInterface $graphQLQueriesExecutor
    ) {
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;
        $this->graphQLQueriesExecutor = $graphQLQueriesExecutor;
    }

    /**
     * @inheritDoc
     */
    public function getPropertyData()
    {
        $template = $this->builderiusTemplatePartProvider->getTemplatePart();
        if ($template) {
            $queries = [
                [
                    'name' => 'saved_fragments',
                    'query' => 'query {
                                layouts: saved_fragments(type: layout, technology:' . $template->getTechnology() . ') {
                                    id
                                    name
                                    title
                                    description
                                    static_content_config
                                    dynamic_content_config
                                    category
                                    tags
                                    created_at
                                    updated_at
                                    image
                                    author {
                                        nickname
                                    }
                                }
                            }'
                ]
            ];
            $results = $this->graphQLQueriesExecutor->execute($queries);

            $data = $results['saved_fragments']['data'];
            foreach ($data as $type => $groupedItems) {
                foreach ($groupedItems as $i => $item) {
                    if($data[$type][$i]['image'] == null) {
                        $data[$type][$i]['image'] = WP_PLUGIN_URL . '/builderius/assets/img/no-image-available.jpeg';
                    }
                }
            }

            return $data;
        }

        return [];
    }
}
