<?php

namespace BuilderiusPro\Bundle\ComponentBundle\Registration;

use Builderius\Bundle\TemplateBundle\ApplyRule\BuilderiusTemplateApplyRuleInterface;
use Builderius\Bundle\TemplateBundle\ApplyRule\Registry\BuilderiusTemplateApplyRulesRegistryInterface;
use Builderius\Bundle\TemplateBundle\Registration\BuilderiusTemplatePossibleApplyRulesScriptLocalization;

class BuilderiusTemplatePartPossibleApplyRulesScriptLocalization extends BuilderiusTemplatePossibleApplyRulesScriptLocalization
{
    const PROPERTY_NAME = 'possibleApplyRules';

    /**
     * @var BuilderiusTemplateApplyRulesRegistryInterface
     */
    private $templateApplyRulesRegistry;

    /**
     * @param BuilderiusTemplateApplyRulesRegistryInterface $templateApplyRulesRegistry
     */
    public function __construct(
        BuilderiusTemplateApplyRulesRegistryInterface $templateApplyRulesRegistry
    ) {
        $this->templateApplyRulesRegistry = $templateApplyRulesRegistry;
    }

    /**
     * @inheritDoc
     */
    public function getPropertyData()
    {
        $result = [];
        foreach ($this->templateApplyRulesRegistry->getRules() as $category => $rulesByNames) {
            /** @var BuilderiusTemplateApplyRuleInterface $rule */
            foreach ($rulesByNames as $rule) {
                if (!$rule->getParent() && $this->ruleHasTemplatePartType($rule)) {
                    $data = $this->convertRule($rule);
                    foreach ($data as $name => $config) {
                        $result[$category][$name] = $config;
                    }
                }
            }
        }
        
        return $result;
    }

    /**
     * @param BuilderiusTemplateApplyRuleInterface $rule
     * @return bool
     */
    private function ruleHasTemplatePartType(BuilderiusTemplateApplyRuleInterface $rule)
    {
        if ($rule->isAppliedToAllTemplateTypes()) {
            $result = true;
            if (!empty($rule->getExcludedTemplateTypes())) {
                if (in_array('template_part', $rule->getExcludedTemplateTypes())) {
                    $result = false;
                }
            }

            return $result;
        } else {
            $result = false;
            if (in_array('template_part', $rule->getTemplateTypes())) {
                $result = true;
            }

            return $result;
        }
    }
}
