<?php

namespace BuilderiusPro\Bundle\ComponentBundle\Registration;

use Builderius\Bundle\BuilderBundle\Registration\AbstractBuilderiusBuilderScriptLocalization;
use Builderius\Bundle\GraphQLBundle\Executor\BuilderiusEntitiesGraphQLQueriesExecutorInterface;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class BuilderiusTemplatePartPossibleTemplatePartTypesScriptLocalization extends AbstractBuilderiusBuilderScriptLocalization
{
    const PROPERTY_NAME = 'possibleTemplatePartTypes';

    /**
     * @var BuilderiusEntitiesGraphQLQueriesExecutorInterface
     */
    private $graphQLQueriesExecutor;

    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $templatePartProvider;

    /**
     * @param BuilderiusEntitiesGraphQLQueriesExecutorInterface $graphQLQueriesExecutor
     * @param BuilderiusTemplatePartProviderInterface $templatePartProvider
     */
    public function __construct(
        BuilderiusEntitiesGraphQLQueriesExecutorInterface $graphQLQueriesExecutor,
        BuilderiusTemplatePartProviderInterface $templatePartProvider
    ) {
        $this->graphQLQueriesExecutor = $graphQLQueriesExecutor;
        $this->templatePartProvider = $templatePartProvider;
    }

    /**
     * @inheritDoc
     */
    public function getPropertyData()
    {
        $queries = [
            [
                'name' => 'templateParts',
                'query' => 'query {
                            templateParts: templates(type: template_part) {
                                type: sub_type
                            }
                        }'
            ]
        ];
        $results = $this->graphQLQueriesExecutor->execute($queries);

        $results = $results['templateParts']['data']['templateParts'];
        $data = [
            'header' => 0,
            'footer' => 0
        ];
        $currentTemplatePartType = $this->templatePartProvider->getTemplatePart()->getSubType();
        foreach ($results as $result) {
            if ($result['type'] !== $currentTemplatePartType) {
                if (!isset($data[$result['type']])) {
                    $data[$result['type']] = 1;
                } else {
                    $data[$result['type']] += 1;
                }
            }
        }

        return $data;
    }
}
