<?php

namespace BuilderiusPro\Bundle\ComponentBundle\Registration;

use Builderius\Bundle\BuilderBundle\Registration\AbstractBuilderiusBuilderScriptLocalization;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicant;
use Builderius\Bundle\TemplateBundle\ApplyRule\Category\BuilderiusTemplateApplicantParametersAwareApplyRuleCategoryInterface;
use Builderius\Bundle\TemplateBundle\ApplyRule\Category\BuilderiusTemplateApplicantsAwareApplyRuleCategoryInterface;
use Builderius\Bundle\TemplateBundle\ApplyRule\Category\Registry\BuilderiusTemplateApplyRuleCategoriesRegistryInterface;
use Builderius\Bundle\TemplateBundle\Model\BuilderiusTemplate;
use Builderius\Bundle\TemplateBundle\Provider\Applicant\BuilderiusTemplateApplicantsProviderInterface;
use Builderius\Bundle\TemplateBundle\Factory\BuilderiusTemplateFromPostFactory;
use Builderius\Bundle\TemplateBundle\Registration\BuilderiusTemplatePostType;
use Builderius\Bundle\TemplateBundle\Registration\BuilderiusTemplateSubTypeTaxonomy;
use Builderius\Bundle\TemplateBundle\Registration\BuilderiusTemplateTechnologyTaxonomy;
use Builderius\Bundle\TemplateBundle\Registration\BuilderiusTemplateTypeTaxonomy;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class BuilderiusTemplatePossibleApplicantsScriptLocalization extends AbstractBuilderiusBuilderScriptLocalization
{
    const PROPERTY_NAME = 'applicants';

    /**
     * @var BuilderiusTemplateFromPostFactory
     */
    private $builderiusTemplateFromPostFactory;

    /**
     * @var BuilderiusTemplateApplicantsProviderInterface
     */
    private $applicantsProvider;

    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @var BuilderiusTemplateApplyRuleCategoriesRegistryInterface
     */
    private $applyRuleCategoriesRegistry;

    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;

    /**
     * @param BuilderiusTemplateFromPostFactory $builderiusTemplateFromPostFactory
     * @param BuilderiusTemplateApplicantsProviderInterface $applicantsProvider
     * @param \WP_Query $wpQuery
     * @param BuilderiusTemplateApplyRuleCategoriesRegistryInterface $applyRuleCategoriesRegistry
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     */
    public function __construct(
        BuilderiusTemplateFromPostFactory $builderiusTemplateFromPostFactory,
        BuilderiusTemplateApplicantsProviderInterface $applicantsProvider,
        \WP_Query $wpQuery,
        BuilderiusTemplateApplyRuleCategoriesRegistryInterface $applyRuleCategoriesRegistry,
        BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
    ) {
        $this->builderiusTemplateFromPostFactory = $builderiusTemplateFromPostFactory;
        $this->applicantsProvider = $applicantsProvider;
        $this->wpQuery = $wpQuery;
        $this->applyRuleCategoriesRegistry = $applyRuleCategoriesRegistry;
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;
    }

    /**
     * @inheritDoc
     */
    public function getPropertyData()
    {
        $template = $this->builderiusTemplatePartProvider->getTemplatePart();
        if ($template) {
            $id = $template->getId();
            $type = $template->getSubType();
            $technology = $template->getTechnology();
            $sortOrder = $template->getSortOrder();

            $applicants = $this->applicantsProvider->getApplicants($template->getApplyRulesConfig());
            if (!empty($applicants)) {
                foreach($applicants as $applicant) {
                    $applicant->setData($this->getAdditionalCategoriesParameters($template->getApplyRulesConfig()));
                }
                $templatePostsWithHigherPriority = $this->wpQuery->query([
                    'post_type' => BuilderiusTemplatePostType::POST_TYPE,
                    'post_status' => get_post_stati(),
                    'post__not_in' => [$id],
                    'posts_per_page' => -1,
                    'no_found_rows' => true,
                    'meta_key' => BuilderiusTemplate::SORT_ORDER_FIELD,
                    'meta_value_num' => $sortOrder,
                    'meta_compare' => '<=',
                    'tax_query' => [
                        'relation' => 'AND',
                        [
                            'taxonomy' => BuilderiusTemplateTypeTaxonomy::NAME,
                            'field' => 'slug',
                            'terms' => ['template_part'],
                            'operator' => 'IN'
                        ],
                        [
                            'taxonomy' => BuilderiusTemplateSubTypeTaxonomy::NAME,
                            'field' => 'slug',
                            'terms' => [$type],
                            'operator' => 'IN'
                        ],
                        [
                            'taxonomy' => BuilderiusTemplateTechnologyTaxonomy::NAME,
                            'field' => 'slug',
                            'terms' => [$technology],
                            'operator' => 'IN'
                        ]
                    ]
                ]);
                $excludeApplicants = [];
                if (!empty($templatePostsWithHigherPriority)) {
                    foreach ($templatePostsWithHigherPriority as $templatePostWithHigherPriority) {
                        $templateWithHigherPriority = $this->builderiusTemplateFromPostFactory->createTemplate($templatePostWithHigherPriority);
                        if (
                            $templateWithHigherPriority->getSortOrder() < $sortOrder ||
                            ($templateWithHigherPriority->getSortOrder() == $sortOrder && $templateWithHigherPriority->getId() < $template->getId())
                        ) {

                            try {
                                $iterationExclApplicants = $this->applicantsProvider->getApplicants($templateWithHigherPriority->getApplyRulesConfig());
                                foreach($iterationExclApplicants as $iterationExclApplicant) {
                                    $iterationExclApplicant->setData($this->getAdditionalCategoriesParameters($templateWithHigherPriority->getApplyRulesConfig()));
                                }
                                $excludeApplicants = array_merge(
                                    $excludeApplicants,
                                    $iterationExclApplicants
                                );
                            } catch (\Exception $e) {
                                continue;
                            }
                        }
                    }
                }


                $data = [];
                foreach ($applicants as $k => $applicant) {
                    if (!in_array($applicant, $excludeApplicants)) {
                        $data[$applicant->getCategory() ?: 'other'][$k] = [
                            BuilderiusTemplateApplicant::LABEL_FIELD => __($applicant->getLabel()),
                            BuilderiusTemplateApplicant::URL_FIELD => $applicant->getUrl(),
                            BuilderiusTemplateApplicant::PARAMETERS_FIELD => $applicant->getParameters(),
                            BuilderiusTemplateApplicant::CATEGORY_FIELD => __($applicant->getCategory()),
                            BuilderiusTemplateApplicant::SORT_ORDER_FIELD => $applicant->getSortOrder()
                        ];
                    }
                }

                return $data;
            }
        }

        return [];
    }

    /**
     * @param array $applyRulesConfig
     * @return array
     */
    private function getAdditionalCategoriesParameters(array $applyRulesConfig)
    {
        $additionalCategoriesParams = [];
        foreach ($applyRulesConfig['categories'] as $categoryName => $configSet) {
            $category = $this->applyRuleCategoriesRegistry->getCategory($categoryName);
            if (
                $category &&
                !$category instanceof BuilderiusTemplateApplicantsAwareApplyRuleCategoryInterface &&
                !$category instanceof BuilderiusTemplateApplicantParametersAwareApplyRuleCategoryInterface
            ) {
                $additionalCategoriesParams[$categoryName] = $configSet;
            }
        }

        return $additionalCategoriesParams;
    }
}