<?php

namespace BuilderiusPro\Bundle\GlobalFragmentBundle\Factory;

use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\VCSBundle\Factory\BuilderiusBranchFromPostFactory;
use Builderius\Bundle\VCSBundle\Model\BuilderiusBranchInterface;
use Builderius\Bundle\VCSBundle\Registration\BuilderiusBranchPostType;
use BuilderiusPro\Bundle\GlobalFragmentBundle\Model\BuilderiusGlobalFragment;
use BuilderiusPro\Bundle\GlobalFragmentBundle\Model\BuilderiusGlobalFragmentInterface;

class BuilderiusGlobalFragmentFromPostFactory
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @var BuilderiusBranchFromPostFactory
     */
    private $branchFromPostFactory;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    private $cache;

    /**
     * @param \WP_Query $wpQuery
     * @param BuilderiusBranchFromPostFactory $branchFromPostFactory
     * @param BuilderiusRuntimeObjectCache $cache
     */
    public function __construct(
        \WP_Query $wpQuery,
        BuilderiusBranchFromPostFactory $branchFromPostFactory,
        BuilderiusRuntimeObjectCache $cache
    ) {
        $this->wpQuery = $wpQuery;
        $this->branchFromPostFactory = $branchFromPostFactory;
        $this->cache = $cache;
    }

    /**
     * @param \WP_Post $post
     * @return BuilderiusGlobalFragmentInterface|null
     */
    public function createGlobalSettingsSet(\WP_Post $post)
    {
        return new BuilderiusGlobalFragment([
            BuilderiusGlobalFragment::ID_FIELD => $post->ID,
            BuilderiusGlobalFragment::NAME_FIELD => $post->post_name,
            BuilderiusGlobalFragment::TITLE_FIELD => $post->post_title,
            BuilderiusGlobalFragment::ENTITY_TYPE_FIELD => 'global_fragment',
            BuilderiusGlobalFragment::AUTHOR_FIELD => get_user_by('ID', $post->post_author),
            BuilderiusGlobalFragment::CREATED_AT_FIELD => $post->post_date,
            BuilderiusGlobalFragment::UPDATED_AT_FIELD => $post->post_modified,
            BuilderiusGlobalFragment::ACTIVE_BRANCH_NAME_FIELD => function () use ($post) {
                $currUserId = apply_filters('builderius_get_current_user', wp_get_current_user())->ID;
                $activeBranchNameString = get_post_meta($post->ID, BuilderiusGlobalFragment::ACTIVE_BRANCH_NAME_FIELD, true);
                $activeBranchName = 'master';
                $activeBranchNameJson = json_decode($activeBranchNameString, true);
                if (json_last_error() === JSON_ERROR_NONE) {
                    if (isset($activeBranchNameJson[$currUserId])) {
                        $activeBranchName = $activeBranchNameJson[$currUserId];
                    }
                } elseif ($activeBranchNameString) {
                    $activeBranchName = $activeBranchNameString;
                }

                return $activeBranchName;
            },
            BuilderiusGlobalFragment::TYPE_FIELD => $typeTechArr[1],
            BuilderiusGlobalFragment::TECHNOLOGY_FIELD => $typeTechArr[0],
            BuilderiusGlobalFragment::BRANCHES_FIELD => function () use ($post) {
                $branches = $this->cache->get(sprintf('builderius_global_fragment_%s_branches', $post->ID));
                if (false === $branches) {
                    $branches = [];
                    $branchesPosts = $this->cache->get(sprintf('builderius_global_fragment_%s_branches_posts', $post->ID));
                    if (false === $branchesPosts) {
                        $branchesPosts = $this->wpQuery->query([
                            'post_type' => BuilderiusBranchPostType::POST_TYPE,
                            'post_parent' => $post->ID,
                            'post_status' => get_post_stati(),
                            'posts_per_page' => -1,
                            'orderby' => 'ID',
                            'order' => 'DESC'
                        ]);
                        $this->cache->set(sprintf('builderius_global_fragment_%s_branches_posts', $post->ID), $branchesPosts);
                    }
                    /** @var BuilderiusBranchInterface[] $branches */
                    foreach ($branchesPosts as $branchPost) {
                        if (false === $this->cache->get(sprintf('builderius_branch_post_%s', $branchPost->ID))) {
                            $this->cache->set(sprintf('builderius_branch_post_%s', $branchPost->ID), $branchPost);
                        }
                        $branch = $this->branchFromPostFactory->createBranch($branchPost);
                        $this->cache->set(sprintf('builderius_branch_%s', $branchPost->ID), $branch);
                        $branches[$branchPost->post_name] = $branch;
                    }
                    $this->cache->set(sprintf('builderius_global_fragment_%s_branches', $post->ID), $branches);
                }

                return $branches;
            }
        ]);
    }
}