<?php

namespace BuilderiusPro\Bundle\GlobalFragmentBundle\Registration;

use Builderius\Bundle\BuilderBundle\Registration\AbstractBuilderiusBuilderScriptLocalization;
use Builderius\Bundle\GraphQLBundle\Executor\BuilderiusEntitiesGraphQLQueriesExecutorInterface;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplateProviderInterface;

class BuilderiusGlobalFragmentsScriptLocalization extends AbstractBuilderiusBuilderScriptLocalization
{
    const PROPERTY_NAME = 'globalFragments';

    /**
     * @var BuilderiusTemplateProviderInterface
     */
    private $builderiusTemplateProvider;

    /**
     * @var BuilderiusEntitiesGraphQLQueriesExecutorInterface
     */
    private $graphQLQueriesExecutor;

    /**
     * @param BuilderiusTemplateProviderInterface $builderiusTemplateProvider
     * @param BuilderiusEntitiesGraphQLQueriesExecutorInterface $graphQLQueriesExecutor
     */
    public function __construct(
        BuilderiusTemplateProviderInterface $builderiusTemplateProvider,
        BuilderiusEntitiesGraphQLQueriesExecutorInterface $graphQLQueriesExecutor
    ) {
        $this->builderiusTemplateProvider = $builderiusTemplateProvider;
        $this->graphQLQueriesExecutor = $graphQLQueriesExecutor;
    }

    /**
     * @inheritDoc
     */
    public function getPropertyData()
    {
        $template = $this->builderiusTemplateProvider->getTemplate();
        if ($template) {
            $queries = [
                [
                    'name' => 'global_fragments',
                    'query' => 'query {
                                modules: saved_fragments(type: module, technology:' . $template->getTechnology() . ') {
                                    id
                                    name
                                    title
                                    content_config
                                    category
                                    tags
                                    created_at
                                    updated_at
                                    image
                                    author {
                                        nickname
                                    }
                                }
                                layouts: saved_fragments(type: layout, technology:' . $template->getTechnology() . ') {
                                    id
                                    name
                                    title
                                    content_config
                                    category
                                    tags
                                    created_at
                                    updated_at
                                    image
                                    author {
                                        nickname
                                    }
                                }
                            }'
                ]
            ];
            $results = $this->graphQLQueriesExecutor->execute($queries);

            return $results['global_fragments']['data'];
        }

        return [];
    }
}
