<?php


namespace BuilderiusPro\Bundle\GraphQLBundle\Resolver;

use Builderius\Bundle\ExpressionLanguageBundle\ExpressionLanguage;
use Builderius\Bundle\GraphQLBundle\Resolver\AbstractLocalVarsAwareResolver;
use Builderius\GraphQL\Cache\GraphQLObjectCache;
use Builderius\GraphQL\Type\Definition\ResolveInfo;

abstract class AbstractPostsResolver extends AbstractLocalVarsAwareResolver
{
    /**
     * @var array
     */
    protected $typeNames;

    /**
     * @var GraphQLObjectCache
     */
    protected $graphqlFacetsCache;

    /**
     * @param array $typeNames
     * @param GraphQLObjectCache $graphqlSubfieldsCache
     * @param GraphQLObjectCache $graphqlFacetsCache
     * @param ExpressionLanguage $expressionLanguage
     */
    public function __construct(
        array $typeNames,
        GraphQLObjectCache $graphqlSubfieldsCache,
        GraphQLObjectCache $graphqlFacetsCache,
        ExpressionLanguage $expressionLanguage
    ) {
        $this->typeNames = $typeNames;
        $this->graphqlFacetsCache = $graphqlFacetsCache;
        parent::__construct($graphqlSubfieldsCache, $expressionLanguage);
    }

    /**
     * @inheritDoc
     */
    public function getTypeNames()
    {
        return $this->typeNames;
    }

    /**
     * @inheritDoc
     */
    public function getSortOrder()
    {
        return 10;
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($objectValue, array $args, $context, ResolveInfo $info)
    {
        return true;
    }

    /**
     * @param array $args
     * @param array $path
     * @return array|mixed
     * @throws \Exception
     */
    protected function processQueryArgs(array $args, array $path)
    {
        $args = $this->processArguments($args, $path);
        $queryArgs = isset($args['query']) ? $args['query'] : [];
        if (isset($queryArgs['meta_query']) && isset($queryArgs['meta_query']['array'])) {
            $metaQuery = [];
            foreach ($queryArgs['meta_query']['array'] as $metaQueryItem) {
                if (isset($metaQueryItem['condition'])) {
                    if ($metaQueryItem['condition'] == true) {
                        unset($metaQueryItem['condition']);
                        $metaQuery[] = $metaQueryItem;
                    }
                } else {
                    $metaQuery[] = $metaQueryItem;
                }
            }
            if (!empty($metaQuery)) {
                if (isset($queryArgs['meta_query']['relation'])) {
                    $metaQuery['relation'] = $queryArgs['meta_query']['relation'];
                }
                $queryArgs['meta_query'] = $metaQuery;
            }
        }
        if (isset($queryArgs['tax_query']) && isset($queryArgs['tax_query']['array'])) {
            $taxQuery = [];
            foreach ($queryArgs['tax_query']['array'] as $taxQueryItem) {
                if (isset($taxQueryItem['condition'])) {
                    if ($taxQueryItem['condition'] == true) {
                        unset($taxQueryItem['condition']);
                        $taxQuery[] = $taxQueryItem;
                    }
                } else {
                    $taxQuery[] = $taxQueryItem;
                }
            }
            if (!empty($taxQuery)) {
                if (isset($queryArgs['tax_query']['relation'])) {
                    $taxQuery['relation'] = $queryArgs['tax_query']['relation'];
                }
                $queryArgs['tax_query'] = $taxQuery;
            }
        }
        if (isset($queryArgs['date_query']) && isset($queryArgs['date_query']['array'])) {
            $dateQuery = [];
            foreach ($queryArgs['date_query']['array'] as $dateQueryItem) {
                if (isset($dateQueryItem['condition'])) {
                    if ($dateQueryItem['condition'] == true) {
                        unset($dateQueryItem['condition']);
                        $dateQuery[] = $dateQueryItem;
                    }
                } else {
                    $dateQuery[] = $dateQueryItem;
                }
            }
            if (!empty($dateQuery)) {
                if (isset($queryArgs['date_query']['relation'])) {
                    $dateQuery['relation'] = $queryArgs['date_query']['relation'];
                }
                if (isset($queryArgs['date_query']['column'])) {
                    $dateQuery['column'] = $queryArgs['date_query']['column'];
                }
                if (isset($queryArgs['date_query']['compare'])) {
                    $dateQuery['compare'] = $queryArgs['date_query']['compare'];
                }
                $queryArgs['date_query'] = $dateQuery;
            }
        }
        if (isset($args['facets'])) {
            $this->graphqlFacetsCache->set('facets_args', $args['facets']);
            $submittedFacets = [];
            $facetsArgs = $args['facets'];
            $request = null;
            if ($facetsArgs['submit_method'] === 'GET') {
                $request= $_GET;
            } elseif ($facetsArgs['submit_method'] === 'POST') {
                $request= $_POST;
            }
            if (null !== $request) {
                if (isset($facetsArgs['wp_query'])) {
                    foreach ($facetsArgs['wp_query'] as $wpQueryFacet) {

                        if (isset($request[$wpQueryFacet['facet_name']]) && !in_array($request[$wpQueryFacet['facet_name']], [null, ''])) {
                            if ((isset($wpQueryFacet['condition']) && $wpQueryFacet['condition'] == true) || !isset($wpQueryFacet['condition'])) {
                                $submittedFacets[$wpQueryFacet['facet_name']] = $request[$wpQueryFacet['facet_name']];
                                if (isset($wpQueryFacet['fixed_query_arguments'])) {
                                    foreach ($wpQueryFacet['fixed_query_arguments'] as $faKey => $faValue) {
                                        $queryArgs[$faKey] = $faValue;
                                    }
                                }
                                $queryArgs[$wpQueryFacet['query_argument']] = $request[$wpQueryFacet['facet_name']];
                            }
                        }
                    }
                }
                if (isset($facetsArgs['wp_meta_query'])) {
                    if (
                        !isset($queryArgs['meta_query']) ||
                        (isset($queryArgs['meta_query']) && $queryArgs['meta_query']['relation'] === $facetsArgs['wp_meta_query']['relation'])
                    ) {
                        foreach ($facetsArgs['wp_meta_query']['array'] as $metaQueryFacet) {
                            if (isset($request[$metaQueryFacet['facet_name']]) && !in_array($request[$metaQueryFacet['facet_name']], [null, ''])) {
                                if ((isset($metaQueryFacet['condition']) && $metaQueryFacet['condition'] == true) || !isset($metaQueryFacet['condition'])) {
                                    $submittedFacets[$metaQueryFacet['facet_name']] = $request[$metaQueryFacet['facet_name']];
                                    $metaQueryArgs = [];
                                    if (isset($metaQueryFacet['fixed_query_arguments'])) {
                                        foreach ($metaQueryFacet['fixed_query_arguments'] as $faKey => $faValue) {
                                            $metaQueryArgs[$faKey] = $faValue;
                                        }
                                    }
                                    if (isset($metaQueryFacet['query_argument'])) {
                                        $metaQueryArgs[$metaQueryFacet['query_argument']] = $request[$metaQueryFacet['facet_name']];
                                    }
                                    $queryArgs['meta_query'][] = $metaQueryArgs;
                                }
                            }
                        }
                        if (isset($queryArgs['meta_query'])) {
                            if (isset($facetsArgs['wp_meta_query']['relation']) && !empty($queryArgs['meta_query'])) {
                                $queryArgs['meta_query']['relation'] = $facetsArgs['wp_meta_query']['relation'];
                            } else {
                                unset($queryArgs['meta_query']);
                            }
                        }
                    } else {
                        throw new \Exception('Facets meta_query relation should be same as main meta_query relation');
                    }
                }
                if (isset($facetsArgs['wp_tax_query'])) {
                    if (
                        !isset($queryArgs['tax_query']) ||
                        (isset($queryArgs['tax_query']) && $queryArgs['tax_query']['relation'] === $facetsArgs['wp_tax_query']['relation'])
                    ) {
                        foreach ($facetsArgs['wp_tax_query']['array'] as $taxQueryFacet) {
                            if (isset($request[$taxQueryFacet['facet_name']]) && !in_array($request[$taxQueryFacet['facet_name']], [null, ''])) {
                                if ((isset($taxQueryFacet['condition']) && $taxQueryFacet['condition'] == true) || !isset($taxQueryFacet['condition'])) {
                                    $submittedFacets[$taxQueryFacet['facet_name']] = $request[$taxQueryFacet['facet_name']];
                                    $taxQueryArgs = [];
                                    if (isset($taxQueryFacet['fixed_query_arguments'])) {
                                        foreach ($taxQueryFacet['fixed_query_arguments'] as $faKey => $faValue) {
                                            $taxQueryArgs[$faKey] = $faValue;
                                        }
                                    }
                                    $taxQueryArgs[$taxQueryFacet['query_argument']] = $request[$taxQueryFacet['facet_name']];
                                    $queryArgs['tax_query'][] = $taxQueryArgs;
                                }
                            }
                        }
                        if (isset($queryArgs['tax_query'])) {
                            if (isset($facetsArgs['wp_tax_query']['relation']) && !empty($queryArgs['tax_query'])) {
                                $queryArgs['tax_query']['relation'] = $facetsArgs['wp_tax_query']['relation'];
                            } else {
                                unset($queryArgs['tax_query']);
                            }
                        }
                    } else {
                        throw new \Exception('Facets tax_query relation should be same as main tax_query relation');
                    }
                }
                if (isset($facetsArgs['wp_date_query'])) {
                    if (
                        !isset($queryArgs['date_query']) ||
                        (isset($queryArgs['date_query']) && $queryArgs['date_query']['relation'] === $facetsArgs['wp_date_query']['relation'])
                    ) {
                        foreach ($facetsArgs['wp_date_query']['array'] as $dateQueryFacet) {
                            if (isset($request[$dateQueryFacet['facet_name']]) && !in_array($request[$dateQueryFacet['facet_name']], [null, ''])) {
                                if ((isset($dateQueryFacet['condition']) && $dateQueryFacet['condition'] == true) || !isset($dateQueryFacet['condition'])) {
                                    $submittedFacets[$dateQueryFacet['facet_name']] = $request[$dateQueryFacet['facet_name']];
                                    $dateQueryArgs = [];
                                    if (isset($dateQueryFacet['fixed_query_arguments'])) {
                                        foreach ($dateQueryFacet['fixed_query_arguments'] as $faKey => $faValue) {
                                            $dateQueryArgs[$faKey] = $faValue;
                                        }
                                    }
                                    if (isset($dateQueryFacet['query_argument'])) {
                                        $dateQueryArgs[$dateQueryFacet['query_argument']] = $request[$dateQueryFacet['facet_name']];
                                    }
                                    $queryArgs['date_query'][] = $dateQueryArgs;
                                }
                            }
                        }
                        if (isset($queryArgs['date_query'])) {
                            if (!empty($queryArgs['date_query'])) {
                                if (isset($facetsArgs['wp_date_query']['relation'])) {
                                    $queryArgs['date_query']['relation'] = $facetsArgs['wp_date_query']['relation'];
                                }
                                if (isset($facetsArgs['wp_date_query']['column'])) {
                                    $queryArgs['date_query']['column'] = $facetsArgs['wp_date_query']['column'];
                                }
                                if (isset($facetsArgs['wp_date_query']['compare'])) {
                                    $queryArgs['date_query']['compare'] = $facetsArgs['wp_date_query']['compare'];
                                }
                            } else {
                                unset($queryArgs['date_query']);
                            }
                        }
                    } else {
                        throw new \Exception('Facets meta_query relation should be same as main meta_query relation');
                    }
                }
            }
            $this->graphqlFacetsCache->set('submitted_facets', $submittedFacets);
        }

        return $queryArgs;
    }
}