<?php

namespace BuilderiusPro\Bundle\GraphQLBundle\Resolver;

use Builderius\Bundle\ExpressionLanguageBundle\ExpressionLanguage;
use Builderius\Bundle\GraphQLBundle\Helper\GraphQLHelper;
use Builderius\Bundle\GraphQLBundle\Resolver\AbstractLocalVarsAwareResolver;
use Builderius\GraphQL\Cache\GraphQLObjectCache;
use Builderius\GraphQL\Type\Definition\ResolveInfo;
use Builderius\Symfony\Component\PropertyAccess\PropertyAccess;

class ExpressionResultResolver extends AbstractLocalVarsAwareResolver
{
    /**
     * @param GraphQLObjectCache $graphqlSubfieldsCache
     * @param ExpressionLanguage $expressionLanguage
     */
    public function __construct(
        GraphQLObjectCache $graphqlSubfieldsCache,
        ExpressionLanguage $expressionLanguage
    ) {
        parent::__construct($graphqlSubfieldsCache, $expressionLanguage);
    }

    /**
     * @inheritDoc
     */
    public function getTypeNames()
    {
        return [];
    }

    /**
     * @inheritDoc
     */
    public function getSortOrder()
    {
        return 10;
    }

    /**
     * @inheritDoc
     */
    public function getFieldName()
    {
        return 'expression_result';
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($objectValue, array $args, $context, ResolveInfo $info)
    {
        return true;
    }

    /**
     * @inheritDoc
     */
    public function resolve($objectValue, array $args, $context, ResolveInfo $info)
    {
        $args = $this->processExpression($args, $info);

        return $args['expression'];
    }

    /**
     * @param array $args
     * @param array $path
     * @return array
     */
    protected function processExpression(array $args, ResolveInfo $info)
    {
        if (!empty($args)) {
            $res = $this->graphqlSubfieldsCache->get('results');
            if (false !== $res) {
                $path = $info->path;
                $selectionSet = $info->operation->toArray(true)['selectionSet'];
                $privacyData = $this->processSelectionSet($selectionSet, $path, $args['expression']);
                $data = $this->processDataLevel($res, $privacyData);
                array_pop($path);
                if (!empty($path)) {
                    $propertyAccessor = PropertyAccess::createPropertyAccessor();
                    $context = $propertyAccessor->getValue($data, sprintf('[%s]', implode('][', $path)));
                } else {
                    $context = $data;
                }
                $args = GraphQLHelper::array_map_deep($args, [$this, 'processLocalVars'], $context);
            }
        }

        return $args;
    }

    /**
     * @param array $data
     * @param array $privacyData
     * @return array
     */
    private function processDataLevel(array $data, array $privacyData)
    {
        foreach ($data as $key => $value) {
            if (isset($privacyData[$key])) {
                $isPrivate = $privacyData[$key];
                if (true === $isPrivate) {
                    unset($data[$key]);
                } else {
                    if (is_array($value) && is_array($privacyData[$key])) {
                        if (!isset($data[$key][0])) {
                            $data[$key] = $this->processDataLevel($data[$key], $privacyData[$key]);
                        } else {
                            foreach ($data[$key] as $k => $item) {
                                $data[$key][$k] = $this->processDataLevel($data[$key][$k], $privacyData[$key]);
                            }
                        }
                    }
                }
            }
        }

        return $data;
    }

    /**
     * @param array $selectionSet
     * @param array $path
     * @param string $expression
     * @return array
     */
    private function processSelectionSet(array $selectionSet, array $path, $expression)
    {
        $result = [];
        if (isset($selectionSet['selections'])) {
            foreach ($selectionSet['selections'] as $selection) {
                $name = (isset($selection['alias']) ? $selection['alias'] : $selection['name'])['value'];
                if (in_array($name, $path, true) || strpos($expression, $name) !== false) {
                    $isPrivate = false;
                } else {
                    $isPrivate = isset($selection['arguments']) ? $this->isPrivate($selection['arguments']) : false;
                }
                if ($isPrivate === false && isset($selection['selectionSet'])) {
                    $result[$name] = $this->processSelectionSet($selection['selectionSet'], $path, $expression);
                } else {
                    $result[$name] = $isPrivate;
                }
            }
        }
        return $result;
    }

    /**
     * @param $arguments
     * @return bool
     */
    private function isPrivate($arguments)
    {
        $isPrivate = false;
        foreach ($arguments as $argument) {
            if ($argument['name']['value'] === 'private' && $argument['value']['value'] == true) {
                $isPrivate = true;
                break;
            }
        }

        return $isPrivate;
    }
}