<?php


namespace BuilderiusPro\Bundle\GraphQLBundle\Resolver;

use Builderius\Bundle\GraphQLBundle\Resolver\AbstractLocalVarsAwareResolver;
use Builderius\GraphQL\Type\Definition\ResolveInfo;

class NavMenuItemsResolver extends AbstractLocalVarsAwareResolver
{
    public function getTypeNames()
    {
        return ['NavMenu'];
    }

    public function getSortOrder()
    {
        return 10;
    }

    public function isApplicable($objectValue, array $args, $context, ResolveInfo $info)
    {
        return true;
    }

    /**
     * @inheritDoc
     */
    public function getFieldName()
    {
        return 'items';
    }

    /**
     * @inheritDoc
     */
    public function resolve($objectValue, array $args, $context, ResolveInfo $info)
    {
        try{
            $items = wp_get_nav_menu_items($objectValue->term_id);
            $final = [];
            foreach ($items as $item) {
                if ($item->menu_item_parent == "0") {
                    $final[] = $this->processItem($item, $items, $objectValue);
                }
            }

            return $final;
        } catch (\Exception $e) {
            return [];
        }
    }

    /**
     * @param $item
     * @param array $items
     * @param $objectValue
     * @return mixed
     */
    private function processItem($item, array $items, $objectValue)
    {
        $item->children = [];
        $item->nav_menu = $objectValue;
        foreach ($items as $child) {
            if ($child->menu_item_parent == $item->db_id) {
                $child->parent = $item;
                $item->children[] = $this->processItem($child, $items, $objectValue);
            }
        }

        return $item;
    }
}