<?php

namespace BuilderiusPro\Bundle\GraphQLBundle\Resolver;

use Builderius\Bundle\ExpressionLanguageBundle\ExpressionLanguage;
use Builderius\Bundle\GraphQLBundle\Resolver\AbstractLocalVarsAwareResolver;
use Builderius\GraphQL\Cache\GraphQLObjectCache;
use Builderius\GraphQL\Type\Definition\ResolveInfo;

class NavMenusResolver extends AbstractLocalVarsAwareResolver
{
    /**
     * @var array
     */
    protected $typeNames;

    /**
     * @param array $typeNames
     * @param GraphQLObjectCache $graphqlSubfieldsCache
     * @param ExpressionLanguage $expressionLanguage
     */
    public function __construct(
        array $typeNames,
        GraphQLObjectCache $graphqlSubfieldsCache,
        ExpressionLanguage $expressionLanguage
    ) {
        $this->typeNames = $typeNames;
        parent::__construct($graphqlSubfieldsCache, $expressionLanguage);
    }

    /**
     * @inheritDoc
     */
    public function getTypeNames()
    {
        return $this->typeNames;
    }

    /**
     * @inheritDoc
     */
    public function getSortOrder()
    {
        return 10;
    }

    /**
     * @inheritDoc
     */
    public function getFieldName()
    {
        return 'nav_menus';
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($objectValue, array $args, $context, ResolveInfo $info)
    {
        return true;
    }

    /**
     * @inheritDoc
     */
    public function resolve($objectValue, array $args, $context, ResolveInfo $info)
    {
        $args = isset($args['query']) ? $args['query'] : [];
        if (!empty($args)) {
            $args = $this->processArguments($args, $info->path);
            if (isset($args['meta_query']) && isset($args['meta_query']['array'])) {
                $metaQuery = [];
                foreach ($args['meta_query']['array'] as $metaQueryItem) {
                    if (isset($metaQueryItem['condition'])) {
                        if ($metaQueryItem['condition'] == true) {
                            unset($metaQueryItem['condition']);
                            $metaQuery[] = $metaQueryItem;
                        }
                    } else {
                        $metaQuery[] = $metaQueryItem;
                    }
                }
                if (!empty($metaQuery)) {
                    if (isset($args['meta_query']['relation'])) {
                        $metaQuery['relation'] = $args['meta_query']['relation'];
                    }
                    $args['meta_query'] = $metaQuery;
                }
            }
        }

        return wp_get_nav_menus($args);
    }
}