<?php


namespace BuilderiusPro\Bundle\GraphQLBundle\Resolver;

use Builderius\Bundle\ExpressionLanguageBundle\ExpressionLanguage;
use Builderius\Bundle\GraphQLBundle\Resolver\AbstractLocalVarsAwareResolver;
use Builderius\GraphQL\Cache\GraphQLObjectCache;
use Builderius\GraphQL\Type\Definition\ResolveInfo;

class PaginationResolver extends AbstractLocalVarsAwareResolver
{
    /**
     * @var array
     */
    protected $typeNames;

    /**
     * @param array $typeNames
     * @param GraphQLObjectCache $graphqlSubfieldsCache
     * @param ExpressionLanguage $expressionLanguage
     */
    public function __construct(
        array $typeNames,
        GraphQLObjectCache $graphqlSubfieldsCache,
        ExpressionLanguage $expressionLanguage
    ) {
        $this->typeNames = $typeNames;
        parent::__construct($graphqlSubfieldsCache, $expressionLanguage);
    }

    /**
     * @inheritDoc
     */
    public function getTypeNames()
    {
        return $this->typeNames;
    }

    /**
     * @inheritDoc
     */
    public function getSortOrder()
    {
        return 10;
    }

    /**
     * @inheritDoc
     */
    public function getFieldName()
    {
        return 'pagination';
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($objectValue, array $args, $context, ResolveInfo $info)
    {
        return true;
    }

    /**
     * @inheritDoc
     */
    public function resolve($objectValue, array $args, $context, ResolveInfo $info)
    {
        $params = $_GET;
        if (isset($args['query'])) {
            $args = $this->processArguments($args['query'], $info->path);
        }
        if (isset($args['paged_query_var_name']) && $args['paged_query_var_name'] === 'paged') {
            throw new \Exception('It is not allowed to use "paged" as paged_query_var_name');
        } elseif (!isset($args['paged_query_var_name'])) {
            $args['paged_query_var_name'] = 'paged';
        }
        $params[$args['paged_query_var_name']] = '%#%';
        $base = add_query_arg($params);
        unset($args['paged_query_var_name']);

        $args['base'] = $base;
        $args['format'] = '';
        $args['current'] = $objectValue->page;
        $args['total'] = $objectValue->pages;
        $args['type'] = 'plain';
        $lin = paginate_links($args);

        return html_entity_decode($lin);
    }
}