<?php

namespace BuilderiusPro\Bundle\GraphQLBundle\Resolver;

use Builderius\Bundle\ExpressionLanguageBundle\ExpressionLanguage;
use Builderius\Bundle\GraphQLBundle\Resolver\AbstractLocalVarsAwareResolver;
use Builderius\GraphQL\Cache\GraphQLObjectCache;
use Builderius\GraphQL\Type\Definition\ResolveInfo;

class PhpFunctionOutputResolver extends AbstractLocalVarsAwareResolver
{
    /**
     * @param GraphQLObjectCache $graphqlSubfieldsCache
     * @param ExpressionLanguage $expressionLanguage
     */
    public function __construct(
        GraphQLObjectCache $graphqlSubfieldsCache,
        ExpressionLanguage $expressionLanguage
    ) {
        parent::__construct($graphqlSubfieldsCache, $expressionLanguage);
    }

    /**
     * @inheritDoc
     */
    public function getTypeNames()
    {
        return [];
    }

    /**
     * @inheritDoc
     */
    public function getSortOrder()
    {
        return 10;
    }

    /**
     * @inheritDoc
     */
    public function getFieldName()
    {
        return 'php_function_output';
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($objectValue, array $args, $context, ResolveInfo $info)
    {
        return true;
    }

    /**
     * @inheritDoc
     */
    public function resolve($objectValue, array $args, $context, ResolveInfo $info)
    {
        $args = $this->processArguments($args, $info->path);
        if (function_exists($args['function'])) {
            try {
                add_filter('query', [$this, 'builderiusForbidDbChange']);
                if (!isset($args['arguments']) || empty($args['arguments'])) {
                    ob_start();
                    $result = call_user_func($args['function']);
                    ob_get_clean();
                    remove_filter('query', [$this, 'builderiusForbidDbChange']);

                    return $result;
                } else {
                    ob_start();
                    $result = call_user_func_array($args['function'], $args['arguments']);
                    ob_get_clean();
                    remove_filter('query', [$this, 'builderiusForbidDbChange']);
                    if ($result instanceof \WP_Error) {
                        return isset($args['fallback']) ? $args['fallback'] : null;
                    }
                    return $result;
                }
            } catch (\Exception $e) {
                remove_filter('query', [$this, 'builderiusForbidDbChange']);

                return isset($args['fallback']) ? $args['fallback'] : null;
            }
        } else {
            return isset($args['fallback']) ? $args['fallback'] : null;
        }
    }

    /**
     * @param string $query
     * @throws \Exception
     */
    public function builderiusForbidDbChange($query)
    {
        if (
            strpos($query, 'INSERT INTO') !== false ||
            (
                strpos($query, 'UPDATE ') !== false &&
                strpos($query, 'SET ') !== false
            ) ||
            strpos($query, 'DELETE FROM') !== false ||
            strpos($query, 'CREATE') !== false ||
            strpos($query, 'ALTER') !== false ||
            strpos($query, 'TRUNCATE') !== false ||
            strpos($query, 'DROP') !== false
        ) {
            throw new \Exception('It is forbidden to modify the database');
        }

        return $query;
    }
}