<?php

namespace BuilderiusPro\Bundle\GraphQLBundle\Resolver;

use Builderius\Bundle\ExpressionLanguageBundle\ExpressionLanguage;
use Builderius\Bundle\GraphQLBundle\Resolver\AbstractLocalVarsAwareResolver;
use Builderius\GraphQL\Cache\GraphQLObjectCache;
use Builderius\GraphQL\Type\Definition\ResolveInfo;

class PostResolver extends AbstractLocalVarsAwareResolver
{
    /**
     * @var array
     */
    protected $typeNames;

    /**
     * @param array $typeNames
     * @param GraphQLObjectCache $graphqlSubfieldsCache
     * @param ExpressionLanguage $expressionLanguage
     */
    public function __construct(
        array $typeNames,
        GraphQLObjectCache $graphqlSubfieldsCache,
        ExpressionLanguage $expressionLanguage
    ) {
        $this->typeNames = $typeNames;
        parent::__construct($graphqlSubfieldsCache, $expressionLanguage);
    }

    /**
     * @inheritDoc
     */
    public function getTypeNames()
    {
        return $this->typeNames;
    }

    /**
     * @inheritDoc
     */
    public function getSortOrder()
    {
        return 10;
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($objectValue, array $args, $context, ResolveInfo $info)
    {
        return true;
    }

    /**
     * @inheritDoc
     */
    public function getFieldName()
    {
        return 'post';
    }

    /**
     * @inheritDoc
     */
    public function resolve($objectValue, array $args, $context, ResolveInfo $info)
    {
        $args = $this->processArguments($args, $info->path);
        if (!isset($args['identifier']) || $args['identifier'] === 'ID') {
            try{
                return get_post($args['value']);
            } catch (\Exception $e) {
                return null;
            }

        } elseif ($args['identifier'] === 'slug') {
            $wpQuery = new \WP_Query();
            $posts = $wpQuery->query(
                [
                    'name' => $args['value'],
                    'post_status' => get_post_stati()
                ]
            );
            if (!empty($posts)) {
                return $posts[0];
            } else {
                $posts = $wpQuery->query(
                    [
                        'pagename' => $args['value'],
                        'post_status' => get_post_stati()
                    ]
                );
                if (!empty($posts)) {
                    return $posts[0];
                } else {
                    return null;
                }
            }
        }

        return null;
    }
}