<?php


namespace BuilderiusPro\Bundle\GraphQLBundle\Resolver;

use Builderius\Bundle\ExpressionLanguageBundle\ExpressionLanguage;
use Builderius\Bundle\GraphQLBundle\Resolver\AbstractLocalVarsAwareResolver;
use Builderius\GraphQL\Cache\GraphQLObjectCache;
use Builderius\GraphQL\Type\Definition\ResolveInfo;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\EventDispatcher\EventDispatcher;
use BuilderiusPro\Bundle\GraphQLBundle\Event\FacetDataSourceCalculationEvent;

class PostsFacetsDataResolver extends AbstractLocalVarsAwareResolver
{
    /**
     * @var GraphQLObjectCache
     */
    protected $graphqlFacetsCache;

    /**
     * @var EventDispatcher
     */
    protected $eventDispatcher;

    /**
     * @var string
     */
    protected $fieldName;

    /**
     * @param string $fieldName
     * @param GraphQLObjectCache $graphqlSubfieldsCache
     * @param GraphQLObjectCache $graphqlFacetsCache
     * @param ExpressionLanguage $expressionLanguage
     * @param EventDispatcher $eventDispatcher
     */
    public function __construct(
        $fieldName,
        GraphQLObjectCache $graphqlSubfieldsCache,
        GraphQLObjectCache $graphqlFacetsCache,
        ExpressionLanguage $expressionLanguage,
        EventDispatcher $eventDispatcher
    ) {
        $this->fieldName = $fieldName;
        $this->graphqlFacetsCache = $graphqlFacetsCache;
        $this->eventDispatcher = $eventDispatcher;
        parent::__construct($graphqlSubfieldsCache, $expressionLanguage);
    }

    /**
     * @inheritDoc
     */
    public function getTypeNames()
    {
        return ['Facets'];
    }

    /**
     * @inheritDoc
     */
    public function getSortOrder()
    {
        return 10;
    }

    /**
     * @inheritDoc
     */
    public function getFieldName()
    {
        return $this->fieldName;
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($objectValue, array $args, $context, ResolveInfo $info)
    {
        return true;
    }

    /**
     * @inheritDoc
     */
    public function resolve($objectValue, array $args, $context, ResolveInfo $info)
    {
        $args = $this->processArguments($args, $info->path);
        $fields = $info->getFieldSelection();
        $queryArgs = $this->graphqlFacetsCache->get('query_args');
        $facetsArgs = $this->graphqlFacetsCache->get('facets_args');
        $submittedFacets = $this->graphqlFacetsCache->get('submitted_facets');
        $facetName = $args['facet_name'];
        if (false !== $facetsArgs) {
            if (isset($facetsArgs['wp_query'])) {
                foreach ($facetsArgs['wp_query'] as $wpQueryFacet) {
                    if ($facetName === $wpQueryFacet['facet_name']) {
                        $event = new FacetDataSourceCalculationEvent(
                            str_replace('_facet_data', '', $this->fieldName),
                            isset($args['data_source']) ? $args['data_source'] : []
                        );
                        $this->eventDispatcher->dispatch($event, 'facet_data_source_calculation');
                        $facetData = $event->getFacetData();
                        $allCnt = 0;
                        if (isset($fields['quantity'])) {
                            $wpQuery = new \WP_Query();
                            $facetQueryArgs = $queryArgs;
                            $facetQueryArgs['fields'] = 'count';
                            $facetQueryArgs['posts_per_page'] = -1;
                            $facetQueryArgs['no_found_rows'] = true;
                            if (isset($wpQueryFacet['fixed_query_arguments'])) {
                                foreach ($wpQueryFacet['fixed_query_arguments'] as $faKey => $faValue) {
                                    $facetQueryArgs[$faKey] = $faValue;
                                }
                            }
                            foreach ($facetData as &$item) {
                                $facetQueryArgs[$wpQueryFacet['query_argument']] = $item->value;
                                $item->quantity = count($wpQuery->query($facetQueryArgs));
                                $allCnt += $item->quantity;
                            }
                        }

                        return $this->processFacetData($facetData, $allCnt, $args, $fields);
                    }
                }
            }
            if (isset($facetsArgs['wp_meta_query'])) {
                foreach ($facetsArgs['wp_meta_query']['array'] as $metaQueryFacet) {
                    if ($facetName === $metaQueryFacet['facet_name']) {

                        $event = new FacetDataSourceCalculationEvent(
                            str_replace('_facet_data', '', $this->fieldName),
                            isset($args['data_source']) ? $args['data_source'] : []
                        );
                        $this->eventDispatcher->dispatch($event, 'facet_data_source_calculation');
                        $facetData = $event->getFacetData();
                        $allCnt = 0;
                        if (isset($fields['quantity'])) {
                            $wpQuery = new \WP_Query();
                            $facetQueryArgs = $queryArgs;
                            $facetQueryArgs['fields'] = 'count';
                            $facetQueryArgs['posts_per_page'] = -1;
                            $facetQueryArgs['no_found_rows'] = true;
                            $metaQueryArgs = [];
                            if (isset($metaQueryFacet['fixed_query_arguments'])) {
                                foreach ($metaQueryFacet['fixed_query_arguments'] as $faKey => $faValue) {
                                    $metaQueryArgs[$faKey] = $faValue;
                                }
                            }
                            $queryArgs['meta_query'][] = $metaQueryArgs;

                            if (isset($submittedFacets[$facetName])) {
                                if (
                                    !isset($args['quantity_calculation_strategy']) ||
                                    strtoupper($args['quantity_calculation_strategy']) === 'AND'
                                ) {
                                    foreach ($facetData as &$item) {
                                        $tmpQueryArgs = $facetQueryArgs;
                                        $metaQueryArgs[$metaQueryFacet['query_argument']] = $item->value;
                                        $tmpQueryArgs['meta_query'][] = $metaQueryArgs;
                                        $item->quantity = count($wpQuery->query($tmpQueryArgs));
                                        $allCnt += $item->quantity;
                                    }
                                } else {
                                    $submittedMetaQueryArgs = $metaQueryArgs;
                                    $submittedMetaQueryArgs[$metaQueryFacet['query_argument']] = $submittedFacets[$facetName];
                                    $key = array_search($submittedMetaQueryArgs, $facetQueryArgs['meta_query']);
                                    foreach ($facetData as &$item) {
                                        $tmpQueryArgs = $facetQueryArgs;
                                        $metaQueryArgs[$metaQueryFacet['query_argument']] = $item->value;
                                        $tmpQueryArgs['meta_query'][$key] = $metaQueryArgs;
                                        $item->quantity = count($wpQuery->query($tmpQueryArgs));
                                        $allCnt += $item->quantity;
                                    }
                                }
                            } else {
                                foreach ($facetData as &$item) {
                                    $tmpQueryArgs = $facetQueryArgs;
                                    $metaQueryArgs[$metaQueryFacet['query_argument']] = $item->value;
                                    $tmpQueryArgs['meta_query'][] = $metaQueryArgs;
                                    $item->quantity = count($wpQuery->query($tmpQueryArgs));
                                    $allCnt += $item->quantity;
                                }
                            }
                        }

                        return $this->processFacetData($facetData, $allCnt, $args, $fields);
                    }
                }
            }
            if (isset($facetsArgs['wp_tax_query'])) {
                foreach ($facetsArgs['wp_tax_query']['array'] as $taxQueryFacet) {
                    if ($facetName === $taxQueryFacet['facet_name']) {

                        $event = new FacetDataSourceCalculationEvent(
                            str_replace('_facet_data', '', $this->fieldName),
                            isset($args['data_source']) ? $args['data_source'] : []
                        );
                        $this->eventDispatcher->dispatch($event, 'facet_data_source_calculation');
                        $facetData = $event->getFacetData();
                        $allCnt = 0;
                        if (isset($fields['quantity'])) {
                            $wpQuery = new \WP_Query();
                            $facetQueryArgs = $queryArgs;
                            $facetQueryArgs['fields'] = 'count';
                            $facetQueryArgs['posts_per_page'] = -1;
                            $facetQueryArgs['no_found_rows'] = true;
                            $taxQueryArgs = [];
                            if (isset($taxQueryFacet['fixed_query_arguments'])) {
                                foreach ($taxQueryFacet['fixed_query_arguments'] as $faKey => $faValue) {
                                    $taxQueryArgs[$faKey] = $faValue;
                                }
                            }
                            $queryArgs['tax_query'][] = $taxQueryArgs;

                            if (isset($submittedFacets[$facetName])) {
                                if (
                                    !isset($args['quantity_calculation_strategy']) ||
                                    strtoupper($args['quantity_calculation_strategy']) === 'AND'
                                ) {
                                    foreach ($facetData as &$item) {
                                        $tmpQueryArgs = $facetQueryArgs;
                                        $taxQueryArgs[$taxQueryFacet['query_argument']] = $item->value;
                                        $tmpQueryArgs['tax_query'][] = $taxQueryArgs;
                                        $item->quantity = count($wpQuery->query($tmpQueryArgs));
                                        $allCnt += $item->quantity;
                                    }
                                } else {
                                    $submittedTaxQueryArgs = $taxQueryArgs;
                                    $submittedTaxQueryArgs[$taxQueryFacet['query_argument']] = $submittedFacets[$facetName];
                                    $key = array_search($submittedTaxQueryArgs, $facetQueryArgs['tax_query']);
                                    foreach ($facetData as &$item) {
                                        $tmpQueryArgs = $facetQueryArgs;
                                        $taxQueryArgs[$taxQueryFacet['query_argument']] = $item->value;
                                        $tmpQueryArgs['tax_query'][$key] = $taxQueryArgs;
                                        $item->quantity = count($wpQuery->query($tmpQueryArgs));
                                        $allCnt += $item->quantity;
                                    }
                                }
                            } else {
                                foreach ($facetData as &$item) {
                                    $tmpQueryArgs = $facetQueryArgs;
                                    $taxQueryArgs[$taxQueryFacet['query_argument']] = $item->value;
                                    $tmpQueryArgs['tax_query'][] = $taxQueryArgs;
                                    $item->quantity = count($wpQuery->query($tmpQueryArgs));
                                    $allCnt += $item->quantity;
                                }
                            }
                        }

                        return $this->processFacetData($facetData, $allCnt, $args, $fields);
                    }
                }
            }
            if (isset($facetsArgs['wp_date_query'])) {
                foreach ($facetsArgs['wp_date_query']['array'] as $dateQueryFacet) {
                    if ($facetName === $dateQueryFacet['facet_name']) {

                        $event = new FacetDataSourceCalculationEvent(
                            str_replace('_facet_data', '', $this->fieldName),
                            isset($args['data_source']) ? $args['data_source'] : []
                        );
                        $this->eventDispatcher->dispatch($event, 'facet_data_source_calculation');
                        $facetData = $event->getFacetData();
                        $allCnt = 0;
                        if (isset($fields['quantity'])) {
                            $wpQuery = new \WP_Query();
                            $facetQueryArgs = $queryArgs;
                            $facetQueryArgs['fields'] = 'count';
                            $facetQueryArgs['posts_per_page'] = -1;
                            $facetQueryArgs['no_found_rows'] = true;
                            $dateQueryArgs = [];
                            if (isset($dateQueryFacet['fixed_query_arguments'])) {
                                foreach ($dateQueryFacet['fixed_query_arguments'] as $faKey => $faValue) {
                                    $dateQueryArgs[$faKey] = $faValue;
                                }
                            }
                            $queryArgs['date_query'][] = $dateQueryArgs;

                            if (isset($submittedFacets[$facetName])) {
                                if (
                                    !isset($args['quantity_calculation_strategy']) ||
                                    strtoupper($args['quantity_calculation_strategy']) === 'AND'
                                ) {
                                    foreach ($facetData as &$item) {
                                        $tmpQueryArgs = $facetQueryArgs;
                                        $dateQueryArgs[$dateQueryFacet['query_argument']] = $item->value;
                                        $tmpQueryArgs['date_query'][] = $dateQueryArgs;
                                        $item->quantity = count($wpQuery->query($tmpQueryArgs));
                                        $allCnt += $item->quantity;
                                    }
                                } else {
                                    $submittedDateQueryArgs = $dateQueryArgs;
                                    $submittedDateQueryArgs[$dateQueryFacet['query_argument']] = $submittedFacets[$facetName];
                                    $key = array_search($submittedDateQueryArgs, $facetQueryArgs['date_query']);
                                    foreach ($facetData as &$item) {
                                        $tmpQueryArgs = $facetQueryArgs;
                                        $dateQueryArgs[$dateQueryFacet['query_argument']] = $item->value;
                                        $tmpQueryArgs['date_query'][$key] = $dateQueryArgs;
                                        $item->quantity = count($wpQuery->query($tmpQueryArgs));
                                        $allCnt += $item->quantity;
                                    }
                                }
                            } else {
                                foreach ($facetData as &$item) {
                                    $tmpQueryArgs = $facetQueryArgs;
                                    $dateQueryArgs[$dateQueryFacet['query_argument']] = $item->value;
                                    $tmpQueryArgs['date_query'][] = $dateQueryArgs;
                                    $item->quantity = count($wpQuery->query($tmpQueryArgs));
                                    $allCnt += $item->quantity;
                                }
                            }
                        }

                        return $this->processFacetData($facetData, $allCnt, $args, $fields);
                    }
                }
            }
        }

        return [];
    }

    private function processFacetData(array $facetData, $allCnt, $args, $fields)
    {
        $sort = (isset($args['sort']) && in_array(strtoupper($args['sort']), ['ASC', 'DESC'])) ? $args['sort'] : 'ASC';
        $sortBy = (isset($args['sort_by']) && in_array($args['sort_by'], ['value', 'label', 'quantity'])) ? $args['sort_by'] : 'value';
        usort($facetData, function ($a, $b) use ($sort, $sortBy) {
            if ($a->$sortBy == $b->$sortBy) {
                return 0;
            }
            if($sort === 'ASC') {
                return ($a->$sortBy < $b->$sortBy) ? -1 : 1;
            } else {
                return ($a->$sortBy > $b->$sortBy) ? -1 : 1;
            }
        });
        if (isset($args['count']) && is_int($args['count']) && $args['count'] > 0) {
            $facetData = array_slice($facetData, 0, $args['count']);
        }
        if  (isset($args['show_placeholder']) && $args['show_placeholder'] == true) {
            $plItem = [];
            if (isset($fields['value'])) {
                $plItem['value'] = '';
            }
            if (isset($fields['label'])) {
                $plItem['label'] = isset($args['placeholder_text']) ? $args['placeholder_text'] : 'All';
            }
            if (isset($fields['quantity'])) {
                $plItem['quantity'] = $allCnt;
            }
            array_unshift(
                $facetData,
                (object)$plItem
            );
        }

        return $facetData;
    }
}