<?php


namespace BuilderiusPro\Bundle\GraphQLBundle\Resolver;

use Builderius\GraphQL\Type\Definition\ResolveInfo;

class QueriedPostsResolver extends AbstractPostsResolver
{
    /**
     * @inheritDoc
     */
    public function getFieldName()
    {
        return 'queried_posts';
    }

    /**
     * @inheritDoc
     */
    public function resolve($objectValue, array $args, $context, ResolveInfo $info)
    {
        if (is_singular() || is_404()) {
            return (object)[
                'count' => 0,
                'page' => 0,
                'pages' => 0,
                'nodes' => [],
                'facets' => []
            ];
        } else {
            $this->graphqlFacetsCache->delete('query_args');
            $this->graphqlFacetsCache->delete('facets_args');
            $this->graphqlFacetsCache->delete('submitted_facets');
            global $wp_query;
            $query_vars = $wp_query->query_vars;
            if ((!isset($args['query']) || empty($args['query'])) && (!isset($args['facets']) || empty($args['facets']))) {
                $posts = $wp_query->posts ?: $wp_query->get_posts();

                return (object)[
                    'count' => $wp_query->post_count,
                    'page' => is_int($query_vars['paged']) && $query_vars['paged'] > 0 ? $query_vars['paged'] : 1,
                    'pages' => $wp_query->max_num_pages ?: 1,
                    'nodes' => $posts,
                    'facets' => []
                ];
            } else {
                $args = $this->processQueryArgs($args, $info->path);
                $defaults = $wp_query->fill_query_vars([]);
                $new_vars = [];
                foreach ($query_vars as $key => $value) {
                    if (!isset($defaults[$key]) || $defaults[$key] !== $value) {
                        $new_vars[$key] = $value;
                    }
                }
                foreach ($args as $key => $value) {
                    $new_vars[$key] = $value;
                }
                if ((isset($args['posts_per_page']) || isset($args['posts_per_archive_page'])) && !isset($args['nopaging'])) {
                    unset($new_vars['nopaging']);
                }
                $wpQuery = new \WP_Query();
                $posts = $wpQuery->query($new_vars);
                $queryVars = $wpQuery->query_vars;

                return (object)[
                    'count' => $wpQuery->post_count,
                    'page' => is_int($queryVars['paged']) && $queryVars['paged'] > 0 ? $queryVars['paged'] : 1,
                    'pages' => $wpQuery->max_num_pages ?: 1,
                    'nodes' => $posts,
                    'facets' => []
                ];
            }
        }
    }
}