<?php


namespace BuilderiusPro\Bundle\GraphQLBundle\Resolver;

use Builderius\Bundle\ExpressionLanguageBundle\ExpressionLanguage;
use Builderius\Bundle\GraphQLBundle\Resolver\AbstractLocalVarsAwareResolver;
use Builderius\GraphQL\Cache\GraphQLObjectCache;
use Builderius\GraphQL\Type\Definition\ResolveInfo;

class TermsResolver extends AbstractLocalVarsAwareResolver
{
    /**
     * @var array
     */
    protected $typeNames;

    /**
     * @param array $typeNames
     * @param GraphQLObjectCache $graphqlSubfieldsCache
     * @param ExpressionLanguage $expressionLanguage
     */
    public function __construct(
        array $typeNames,
        GraphQLObjectCache $graphqlSubfieldsCache,
        ExpressionLanguage $expressionLanguage
    ) {
        $this->typeNames = $typeNames;
        parent::__construct($graphqlSubfieldsCache, $expressionLanguage);
    }

    /**
     * @inheritDoc
     */
    public function getTypeNames()
    {
        return $this->typeNames;
    }

    /**
     * @inheritDoc
     */
    public function getSortOrder()
    {
        return 10;
    }

    /**
     * @inheritDoc
     */
    public function getFieldName()
    {
        return 'terms';
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($objectValue, array $args, $context, ResolveInfo $info)
    {
        return true;
    }

    /**
     * @inheritDoc
     */
    public function resolve($objectValue, array $args, $context, ResolveInfo $info)
    {
        $args = isset($args['query']) ? $args['query'] : [];
        if (!empty($args)) {
            $args = $this->processArguments($args, $info->path);
            if (isset($args['meta_query']) && isset($args['meta_query']['array'])) {
                $metaQuery = [];
                foreach ($args['meta_query']['array'] as $metaQueryItem) {
                    if (isset($metaQueryItem['condition'])) {
                        if ($metaQueryItem['condition'] == true) {
                            unset($metaQueryItem['condition']);
                            $metaQuery[] = $metaQueryItem;
                        }
                    } else {
                        $metaQuery[] = $metaQueryItem;
                    }
                }
                if (!empty($metaQuery)) {
                    if (isset($args['meta_query']['relation'])) {
                        $metaQuery['relation'] = $args['meta_query']['relation'];
                    }
                    $args['meta_query'] = $metaQuery;
                }
            }
        }
        $terms = get_terms($args);
        $cargs = $args;
        unset($cargs['number']);
        unset($cargs['offset']);
        if (!isset($cargs['hide_empty'])) {
            $cargs['hide_empty'] = true;
        }
        $total = wp_count_terms($cargs);
        $page = 1;
        $pages = 1;
        if (isset($args['number']) && is_int($args['number']) && $args['number'] > 0) {
            $pages = ceil($total / $args['number']);
            if (isset($args['offset']) && is_int($args['offset']) && $args['offset'] > 0) {
                $page = ceil(($args['offset'] + $args['number']) / $args['number']);
            }
        }

        return (object)[
            'count' => $total,
            'page' => $page <= $pages ? $page : $pages,
            'pages' => $pages,
            'nodes' => $terms
        ];
    }
}