<?php


namespace BuilderiusPro\Bundle\GraphQLBundle\Resolver;

use Builderius\Bundle\ExpressionLanguageBundle\ExpressionLanguage;
use Builderius\Bundle\GraphQLBundle\Resolver\AbstractLocalVarsAwareResolver;
use Builderius\GraphQL\Cache\GraphQLObjectCache;
use Builderius\GraphQL\Type\Definition\ResolveInfo;

class UserResolver extends AbstractLocalVarsAwareResolver
{
    /**
     * @var array
     */
    protected $typeNames;

    /**
     * @param array $typeNames
     * @param GraphQLObjectCache $graphqlSubfieldsCache
     * @param ExpressionLanguage $expressionLanguage
     */
    public function __construct(
        array $typeNames,
        GraphQLObjectCache $graphqlSubfieldsCache,
        ExpressionLanguage $expressionLanguage
    ) {
        $this->typeNames = $typeNames;
        parent::__construct($graphqlSubfieldsCache, $expressionLanguage);
    }

    /**
     * @inheritDoc
     */
    public function getTypeNames()
    {
        return $this->typeNames;
    }

    /**
     * @inheritDoc
     */
    public function getSortOrder()
    {
        return 10;
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($objectValue, array $args, $context, ResolveInfo $info)
    {
        return true;
    }

    /**
     * @inheritDoc
     */
    public function getFieldName()
    {
        return 'user';
    }

    /**
     * @inheritDoc
     */
    public function resolve($objectValue, array $args, $context, ResolveInfo $info)
    {
        $args = $this->processArguments($args, $info->path);
        if (!isset($args['identifier'])) {
            $args['identifier'] = 'ID';
        }

        try{
            return get_user_by($args['identifier'], $args['value']);
        } catch (\Exception $e) {
            return null;
        }
    }
}