<?php

namespace BuilderiusPro\Bundle\MetaBoxBundle\DynamicDataHelper;

use Builderius\Bundle\TemplateBundle\DynamicDataHelper\DynamicDataHelper;
use Builderius\Bundle\TemplateBundle\DynamicDataHelper\DynamicDataHelperArgument;
use Builderius\Bundle\TemplateBundle\DynamicDataHelper\DynamicDataHelperInterface;
use Builderius\Bundle\TemplateBundle\DynamicDataHelper\DynamicDataHelpersProviderInterface;

abstract class AbstractMetaBoxFieldsDynamicDataHelpersProvider implements DynamicDataHelpersProviderInterface
{
    const SCALAR_TYPES = [
        'button_group', 'checkbox', 'color', 'custom_html', 'date', 'datetime', 'time', 'file_input', 'image_select',
        'text', 'textarea', 'video', 'radio', 'select', 'email', 'number', 'range', 'url', 'button', 'input', 'oembed',
        'switch', 'wysiwyg', 'password'
    ];
    const ARRAY_TYPES = [
        'autocomplete', 'checkbox_list'
    ];

    /**
     * @var DynamicDataHelperInterface
     */
    protected $ddHelpers = [];

    /**
     * @param array $params
     * @return DynamicDataHelper
     */
    protected function createHelper(array $params)
    {
        return new DynamicDataHelper($params);
    }

    /**
     * @param array $params
     * @return DynamicDataHelper
     */
    protected function createBackgroundFieldHelper(
        array $params
    ) {
        $ddHelper = $this->createHelper($params);
        $argument = new DynamicDataHelperArgument([
            DynamicDataHelperArgument::NAME_FIELD => 'argument',
            DynamicDataHelperArgument::TYPE_FIELD => 'select',
            DynamicDataHelperArgument::PLACEHOLDER_FIELD => __('Please select subfield'),
            DynamicDataHelperArgument::VALUE_LIST_FIELD => [
                ['value' => 'color', 'title' => 'color'],
                ['value' => 'image', 'title' => 'image'],
                ['value' => 'position', 'title' => 'position'],
                ['value' => 'attachment', 'title' => 'attachment'],
                ['value' => 'size', 'title' => 'size'],
                ['value' => 'repeat', 'title' => 'repeat']
            ]
        ]);
        $ddHelper->addArgument($argument);

        return $ddHelper;
    }

    /**
     * @param array $params
     * @return DynamicDataHelper
     */
    protected function createFieldsetTextFieldHelper(
        array $params
    ) {
        $ddHelper = $this->createHelper($params);
        $argument = new DynamicDataHelperArgument([
            DynamicDataHelperArgument::NAME_FIELD => 'argument',
            DynamicDataHelperArgument::TYPE_FIELD => 'select',
            DynamicDataHelperArgument::PLACEHOLDER_FIELD => __('Please select subfield'),
            DynamicDataHelperArgument::VALUE_LIST_FIELD => [
                ['value' => 'color', 'title' => 'color'],
                ['value' => 'position', 'title' => 'position'],
                ['value' => 'attachment', 'title' => 'attachment'],
                ['value' => 'size', 'title' => 'size']
            ]
        ]);
        $ddHelper->addArgument($argument);

        return $ddHelper;
    }

    /**
     * @param array $params
     * @return DynamicDataHelper
     */
    protected function createImageFieldHelper(
        array $params
    ) {
        $ddHelper = $this->createHelper($params);
        $argument = new DynamicDataHelperArgument([
            DynamicDataHelperArgument::NAME_FIELD => 'argument',
            DynamicDataHelperArgument::TYPE_FIELD => 'select',
            DynamicDataHelperArgument::PLACEHOLDER_FIELD => __('Please select subfield'),
            DynamicDataHelperArgument::VALUE_LIST_FIELD => [
                ['value' => 'ID', 'title' => 'ID'],
                ['value' => 'name', 'title' => 'name'],
                ['value' => 'path', 'title' => 'path'],
                ['value' => 'url', 'title' => 'url'],
                ['value' => 'width', 'title' => 'width'],
                ['value' => 'height', 'title' => 'height'],
                ['value' => 'full_url', 'title' => 'full_url'],
                ['value' => 'title', 'title' => 'title'],
                ['value' => 'caption', 'title' => 'caption'],
                ['value' => 'description', 'title' => 'description'],
                ['value' => 'alt', 'title' => 'alt'],
                ['value' => 'srcset', 'title' => 'srcset'],
                ['value' => 'sizes', 'title' => 'sizes'],
                ['value' => 'image_meta', 'title' => 'image_meta']
            ]
        ]);
        $ddHelper->addArgument($argument);

        return $ddHelper;
    }

    /**
     * @param array $params
     * @return DynamicDataHelper
     */
    protected function createFileFieldHelper(
        array $params
    ) {
        $ddHelper = $this->createHelper($params);
        $argument = new DynamicDataHelperArgument([
            DynamicDataHelperArgument::NAME_FIELD => 'argument',
            DynamicDataHelperArgument::TYPE_FIELD => 'select',
            DynamicDataHelperArgument::PLACEHOLDER_FIELD => __('Please select subfield'),
            DynamicDataHelperArgument::VALUE_LIST_FIELD => [
                ['value' => 'ID', 'title' => 'ID'],
                ['value' => 'name', 'title' => 'name'],
                ['value' => 'path', 'title' => 'path'],
                ['value' => 'url', 'title' => 'url'],
                ['value' => 'title', 'title' => 'title']
            ]
        ]);
        $ddHelper->addArgument($argument);
        
        return $ddHelper;
    }

    /**
     * @param array $field
     * @param array $params
     * @return DynamicDataHelper
     */
    protected function createKeyValueFieldHelper(
        array $field,
        array $params
    ) {
        $ddHelper = $this->createHelper($params);
        $argument = new DynamicDataHelperArgument([
            DynamicDataHelperArgument::NAME_FIELD => 'argument',
            DynamicDataHelperArgument::TYPE_FIELD => 'input',
            DynamicDataHelperArgument::PLACEHOLDER_FIELD => __('Please input key')
        ]);
        $ddHelper->addArgument($argument);

        return $ddHelper;
    }

    /**
     * @param array $field
     * @param array $params
     * @return DynamicDataHelper
     */
    protected function createGroupFieldHelper(
        array $field,
        array $params
    ) {
        $valueList = [];
        foreach ($field['fields'] as $subField) {
            if (in_array($subField['type'], self::SCALAR_TYPES)) {
                $valueList[] = ['value' => $subField['id'], 'title' => $subField['name']];
            }
        }

        $ddHelper = $this->createHelper($params);
        $argument = new DynamicDataHelperArgument([
            DynamicDataHelperArgument::NAME_FIELD => 'argument',
            DynamicDataHelperArgument::TYPE_FIELD => 'select',
            DynamicDataHelperArgument::PLACEHOLDER_FIELD => __('Please select subfield'),
            DynamicDataHelperArgument::VALUE_LIST_FIELD => $valueList
        ]);
        $ddHelper->addArgument($argument);

        return $ddHelper;
    }

    /**
     * @param array $params
     * @return DynamicDataHelper
     */
    protected function createPostFieldHelper(
        array $params
    ) {
        $ddHelper = $this->createHelper($params);
        $argument = new DynamicDataHelperArgument([
            DynamicDataHelperArgument::NAME_FIELD => 'argument',
            DynamicDataHelperArgument::TYPE_FIELD => 'select',
            DynamicDataHelperArgument::PLACEHOLDER_FIELD => __('Please select subfield'),
            DynamicDataHelperArgument::VALUE_LIST_FIELD => [
                ['value' => 'post_title', 'title' => 'Post title'],
                ['value' => 'ID', 'title' => 'Post ID'],
                ['value' => 'guid', 'title' => 'Post link'],
                ['value' => 'post_date', 'title' => 'Post datetime'],
                ['value' => 'post_modified', 'title' => 'Post modified datetime'],
                ['value' => 'post_content', 'title' => 'Post content'],
                ['value' => 'post_excerpt', 'title' => 'Post excerpt'],
                ['value' => 'post_status', 'title' => 'Post status'],
                ['value' => 'post_type', 'title' => 'Post type'],
                ['value' => 'post_author.user_login', 'title' => 'Author name'],
                ['value' => 'post_author.ID', 'title' => 'Author ID'],
                ['value' => 'post_author.display_name', 'title' => 'Author display name'],
                ['value' => 'post_author.first_name', 'title' => 'Author first name'],
                ['value' => 'post_author.last_name', 'title' => 'Author last name'],
                ['value' => 'post_author.user_email', 'title' => 'Author email'],
                ['value' => 'post_author.avatar_url', 'title' => 'Author avatar'],
            ]
        ]);
        $ddHelper->addArgument($argument);
        $argument = new DynamicDataHelperArgument([
            DynamicDataHelperArgument::NAME_FIELD => 'config',
            DynamicDataHelperArgument::TYPE_FIELD => 'hidden',
            DynamicDataHelperArgument::VALUE_FIELD => [
                'post_title' => ['alias' => 'title', 'field' => 'post_title'],
                'ID' => ['alias' => 'ID', 'field' => 'ID'],
                'guid' => ['alias' => 'link', 'field' => 'guid'],
                'post_date' => ['alias' => 'datetime', 'field' => 'post_date'],
                'post_modified' => ['alias' => 'modified_datetime', 'field' => 'post_modified'],
                'post_content' => ['alias' => 'content', 'field' => 'post_content'],
                'post_excerpt' => ['alias' => 'excerpt', 'field' => 'post_excerpt'],
                'post_status' => ['alias' => 'status', 'field' => 'post_status'],
                'post_type' => ['alias' => 'type', 'field' => 'post_type'],
                'post_author.user_login' => ['alias' => 'name', 'field' => 'user_login', 'parent_alias' => 'author', 'parent_field' => 'post_author'],
                'post_author.ID' => ['alias' => 'ID', 'field' => 'ID', 'parent_alias' => 'author', 'parent_field' => 'post_author'],
                'post_author.display_name' => ['alias' => 'display_name', 'field' => 'display_name', 'parent_alias' => 'author', 'parent_field' => 'post_author'],
                'post_author.first_name' => ['alias' => 'first_name', 'field' => 'first_name', 'parent_alias' => 'author', 'parent_field' => 'post_author'],
                'post_author.last_name' => ['alias' => 'last_name', 'field' => 'last_name', 'parent_alias' => 'author', 'parent_field' => 'post_author'],
                'post_author.user_email' => ['alias' => 'email', 'field' => 'user_email', 'parent_alias' => 'author', 'parent_field' => 'post_author'],
                'post_author.avatar_url' => ['alias' => 'avatar', 'field' => 'avatar_url', 'parent_alias' => 'author', 'parent_field' => 'post_author'],
            ]
        ]);
        $ddHelper->addArgument($argument);

        return $ddHelper;
    }

    /**
     * @param array $params
     * @return DynamicDataHelper
     */
    protected function createUserFieldHelper(
        array $params
    ) {
        $ddHelper = $this->createHelper($params);
        $argument = new DynamicDataHelperArgument([
            DynamicDataHelperArgument::NAME_FIELD => 'argument',
            DynamicDataHelperArgument::TYPE_FIELD => 'select',
            DynamicDataHelperArgument::PLACEHOLDER_FIELD => __('Please select subfield'),
            DynamicDataHelperArgument::VALUE_LIST_FIELD => [
                ['value' => 'user_login', 'title' => 'User name'],
                ['value' => 'ID', 'title' => 'User ID'],
                ['value' => 'display_name', 'title' => 'User display name'],
                ['value' => 'first_name', 'title' => 'User first name'],
                ['value' => 'last_name', 'title' => 'User last name'],
                ['value' => 'user_email', 'title' => 'User email'],
                ['value' => 'avatar_url', 'title' => 'User avatar']
            ]
        ]);
        $ddHelper->addArgument($argument);
        $argument = new DynamicDataHelperArgument([
            DynamicDataHelperArgument::NAME_FIELD => 'config',
            DynamicDataHelperArgument::TYPE_FIELD => 'hidden',
            DynamicDataHelperArgument::VALUE_FIELD => [
                'user_login' => ['alias' => 'name', 'field' => 'user_login'],
                'ID' => ['alias' => 'ID', 'field' => 'ID'],
                'display_name' => ['alias' => 'display_name', 'field' => 'display_name'],
                'first_name' => ['alias' => 'first_name', 'field' => 'first_name'],
                'last_name' => ['alias' => 'last_name', 'field' => 'last_name'],
                'user_email' => ['alias' => 'email', 'field' => 'user_email'],
                'avatar_url' => ['alias' => 'avatar', 'field' => 'avatar_url']
            ]
        ]);
        $ddHelper->addArgument($argument);

        return $ddHelper;
    }
    abstract protected function getDefaultParams(array $field, $idx);

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addScalarFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createHelper($params);
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addArrayFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $ddHelper = $this->createHelper($params);
        $ddHelper->setType('array');
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $ddHelper;
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addBackgroundFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createBackgroundFieldHelper($params);
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addFieldsetTextFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createFieldsetTextFieldHelper($params);
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addImageFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createImageFieldHelper($params);
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addFileFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createFileFieldHelper($field, $params);
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    abstract protected function addKeyValueFieldHelper(array $field, $idx);

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    abstract protected function addGroupFieldHelper(array $field, $idx);

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    abstract protected function addPostFieldHelper(array $field, $idx);

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    abstract protected function addUserFieldHelper(array $field, $idx);

    /**
     * @inheritDoc
     */
    public function getDynamicDataHelpers()
    {
        if (function_exists('rwmb_get_registry')) {
            $field_registry = rwmb_get_registry('field');
            $fields = $field_registry->get_by_object_type(static::LOCATION);
            $processed_fields = [];
            foreach ($fields as $group_fields) {
                $k = 0;
                foreach ($group_fields as $field) {
                    if (!in_array($field['id'], $processed_fields)) {
                        if ($field['clone'] === false && $field['multiple'] === false) {
                            $k++;
                            if ($field['type'] === 'background') {
                                $this->addBackgroundFieldHelper($field, $k);
                            } elseif ($field['type'] === 'fieldset_text') {
                                $this->addFieldsetTextFieldHelper($field, $k);
                            } elseif (in_array($field['type'], ['file_advanced', 'file_upload'])) {
                                $this->addFileFieldHelper($field, $k);
                            } elseif (in_array($field['type'], ['image_advanced', 'image_upload', 'single_image'])) {
                                $this->addImageFieldHelper($field, $k);
                            } elseif ($field['type'] === 'group') {
                                $this->addGroupFieldHelper($field, $k);
                            } elseif ($field['type'] === 'post') {
                                $this->addPostFieldHelper($field, $k);
                            } elseif ($field['type'] === 'user') {
                                $this->addUserFieldHelper($field, $k);
                            } else {
                                $this->addScalarFieldHelper($field, $k);
                            }
                        } elseif ($field['type'] === 'key_value') {
                            $this->addKeyValueFieldHelper($field, $k);
                        } elseif ($field['type'] === 'image') {
                            $this->addImageArrayFieldHelper($field, $k);
                        } else {
                            $this->addArrayFieldHelper($field, $k);
                        }
                        $processed_fields[] = $field['id'];
                    }
                }
            }
        }

        return $this->ddHelpers;
    }

    /**
     * @inheritDoc
     */
    public function getDynamicDataHelper($name)
    {
        if ($this->hasDynamicDataHelper($name)) {
            return $this->ddHelpers[$name];
        }

        return null;
    }

    /**
     * @inheritDoc
     */
    public function hasDynamicDataHelper($name)
    {
        return array_key_exists($name, $this->ddHelpers);
    }
}