<?php

namespace BuilderiusPro\Bundle\MetaBoxBundle\DynamicDataHelper;

use Builderius\Bundle\TemplateBundle\DynamicDataHelper\DynamicDataHelper;

class PostAuthorMetaBoxFieldsDynamicDataHelpersProvider extends AbstractMetaBoxFieldsDynamicDataHelpersProvider
{
    const LOCATION = 'user';

    protected function getDefaultParams(array $field, $idx)
    {
        $sanitizedName = strtolower(preg_replace('/[^\\w]+/', '_', $field['name'], -1));
        $expression = '"wp.post.author.mb_field__' . $sanitizedName . '"';
        return [
            DynamicDataHelper::NAME_FIELD => 'post_author_mb_value_' . $sanitizedName,
            DynamicDataHelper::LABEL_FIELD => 'Post author MB "' . $field['name'] . '" field value',
            DynamicDataHelper::CATEGORY_FIELD => 'post_author',
            DynamicDataHelper::SORT_ORDER_FIELD => 400 + 10 * $idx,
            DynamicDataHelper::GRAPHQL_PATH_FIELD => [
                '"query.post.__aliasFor"' => '"queried_post"',
                '"query.post.author.__aliasFor"' => '"post_author"',
                '"query.post.author.mb_field__' . $sanitizedName. '.__aliasFor"' => '"metabox_value"',
                '"query.post.author.mb_field__' . $sanitizedName. '.__args.field_id"' => '"' . $field['id'] . '"',
            ],
            DynamicDataHelper::EXPRESSION_FIELD => $expression,
            DynamicDataHelper::ESCAPED_FIELD => false
        ];
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addKeyValueFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $sanitizedName = strtolower(preg_replace('/[^\\w]+/', '_', $field['name'], -1));
        $params[DynamicDataHelper::GRAPHQL_PATH_FIELD] = [
            '"query.post.__aliasFor"' => '"queried_post"',
            '"query.post.author.__aliasFor"' => '"post_author"',
            '"query.post.author.mb_field__' . $sanitizedName. '_private.__aliasFor"' => '"metabox_value"',
            '"query.post.author.mb_field__' . $sanitizedName. '_private.__args.field_id"' => '"' . $field['id'] . '"',
            '"query.post.author.mb_field__' . $sanitizedName. '_private.__directives.private"' => true,
            '"query.post.author.mb_field__' . $sanitizedName. '.__aliasFor"' => '"expression_result"',
            '"query.post.author.mb_field__' . $sanitizedName. '.__args.expression"' => '"{{{foreach(foreach(group_by(mb_field__' . $sanitizedName. '_private, 0), (value) -> {value[0]}), (val) -> {val[1]}) }}}"',
        ];
        $params[DynamicDataHelper::EXPRESSION_FIELD] = '"wp.post.author.mb_field__' . $sanitizedName . '." ~ argument';
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createKeyValueFieldHelper($field, $params);
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addImageArrayFieldHelper(array $field, $idx)
    {
        $sanitizedName = strtolower(preg_replace('/[^\\w]+/', '_', $field['name'], -1));
        $expression = '"wp.post.author.mb_field__' . $sanitizedName . '"';
        $params = [
            DynamicDataHelper::NAME_FIELD => 'post_author_mb_value_' . $sanitizedName,
            DynamicDataHelper::LABEL_FIELD => 'Post author MB "' . $field['name'] . '" field value',
            DynamicDataHelper::CATEGORY_FIELD => 'post_author',
            DynamicDataHelper::SORT_ORDER_FIELD => 400 + 10 * $idx,
            DynamicDataHelper::GRAPHQL_PATH_FIELD => [
                '"query.post.__aliasFor"' => '"queried_post"',
                '"query.post.author.__aliasFor"' => '"post_author"',
                '"query.post.author.mb_field__' . $sanitizedName. '_private.__aliasFor"' => '"metabox_value"',
                '"query.post.author.mb_field__' . $sanitizedName. '_private.__args.field_id"' => '"' . $field['id'] . '"',
                '"query.post.author.mb_field__' . $sanitizedName. '_private.__directives.private"' => true,
                '"query.post.author.mb_field__' . $sanitizedName. '.__aliasFor"' => '"expression_result"',
                '"query.post.author.mb_field__' . $sanitizedName. '.__args.expression"' => '"{{{foreach(mb_field__' . $sanitizedName. '_private, (value) -> {value}, false) }}}"',
            ],
            DynamicDataHelper::EXPRESSION_FIELD => $expression,
            DynamicDataHelper::ESCAPED_FIELD => false
        ];
        $ddHelper = $this->createHelper($params);
        $ddHelper->setType('array');
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $ddHelper;
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addGroupFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $sanitizedName = strtolower(preg_replace('/[^\\w]+/', '_', $field['name'], -1));
        $params[DynamicDataHelper::EXPRESSION_FIELD] = '"wp.post.author.mb_field__' . $sanitizedName . '." ~ argument';
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createGroupFieldHelper($field, $params);
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addPostFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $sanitizedName = strtolower(preg_replace('/[^\\w]+/', '_', $field['name'], -1));
        $params[DynamicDataHelper::GRAPHQL_PATH_FIELD] = [
            '"query.post.__aliasFor"' => '"queried_post"',
            '"query.post.author.__aliasFor"' => '"post_author"',
            '"query.post.author.mb_field__' . $sanitizedName. '.__aliasFor"' => '"metabox_post_value"',
            '"query.post.author.mb_field__' . $sanitizedName. '.__args.field_id"' => '"' . $field['id'] . '"',
            '"query.post.author.mb_field__' . $sanitizedName. '." ~ (isset(config[argument], "parent_alias") ? config[argument].parent_alias : config[argument].alias) ~ ".__aliasFor"' => 'isset(config[argument], "parent_field") ? config[argument].parent_field : config[argument].field',
            '"query.post.author.mb_field__' . $sanitizedName. '." ~ (isset(config[argument], "parent_alias") ? config[argument].parent_alias ~ "." : "") ~ config[argument].alias ~ ".__aliasFor"' => 'config[argument].field',
        ];
        $params[DynamicDataHelper::EXPRESSION_FIELD] = '"wp.post.author.mb_field__' . $sanitizedName . '." ~ (isset(config[argument], "parent_alias") ? config[argument].parent_alias ~ "." : "") ~ config[argument].alias';
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createPostFieldHelper($params);
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addUserFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $sanitizedName = strtolower(preg_replace('/[^\\w]+/', '_', $field['name'], -1));
        $params[DynamicDataHelper::GRAPHQL_PATH_FIELD] = [
            '"query.post.__aliasFor"' => '"queried_post"',
            '"query.post.author.__aliasFor"' => '"post_author"',
            '"query.post.author.mb_field__' . $sanitizedName. '.__aliasFor"' => '"metabox_user_value"',
            '"query.post.author.mb_field__' . $sanitizedName. '.__args.field_id"' => '"' . $field['id'] . '"',
            '"query.post.author.mb_field__' . $sanitizedName. '." ~ config[argument].alias ~ ".__aliasFor"' => 'config[argument].field',
        ];
        $params[DynamicDataHelper::EXPRESSION_FIELD] = '"wp.post.author.mb_field__' . $sanitizedName . '." ~ config[argument].alias';
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createUserFieldHelper($params);
    }
}