<?php

namespace BuilderiusPro\Bundle\MetaBoxBundle\DynamicDataHelper;

use Builderius\Bundle\TemplateBundle\DynamicDataHelper\DynamicDataHelper;

class SiteMetaBoxFieldsDynamicDataHelpersProvider extends AbstractMetaBoxFieldsDynamicDataHelpersProvider
{
    const LOCATION = 'setting';

    /**
     * @var string
     */
    private $settingPageId;

    /**
     * @var string
     */
    private $settingPageTitle;

    /**
     * {@inheritDoc}
     */
    protected function getDefaultParams(array $field, $idx)
    {
        $sanitizedSettingsPageId = strtolower(preg_replace('/[^\\w]+/', '_', $this->settingPageId, -1));
        $sanitizedName = strtolower(preg_replace('/[^\\w]+/', '_', $field['name'], -1));
        $expression = '"wp.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName . '"';
        return [
            DynamicDataHelper::NAME_FIELD => 'site_mb_value_'. $sanitizedSettingsPageId . '_' . $sanitizedName,
            DynamicDataHelper::LABEL_FIELD => 'MB ' . $this->settingPageTitle . ' page "' . $field['name'] . '" field value',
            DynamicDataHelper::CATEGORY_FIELD => 'site',
            DynamicDataHelper::SORT_ORDER_FIELD => 400 + 10 * $idx,
            DynamicDataHelper::GRAPHQL_PATH_FIELD => [
                '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '.__aliasFor"' => '"metabox_option_value"',
                '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '.__args.option_name"' => '"' . $this->settingPageId . '"',
                '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '.__args.field_id"' => '"' . $field['id'] . '"',
            ],
            DynamicDataHelper::EXPRESSION_FIELD => $expression,
            DynamicDataHelper::ESCAPED_FIELD => false
        ];
    }

    /**
     * {@inheritDoc}
     */
    public function getDynamicDataHelpers()
    {
        if (function_exists('rwmb_get_registry')) {
            $field_registry = rwmb_get_registry('field');
            $settings_pages = apply_filters('mb_settings_pages', []);
            $fields = $field_registry->get_by_object_type(static::LOCATION);
            $processed_fields = [];
            foreach ($fields as $settingPageId => $group_fields) {
                $this->settingPageId = $settingPageId;
                foreach ($settings_pages as $settings_page) {
                    if ($settingPageId === $settings_page['id']) {
                        $this->settingPageTitle = $settings_page['menu_title'];
                        break;
                    }
                }
                $k = 0;
                foreach ($group_fields as $field) {
                    if (!in_array($settingPageId . '__' . $field['id'], $processed_fields)) {
                        if ($field['clone'] === false && $field['multiple'] === false) {
                            $k++;
                            if ($field['type'] === 'background') {
                                $this->addBackgroundFieldHelper($field, $k);
                            } elseif ($field['type'] === 'fieldset_text') {
                                $this->addFieldsetTextFieldHelper($field, $k);
                            } elseif (in_array($field['type'], ['file_advanced', 'file_upload'])) {
                                $this->addFileFieldHelper($field, $k);
                            } elseif (in_array($field['type'], ['image_advanced', 'image_upload', 'single_image'])) {
                                $this->addImageFieldHelper($field, $k);
                            } elseif ($field['type'] === 'group') {
                                $this->addGroupFieldHelper($field, $k);
                            } elseif ($field['type'] === 'post') {
                                $this->addPostFieldHelper($field, $k);
                            } elseif ($field['type'] === 'user') {
                                $this->addUserFieldHelper($field, $k);
                            } else {
                                $this->addScalarFieldHelper($field, $k);
                            }
                        } elseif ($field['type'] === 'key_value') {
                            $this->addKeyValueFieldHelper($field, $k);
                        } elseif ($field['type'] === 'image') {
                            $this->addImageArrayFieldHelper($field, $k);
                        } else {
                            $this->addArrayFieldHelper($field, $k);
                        }
                        $processed_fields[] = $settingPageId . '__' . $field['id'];
                    }
                }
            }
        }

        return $this->ddHelpers;
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addKeyValueFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $sanitizedSettingsPageId = strtolower(preg_replace('/[^\\w]+/', '_', $this->settingPageId, -1));
        $sanitizedName = strtolower(preg_replace('/[^\\w]+/', '_', $field['name'], -1));
        $params[DynamicDataHelper::GRAPHQL_PATH_FIELD] = [
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName . '_private.__aliasFor"' => '"metabox_option_value"',
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName . '_private.__args.option_name"' => '"' . $this->settingPageId . '"',
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName . '_private.__args.field_id"' => '"' . $field['id'] . '"',
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName . '_private.__directives.private"' => true,
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName . '.__aliasFor"' => '"expression_result"',
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName . '.__args.expression"' => '"{{{foreach(foreach(group_by(mb_field__' . $sanitizedName. '_private, 0), (value) -> {value[0]}), (val) -> {val[1]}) }}}"',
        ];
        $params[DynamicDataHelper::EXPRESSION_FIELD] = '"wp.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName . '." ~ argument';
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createKeyValueFieldHelper($field, $params);
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addImageArrayFieldHelper(array $field, $idx)
    {
        $sanitizedSettingsPageId = strtolower(preg_replace('/[^\\w]+/', '_', $this->settingPageId, -1));
        $sanitizedName = strtolower(preg_replace('/[^\\w]+/', '_', $field['name'], -1));
        $expression = '"wp.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName . '"';
        $params = [
            DynamicDataHelper::NAME_FIELD => 'site_mb_value_'. $sanitizedSettingsPageId . '_' . $sanitizedName,
            DynamicDataHelper::LABEL_FIELD => 'MB ' . $this->settingPageTitle . ' page "' . $field['name'] . '" field value',
            DynamicDataHelper::CATEGORY_FIELD => 'site',
            DynamicDataHelper::SORT_ORDER_FIELD => 400 + 10 * $idx,
            DynamicDataHelper::GRAPHQL_PATH_FIELD => [
                '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '_private.__aliasFor"' => '"metabox_option_value"',
                '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '_private.__args.option_name"' => '"' . $this->settingPageId . '"',
                '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '_private.__args.field_id"' => '"' . $field['id'] . '"',
                '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '_private.__directives.private"' => true,
                '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '.__aliasFor"' => '"expression_result"',
                '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '.__args.expression"' => '"{{{foreach(mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '_private, (value) -> {value}, false) }}}"',
            ],
            DynamicDataHelper::EXPRESSION_FIELD => $expression,
            DynamicDataHelper::ESCAPED_FIELD => false
        ];
        $ddHelper = $this->createHelper($params);
        $ddHelper->setType('array');
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $ddHelper;
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addGroupFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $sanitizedSettingsPageId = strtolower(preg_replace('/[^\\w]+/', '_', $this->settingPageId, -1));
        $sanitizedName = strtolower(preg_replace('/[^\\w]+/', '_', $field['name'], -1));
        $params[DynamicDataHelper::EXPRESSION_FIELD] = '"wp.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName . '." ~ argument';
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createGroupFieldHelper($field, $params);
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addPostFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $sanitizedSettingsPageId = strtolower(preg_replace('/[^\\w]+/', '_', $this->settingPageId, -1));
        $sanitizedName = strtolower(preg_replace('/[^\\w]+/', '_', $field['name'], -1));
        $params[DynamicDataHelper::GRAPHQL_PATH_FIELD] = [
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '.__aliasFor"' => '"metabox_option_value"',
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '.__args.option_name"' => '"' . $this->settingPageId . '"',
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '.__args.field_id"' => '"' . $field['id'] . '"',
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '." ~ (isset(config[argument], "parent_alias") ? config[argument].parent_alias : config[argument].alias) ~ ".__aliasFor"' => 'isset(config[argument], "parent_field") ? config[argument].parent_field : config[argument].field',
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '." ~ (isset(config[argument], "parent_alias") ? config[argument].parent_alias ~ "." : "") ~ config[argument].alias ~ ".__aliasFor"' => 'config[argument].field',
        ];
        $params[DynamicDataHelper::EXPRESSION_FIELD] = '"wp.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '." ~ (isset(config[argument], "parent_alias") ? config[argument].parent_alias ~ "." : "") ~ config[argument].alias';
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createPostFieldHelper($params);
    }

    /**
     * @param array $field
     * @param $idx
     * @return void
     */
    protected function addUserFieldHelper(array $field, $idx)
    {
        $params = $this->getDefaultParams($field, $idx);
        $sanitizedSettingsPageId = strtolower(preg_replace('/[^\\w]+/', '_', $this->settingPageId, -1));
        $sanitizedName = strtolower(preg_replace('/[^\\w]+/', '_', $field['name'], -1));
        $params[DynamicDataHelper::GRAPHQL_PATH_FIELD] = [
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '.__aliasFor"' => '"metabox_option_value"',
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '.__args.option_name"' => '"' . $this->settingPageId . '"',
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '.__args.field_id"' => '"' . $field['id'] . '"',
            '"query.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '." ~ config[argument].alias ~ ".__aliasFor"' => 'config[argument].field',
        ];
        $params[DynamicDataHelper::EXPRESSION_FIELD] = '"wp.mb_' . $sanitizedSettingsPageId . '_page_field__' . $sanitizedName. '." ~ config[argument].alias';
        $this->ddHelpers[$params[DynamicDataHelper::NAME_FIELD]] = $this->createUserFieldHelper($params);
    }
}