<?php

namespace BuilderiusPro\Bundle\PolylangBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicant;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Converter\PostToApplicantDataConverter;
use Builderius\Bundle\TemplateBundle\ApplyRule\Checker\BuilderiusTemplateApplyRulesChecker;
use Builderius\Bundle\TemplateBundle\ApplyRule\Converter\ApplyRuleConfigConverter;
use Builderius\Bundle\TemplateBundle\Event\ApplicantSingleConfigEvent;

class PolylangApplicantsModificationEventListener
{
    /**
     * @var ApplicantSingleConfigEvent
     */
    private $event;

    /**
     * @param ApplicantSingleConfigEvent $event
     * @throws \Exception
     */
    public function onSingleConfigApplicantsCalculation(ApplicantSingleConfigEvent $event)
    {
        try {
            $this->event = $event;
            $applicants = $event->getApplicants();
            $applyRulesConfig = $event->getApplyRulesConfig();
            if (isset($applyRulesConfig['categories']['multilingual'])) {
                foreach (ApplyRuleConfigConverter::convert($applyRulesConfig['categories']['multilingual']) as $key => $config) {
                    if (in_array($key, BuilderiusTemplateApplyRulesChecker::CONJUNCTIONS)) {
                        $applicants = $this->checkApplicantGroupConfig($config, $key);
                    } else {
                        $applicants = $this->checkApplicantSingleConfig($config, $key);
                    }
                }
            }
            $event->setApplicants($applicants);
        } catch (\Throwable $e) {
            return;
        }
    }


    /**
     * @param array $subConfigs
     * @param string $conjunction
     * @return BuilderiusTemplateApplicantInterface[]
     */
    private function checkApplicantGroupConfig(array $subConfigs, $conjunction)
    {
        $applicantsSets = [];
        foreach ($subConfigs as $subConfig) {
            foreach ($subConfig as $key => $subSubConfig) {
                if (in_array($key, BuilderiusTemplateApplyRulesChecker::CONJUNCTIONS)) {
                    $applicantsSets[] = $this->checkApplicantGroupConfig($subSubConfig, $key);
                } else {
                    $applicantsSets[] = $this->checkApplicantSingleConfig($subSubConfig, $key);
                }
            }
        }
        $applicants = [];
        if ($conjunction === 'and') {
            foreach ($applicantsSets as $key => $applicantsSet) {
                if ($key === 0) {
                    $applicants = $applicantsSet;
                } else {
                    $existingApplicants = $applicants;
                    $applicants = [];
                    foreach ($applicantsSet as $k => $applicant) {
                        if (in_array($applicant, $existingApplicants)) {
                            $applicants[$k] = $applicant;
                        }
                    }
                }
            }
        } else {
            foreach ($applicantsSets as $key => $applicantsSet) {
                if ($key === 0) {
                    $applicants = $applicantsSet;
                } else {
                    foreach ($applicantsSet as $k => $applicant) {
                        if (!in_array($applicant, $applicants)) {
                            $applicants[$k] = $applicant;
                        }
                    }
                }
            }
        }

        return $applicants;
    }

    /**
     * @param array $config
     * @param string $operator
     * @return BuilderiusTemplateApplicantInterface[]
     */
    private function checkApplicantSingleConfig(array $config, $operator)
    {
        $rule = $config[0]['var'];
        $argument = $config[1];
        $frontpageId = get_option('page_on_front');
        $mainApplicants = $this->event->getApplicants();
        $newApplicants = [];
        if ($rule === 'polylang.all') {
            $pllLang = 'all';
        } else {
            $pllLang = $argument;
        }
        /** @var BuilderiusTemplateApplicantInterface $mainApplicant */
        foreach ($mainApplicants as $name => $mainApplicant) {
        if ($name === 'singular.single.page.' . $frontpageId) {
            $pllTranslations = pll_get_post_translations($frontpageId);
            if ($pllLang === 'all') {
                foreach ($pllTranslations as $lang => $postId) {
                    $post = get_post($postId);
                    $key = sprintf('singular.single.page.%s.%s', $postId, $lang);
                    if (!in_array($key, $newApplicants)) {
                        $newApplicants[sprintf('%s.%s', $name, $lang)] =
                            $this->createSimpleApplicant(
                                sprintf('Front page: %s', $post->post_title),
                                $mainApplicant->getUrl(),
                                $lang,
                                $mainApplicant->getCategory(),
                                $mainApplicant->getSortOrder(),
                                $mainApplicant->getParameters()
                            );
                    }
                }
            } else {
                foreach ($pllTranslations as $lang => $postId) {
                    if ($lang === $pllLang) {
                        $post = get_post($postId);
                        $key = sprintf('singular.single.page.%s.%s', $postId, $lang);
                        if (!in_array($key, $newApplicants)) {
                        $newApplicants[$key] = $this->createSimpleApplicant(
                            sprintf('Front page: %s', $post->post_title),
                            $mainApplicant->getUrl(),
                            $lang,
                            $mainApplicant->getCategory(),
                            $mainApplicant->getSortOrder(),
                            $mainApplicant->getParameters()
                        );
                        }
                    }
                }
            }
        } elseif (
                strpos($name, 'singular.') !== false &&
                strpos($name, 'error_404') === false &&
                strpos($name, '.attachment') === false &&
                $name !== 'singular.single.page.' . $frontpageId
            ) {
                $frontPageTranslations = pll_get_post_translations($frontpageId);
                $frontPageTranslations = array_values($frontPageTranslations);

                $arr = explode('.', $name);
                $mainApplicantId = end($arr);
                $nameMainPart = rtrim($name, '.' . $mainApplicantId);
                $pllTranslations = pll_get_post_translations($mainApplicantId);
                if ($pllLang === 'all') {
                    foreach ($pllTranslations as $lang => $postId) {
                        if (in_array($postId, $frontPageTranslations)) {
                            continue;
                        }
                        $post = get_post($postId);
                        $key = sprintf('%s.%s.%s', $nameMainPart, $postId, $lang);
                        if (!in_array($key, $newApplicants)) {
                            $newApplicants[$key] =
                                $this->convertPostToApplicant(
                                    $post,
                                    $lang,
                                    explode(':', $mainApplicant->getLabel())[0] . ': ' . $post->post_title,
                                    false,
                                    $mainApplicant->getCategory(),
                                    $mainApplicant->getParameters(),
                                    $mainApplicant->getSortOrder()
                                );
                        }
                    }
                } else {
                    foreach ($pllTranslations as $lang => $postId) {
                        if (in_array($postId, $frontPageTranslations)) {
                            continue;
                        }
                        if ($lang === $pllLang) {
                            $post = get_post($postId);
                            $key = sprintf('%s.%s.%s', $nameMainPart, $postId, $lang);
                            if (!in_array($key, $newApplicants)) {
                                $newApplicants[$key] =
                                    $this->convertPostToApplicant(
                                        $post,
                                        $lang,
                                        explode(':', $mainApplicant->getLabel())[0] . ': ' . $post->post_title,
                                        false,
                                        $mainApplicant->getCategory(),
                                        $mainApplicant->getParameters(),
                                        $mainApplicant->getSortOrder()
                                    );
                            }
                        }
                    }
                }
            } elseif (
                strpos($name, 'archive.category') !== false ||
                strpos($name, 'archive.tag') !== false ||
                strpos($name, 'archive.taxonomy') !== false
            ) {
                $taxLabel = '';
                if (strpos($name, 'archive.category') !== false) {
                    $taxLabel = 'Category';
                } elseif (strpos($name, 'archive.tag') !== false) {
                    $taxLabel = 'Tag';
                }
                $arr = explode('.', $name);
                $mainApplicantId = end($arr);
                if (strpos($name, 'archive.taxonomy') !== false) {
                    $taxName = str_replace('is_tax_', '', $arr[count($arr) - 2]);
                    $taxonomy = get_taxonomy($taxName);
                    $taxLabel = $taxonomy->labels->singular_name;
                }
                $pllTranslations = pll_get_term_translations($mainApplicantId);
                if ($pllLang === 'all') {
                    foreach ($pllTranslations as $lang => $termId) {
                        $term = get_term($termId);
                        $arr[count($arr) - 1] = $termId;
                        $applicantKey = implode('.', $arr);
                        if (!in_array($applicantKey, $newApplicants)) {
                            $newApplicants[$applicantKey] =
                                $this->createTermApplicant(
                                    $term,
                                    $taxLabel,
                                    $lang,
                                    $mainApplicant->getCategory(),
                                    $mainApplicant->getSortOrder(),
                                    $mainApplicant->getParameters()
                                );
                        }
                    }
                } else {
                    foreach ($pllTranslations as $lang => $termId) {
                        if ($lang === $pllLang) {
                            $term = get_term($termId);
                            $arr[count($arr) - 1] = $termId;
                            $applicantKey = implode('.', $arr);
                            if (!in_array($applicantKey, $newApplicants)) {
                                $newApplicants[$applicantKey] =
                                    $this->createTermApplicant(
                                        $term,
                                        $taxLabel,
                                        $lang,
                                        $mainApplicant->getCategory(),
                                        $mainApplicant->getSortOrder(),
                                        $mainApplicant->getParameters()
                                    );
                            }
                        }
                    }
                }
            } else {
                $pllLanguages = pll_languages_list();
                if ($pllLang === 'all') {
                    foreach ($pllLanguages as $lang) {
                        $key = sprintf('%s.%s', $name, $lang);
                        if (!in_array($key, $newApplicants)) {
                            $newApplicants[$key] =
                                $this->createSimpleApplicant(
                                    $mainApplicant->getLabel(),
                                    $mainApplicant->getUrl(),
                                    $lang,
                                    $mainApplicant->getCategory(),
                                    $mainApplicant->getSortOrder(),
                                    $mainApplicant->getParameters()
                                );
                        }
                    }
                } else {
                    foreach ($pllLanguages as $lang) {
                        if ($lang === $pllLang) {
                            $key = sprintf('%s.%s', $name, $lang);
                            if (!in_array($key, $newApplicants)) {
                                $newApplicants[$key] =
                                    $this->createSimpleApplicant(
                                        $mainApplicant->getLabel(),
                                        $mainApplicant->getUrl(),
                                        $lang,
                                        $mainApplicant->getCategory(),
                                        $mainApplicant->getSortOrder(),
                                        $mainApplicant->getParameters()
                                    );
                            }
                        }
                    }
                }
            }
        }

        return $newApplicants;
    }

    /**
     * @param \WP_Post $post
     * @param string $lang
     * @param string|null $label
     * @param bool $withData
     * @param string|null $category
     * @param array $params
     * @param int $sortOrder
     * @return BuilderiusTemplateApplicantInterface
     */
    protected function convertPostToApplicant(
        \WP_Post $post,
                 $lang,
                 $label,
                 $withData = false,
                 $category = null,
        array    $params = [],
                 $sortOrder = 10
    )
    {
        $label = sprintf('%s (%s)', $label, $lang);
        return new BuilderiusTemplateApplicant([
            BuilderiusTemplateApplicant::LABEL_FIELD => $label,
            BuilderiusTemplateApplicant::URL_FIELD => get_permalink($post),
            BuilderiusTemplateApplicant::CATEGORY_FIELD => $category,
            BuilderiusTemplateApplicant::PARAMETERS_FIELD => $params,
            BuilderiusTemplateApplicant::DATA_FIELD => $withData ? PostToApplicantDataConverter::convert($post) : [],
            BuilderiusTemplateApplicant::SORT_ORDER_FIELD => $sortOrder
        ]);
    }

    /**
     * @param \WP_Term $category
     * @param string $lang
     * @param array $data
     * @return BuilderiusTemplateApplicant
     */
    protected function createTermApplicant(\WP_Term $term, $taxLabel, $lang, $category, $sortOrder, $data = [])
    {
        return new BuilderiusTemplateApplicant([
            BuilderiusTemplateApplicant::LABEL_FIELD => sprintf('%s %s: %s (%s)', $taxLabel, __('archive'), $term->name, $lang),
            BuilderiusTemplateApplicant::URL_FIELD => get_term_link($term),
            BuilderiusTemplateApplicant::DATA_FIELD => $data,
            BuilderiusTemplateApplicant::CATEGORY_FIELD => $category,
            BuilderiusTemplateApplicant::SORT_ORDER_FIELD => $sortOrder
        ]);
    }

    /**
     * @param string $label
     * @param string $url
     * @param string $lang
     * @param string $category
     * $param int $sortOrder
     * @param array $data
     * @return BuilderiusTemplateApplicant
     */
    protected function createSimpleApplicant($label, $url, $lang, $category, $sortOrder, $data = [])
    {
        $language = \PLL()->model->get_language($lang);
        if ($url === get_site_url()) {
            $langUrl = \PLL()->links_model->home_url($language);
        } elseif (strpos($url, 'p=-1') !== false) {
            $langUrl = sprintf('%s?%s', \PLL()->links_model->home_url($language), explode('?', $url)[1]);
        } else {
            $langUrl = \PLL()->links_model->switch_language_in_link($url, $language);
        }

        return new BuilderiusTemplateApplicant([
            BuilderiusTemplateApplicant::LABEL_FIELD => sprintf('%s (%s)', $label, $lang),
            BuilderiusTemplateApplicant::URL_FIELD => $langUrl,
            BuilderiusTemplateApplicant::CATEGORY_FIELD => $category,
            BuilderiusTemplateApplicant::SORT_ORDER_FIELD => $sortOrder,
            BuilderiusTemplateApplicant::DATA_FIELD => $data
        ]);
    }
}