<?php

namespace BuilderiusPro\Bundle\PolylangBundle\EventListener;

use Builderius\Bundle\TemplateBundle\ApplyRule\Checker\BuilderiusTemplateApplyRulesChecker;
use Builderius\Bundle\TemplateBundle\ApplyRule\Converter\ApplyRuleConfigConverter;
use Builderius\Bundle\TemplateBundle\Event\ApplyRuleSingleConfigEvent;
use Builderius\Symfony\Component\ExpressionLanguage\ExpressionLanguage;

class PolylangApplyRulesCheckingEventListener
{
    /**
     * @var ExpressionLanguage
     */
    private $expressionLanguage;

    /**
     * @var ApplyRuleSingleConfigEvent
     */
    private $event;

    /**
     * @param ExpressionLanguage $expressionLanguage
     */
    public function __construct(
        ExpressionLanguage $expressionLanguage
    ) {
        $this->expressionLanguage = $expressionLanguage;
    }

    /**
     * @param ApplyRuleSingleConfigEvent $event
     * @throws \Exception
     */
    public function onSingleConfigApplyRuleChecking(ApplyRuleSingleConfigEvent $event)
    {
        try {
            $this->event = $event;
            $expressionResult = $event->getExpressionResult();
            $applyRulesConfig = $event->getApplyRulesConfig();
            if (isset($applyRulesConfig['categories']['multilingual'])) {
                foreach (ApplyRuleConfigConverter::convert($applyRulesConfig['categories']['multilingual']) as $key => $config) {
                    if (in_array($key, BuilderiusTemplateApplyRulesChecker::CONJUNCTIONS)) {
                        $expressionResult = $this->checkGroup('multilingual', $config, $key);
                    } else {
                        $expressionResult = $this->checkSingle('multilingual', $config, $key);
                    }
                }
            }
            $event->setExpressionResult($expressionResult);
        } catch (\Throwable $e) {
            return;
        }
    }


    /**
     * @param string $categoryName
     * @param array $subConfigs
     * @param string $conjunction
     * @return bool
     */
    private function checkGroup($categoryName, array $subConfigs, $conjunction) {
        $subConfigsResults = [];
        foreach ($subConfigs as $subConfig) {
            foreach ($subConfig as $key => $subSubConfig) {
                if (in_array($key, BuilderiusTemplateApplyRulesChecker::CONJUNCTIONS)) {
                    $subConfigsResults[] = $this->checkGroup($categoryName, $subSubConfig, $key);
                } else {
                    $subConfigsResults[] = $this->checkSingle($categoryName, $subSubConfig, $key);
                }
            }
        }
        if ($conjunction === 'and') {
            return !in_array(false, $subConfigsResults);
        } else {
            return in_array(true, $subConfigsResults);
        }
    }

    /**
     * @param string $categoryName
     * @param array $config
     * @param string $operator
     * @return bool
     */
    private function checkSingle($categoryName, array $config, $operator)
    {
        $rule = $config[0]['var'];
        $argument = $config[1];

        $mainRule = $this->event->getRule();
        $mainExpression = $this->event->getExpression();
        $mainExpressionResult = $this->event->getExpressionResult();
        $category = $this->event->getCategory();
        if ($rule === 'polylang.all') {
            $pllLang = 'all';
        } else {
            $pllLang = $argument;
        }
        if ($mainRule[0] === 'singular') {
            global $wp_query;
            $post = $wp_query->post;
            if ($post instanceof \WP_Post) {
                $pllTranslations = pll_get_post_translations($post->ID);
                $pllTranslationsReverted = array_flip($pllTranslations);
                if ($pllLang === 'all') {
                    if ($mainExpressionResult === true) {
                        return $mainExpressionResult;
                    } else {
                        return $this->processSingularTranslations(
                            $pllTranslationsReverted,
                            $post,
                            $mainExpression,
                            $category->getVariableAlias()
                        );
                    }
                } else {
                    if ($mainExpressionResult === true) {
                        if ($pllTranslationsReverted[$post->ID] === $pllLang) {
                            return true;
                        } else {
                            return false;
                        }
                    } else {
                        return $this->processSingularTranslations(
                            $pllTranslationsReverted,
                            $post,
                            $mainExpression,
                            $category->getVariableAlias()
                        );
                    }
                }
            }
        } elseif ($mainRule[0] === 'archive' && in_array($mainRule[1], ['category_archive', 'tag_archive', 'taxonomy_archive'])) {
            global $wp_query;
            /** @var \WP_Term $term */
            $term = $wp_query->queried_object;
            if ($term instanceof \WP_Term) {
                $pllTranslations = pll_get_term_translations($term->term_id);
                $pllTranslationsReverted = array_flip($pllTranslations);
                if ($pllLang === 'all') {
                    if ($mainExpressionResult === true) {
                        return $mainExpressionResult;
                    } else {
                        return $this->processTermTranslations(
                            $pllTranslationsReverted,
                            $term,
                            $mainExpression,
                            $category->getVariableAlias()
                        );
                    }
                } else {
                    if ($mainExpressionResult === true) {
                        if ($pllTranslationsReverted[$term->term_id] === $pllLang) {
                            return true;
                        } else {
                            return false;
                        }
                    } else {
                        return $this->processTermTranslations(
                            $pllTranslationsReverted,
                            $term,
                            $mainExpression,
                            $category->getVariableAlias()
                        );
                    }
                }
            }
        }

        return $mainExpressionResult;
    }

    /**
     * @param array $pllTranslationsReverted
     * @param \WP_Post $post
     * @param string $mainExpression
     * @param string $categoryAlias
     * @return bool
     */
    private function processSingularTranslations(
        array $pllTranslationsReverted,
        \WP_Post $post,
        $mainExpression,
        $categoryAlias
    ) {
        foreach ($pllTranslationsReverted as $id => $lang) {
            if ($id !== $post->ID) {
                if ($post->post_type === 'page') {
                    $key = 'page_id';
                } else {
                    $key = 'p';
                }
                $query = new \WP_Query([
                    $key => $id,
                    'lang' => $lang,
                ]);
                $expressionResult = $this->expressionLanguage
                    ->evaluate(
                        $mainExpression,
                        [
                            $categoryAlias => $query,
                            'argument' => $this->event->getArgument()
                        ]
                    );
                if (true === $expressionResult) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * @param array $pllTranslationsReverted
     * @param \WP_Term $term
     * @param string $mainExpression
     * @param string $categoryAlias
     * @return bool
     */
    private function processTermTranslations(
        array $pllTranslationsReverted,
        \WP_Term $term,
        $mainExpression,
        $categoryAlias
    ) {
        foreach ($pllTranslationsReverted as $id => $lang) {
            if ($id !== $term->term_id) {
                if ($term->taxonomy === 'category') {
                    $args = [
                        'lang' => $lang,
                        'cat' => $id
                    ];
                } elseif ($term->taxonomy === 'post_tag') {
                    $args = [
                        'lang' => $lang,
                        'tag_id' => $id
                    ];
                } else {
                    $args = [
                        'lang' => $lang,
                        'tax_query' => [
                            'relation' => 'AND',
                            [
                                'taxonomy' => $term->taxonomy,
                                'field' => 'id',
                                'terms' => [$id]
                            ]
                        ]
                    ];
                }
                $query = new \WP_Query($args);
                $expressionResult = $this->expressionLanguage
                    ->evaluate(
                        $mainExpression,
                        [
                            $categoryAlias => $query,
                            'argument' => $this->event->getArgument()
                        ]
                    );
                if (true === $expressionResult) {
                    return true;
                }
            }
        }

        return false;
    }
}