<?php

namespace BuilderiusPro\Bundle\SettingBundle\EventListener;

use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\AssetDataItem;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\AssetInterface;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\Script;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\Style;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\EventDispatcher\ConditionAwareEventListener;

abstract class AbstractLibrariesRegistrationEventListener extends ConditionAwareEventListener
{
    /**
     * @param AssetInterface[] $assets
     * @param $libsContent
     * @return AssetInterface[]
     */
    protected function processAssets(array $assets, $libsContent)
    {
        if (is_array($libsContent) && !empty($libsContent)) {
            foreach ($libsContent as $libsContentItem) {
                if (is_array($libsContentItem['registeredLibs'])) {
                    foreach ($libsContentItem['registeredLibs'] as $libConfig) {
                        $asset = $this->createAssetForRegisteredLibrary((array)$libConfig);
                        if ($asset instanceof AssetInterface && !in_array($asset, $assets)) {
                            $assets[] = $asset;
                        }
                    }
                }
                if (is_array($libsContentItem['notRegisteredLibs'])) {
                    foreach ($libsContentItem['notRegisteredLibs'] as $libConfig) {
                        $asset = $this->createAssetForNotRegisteredLibrary((array)$libConfig);
                        if ($asset instanceof AssetInterface && !in_array($asset, $assets)) {
                            $assets[] = $asset;
                        }
                    }
                }
            }
        }

        return $assets;
    }

    /**
     * @param array $config
     * @return AssetInterface
     */
    protected function createAssetForRegisteredLibrary(array $config)
    {
        if ($config['handle'] === null) {
            return null;
        }
        $asset = null;
        if ($config['type'] === 'script') {
            $asset = new Script([
                Script::CATEGORY_FIELD => 'frontend',
                Script::HANDLE_FIELD => $config['handle']
            ]);
        } elseif ($config['type'] === 'style') {
            $asset = new Style([
                Style::CATEGORY_FIELD => 'frontend',
                Style::HANDLE_FIELD => $config['handle']
            ]);
        }
        if ($asset !== null && isset($config['attributes']) && is_array($config['attributes'])) {
            foreach ($config['attributes'] as $attribute) {
                $attribute = (array)$attribute;
                if (array_key_exists('a1', $attribute) && array_key_exists('b1', $attribute)) {
                    $dataItem = new AssetDataItem([
                        AssetDataItem::GROUP_FIELD => 'htmlAttributes',
                        AssetDataItem::KEY_FIELD => $attribute['a1'],
                        AssetDataItem::VALUE_FIELD => $attribute['b1']
                    ]);
                    $asset->addAssetDataItem($dataItem);
                }
            }
        }

        return $asset;
    }

    /**
     * @param array $config
     * @return AssetInterface
     */
    protected function createAssetForNotRegisteredLibrary(array $config)
    {
        if ($config['src'] === null) {
            return null;
        }
        $asset = null;
        if ($config['type'] === 'script') {
            $asset = new Script([
                Script::CATEGORY_FIELD => 'frontend',
                Script::HANDLE_FIELD => $config['handle'],
                Script::SOURCE_FIELD => $config['src'],
                Script::IN_FOOTER_FIELD => $config['inFooter'] ? : false
            ]);
        } elseif ($config['type'] === 'style') {
            $asset = new Style([
                Style::CATEGORY_FIELD => 'frontend',
                Style::HANDLE_FIELD => $config['handle'],
                Style::SOURCE_FIELD => $config['src'],
            ]);
        }
        if ($asset !== null && isset($config['attributes']) && is_array($config['attributes'])) {
            foreach ($config['attributes'] as $attribute) {
                $attribute = (array)$attribute;
                if (array_key_exists('a1', $attribute) && array_key_exists('b1', $attribute)) {
                    $dataItem = new AssetDataItem([
                        AssetDataItem::GROUP_FIELD => 'htmlAttributes',
                        AssetDataItem::KEY_FIELD => $attribute['a1'],
                        AssetDataItem::VALUE_FIELD => $attribute['b1']
                    ]);
                    $asset->addAssetDataItem($dataItem);
                }
            }
        }

        return $asset;
    }
}