<?php

namespace BuilderiusPro\Bundle\SettingBundle\EventListener;

use Builderius\Bundle\DeliverableBundle\Provider\BuilderiusDeliverableProviderInterface;
use Builderius\Bundle\TemplateBundle\Event\AssetRemovalEvent;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModuleProviderInterface;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModulesProviderInterface;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateLibrariesContentProvider;

class AssetRemovalCheckingInFrontendModeEventListener extends AbstractAssetRemovalCheckingEventListener
{
    /**
     * @var BuilderiusDeliverableProviderInterface
     */
    private $deliverableProvider;

    /**
     * @var DeliverableTemplateSubModuleProviderInterface
     */
    private $dtsmProvider;

    /**
     * @var DeliverableTemplateSubModulesProviderInterface
     */
    private $dhtsmsProvider;

    /**
     * @param BuilderiusDeliverableProviderInterface $deliverableProvider
     * @param DeliverableTemplateSubModuleProviderInterface $dtsmProvider
     * @param DeliverableTemplateSubModulesProviderInterface $dhtsmsProvider
     */
    public function __construct(
        BuilderiusDeliverableProviderInterface $deliverableProvider,
        DeliverableTemplateSubModuleProviderInterface $dtsmProvider,
        DeliverableTemplateSubModulesProviderInterface $dhtsmsProvider
    ) {
        $this->deliverableProvider = $deliverableProvider;
        $this->dtsmProvider = $dtsmProvider;
        $this->dhtsmsProvider = $dhtsmsProvider;
    }

    /**
     * @param AssetRemovalEvent $event
     * @throws \Exception
     */
    public function beforeAssetRemoval(AssetRemovalEvent $event)
    {
        $templateSubModule = $this->dtsmProvider->getTemplateSubModule();
        if (!$templateSubModule) {
            $hookTemplateSubModules = $this->dhtsmsProvider->getTemplateSubModules();
            if (!empty($hookTemplateSubModules)) {
                $templateSubModule = reset($hookTemplateSubModules);
            }
        }
        if ($templateSubModule) {
            $technology = $templateSubModule->getTechnology();
            $deliverable = $this->deliverableProvider->getDeliverable();
            $gssAll = $deliverable->getSubModules('global_settings_set', $technology);
            if (!empty($gssAll)) {
                $gssAll = reset($gssAll);
                if ($libsContent = $gssAll->getContent(BuilderiusTemplateLibrariesContentProvider::CONTENT_TYPE)) {
                    $this->processAsset($event, $libsContent);
                }
            }
        }
    }
}