<?php

namespace BuilderiusPro\Bundle\SettingBundle\EventListener;

use Builderius\Bundle\DeliverableBundle\Provider\BuilderiusDeliverableProviderInterface;
use Builderius\Bundle\TemplateBundle\Event\AssetRemovalEvent;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateLibrariesContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule\DeliverableTemplatePartSubModuleProviderInterface;

class AssetRemovalCheckingInTemplatePartFrontendModeEventListener extends AbstractAssetRemovalCheckingEventListener
{
    /**
     * @var BuilderiusDeliverableProviderInterface
     */
    private $deliverableProvider;

    /**
     * @var DeliverableTemplatePartSubModuleProviderInterface
     */
    private $dtpsmProvider;

    /**
     * @param BuilderiusDeliverableProviderInterface $deliverableProvider
     * @param DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
     */
    public function __construct(
        BuilderiusDeliverableProviderInterface $deliverableProvider,
        DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
    ) {
        $this->deliverableProvider = $deliverableProvider;
        $this->dtpsmProvider = $dtpsmProvider;
    }

    /**
     * @param AssetRemovalEvent $event
     * @throws \Exception
     */
    public function beforeAssetRemoval(AssetRemovalEvent $event)
    {
        $templatePartSubModule = $this->dtpsmProvider->getTemplatePartSubModule($_GET['builderius-template-part-type']);
        if ($templatePartSubModule) {
            $technology = $templatePartSubModule->getTechnology();
            $deliverable = $this->deliverableProvider->getDeliverable();
            $gssAll = $deliverable->getSubModules('global_settings_set', $technology);
            if (!empty($gssAll)) {
                $gssAll = reset($gssAll);
                if ($libsContent = $gssAll->getContent(BuilderiusTemplateLibrariesContentProvider::CONTENT_TYPE)) {
                    $this->processAsset($event, $libsContent);
                }
            }
        }
    }
}