<?php

namespace BuilderiusPro\Bundle\SettingBundle\EventListener;

use Builderius\Bundle\DeliverableBundle\Provider\BuilderiusDeliverableProviderInterface;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModuleProviderInterface;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModulesProviderInterface;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\InlineAssetsContainingEvent;
use BuilderiusPro\Bundle\TemplateBundle\EventListener\AbstractAssetsRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateCustomCssContentProvider;

class CustomCssRegistrationInFrontendModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusDeliverableProviderInterface
     */
    private $deliverableProvider;

    /**
     * @var DeliverableTemplateSubModuleProviderInterface
     */
    private $dtsmProvider;

    /**
     * @var DeliverableTemplateSubModulesProviderInterface
     */
    private $dhtsmsProvider;

    /**
     * @param BuilderiusDeliverableProviderInterface $deliverableProvider
     * @param DeliverableTemplateSubModuleProviderInterface $dtsmProvider
     * @param DeliverableTemplateSubModulesProviderInterface $dhtsmsProvider
     */
    public function __construct(
        BuilderiusDeliverableProviderInterface $deliverableProvider,
        DeliverableTemplateSubModuleProviderInterface $dtsmProvider,
        DeliverableTemplateSubModulesProviderInterface $dhtsmsProvider
    ) {
        $this->deliverableProvider = $deliverableProvider;
        $this->dtsmProvider = $dtsmProvider;
        $this->dhtsmsProvider = $dhtsmsProvider;
    }

    /**
     * @param InlineAssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(InlineAssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $templateSubModule = $this->dtsmProvider->getTemplateSubModule();
        if (!$templateSubModule) {
            $hookTemplateSubModules = $this->dhtsmsProvider->getTemplateSubModules();
            if (!empty($hookTemplateSubModules)) {
                $templateSubModule = reset($hookTemplateSubModules);
            }
        }
        if ($templateSubModule) {
            $technology = $templateSubModule->getTechnology();
            $deliverable = $this->deliverableProvider->getDeliverable();
            $gssAll = $deliverable->getSubModules('global_settings_set', $technology);
            if (!empty($gssAll)) {
                $gssAll = reset($gssAll);
                if ($cssContent = $gssAll->getContent(BuilderiusTemplateCustomCssContentProvider::CONTENT_TYPE)) {
                    if (is_array($cssContent) && !empty($cssContent)) {
                        foreach ($cssContent as $cssContentItem) {
                            $asset = $this->createAsset((array)$cssContentItem, 'style', 'builderius-css');
                            if (!in_array($asset, $assets)) {
                                $assets[] = $asset;
                            }
                        }
                    }
                }
            }
        }
        $event->setAssets($assets);
    }
}