<?php

namespace BuilderiusPro\Bundle\SettingBundle\EventListener;

use Builderius\Bundle\DeliverableBundle\Provider\BuilderiusDeliverableProviderInterface;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\InlineAssetsContainingEvent;
use BuilderiusPro\Bundle\TemplateBundle\EventListener\AbstractAssetsRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateCustomCssContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule\DeliverableTemplatePartSubModuleProviderInterface;

class CustomCssRegistrationInTemplatePartFrontendModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusDeliverableProviderInterface
     */
    private $deliverableProvider;

    /**
     * @var DeliverableTemplatePartSubModuleProviderInterface
     */
    private $dtpsmProvider;

    /**
     * @param BuilderiusDeliverableProviderInterface $deliverableProvider
     * @param DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
     */
    public function __construct(
        BuilderiusDeliverableProviderInterface $deliverableProvider,
        DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
    ) {
        $this->deliverableProvider = $deliverableProvider;
        $this->dtpsmProvider = $dtpsmProvider;
    }

    /**
     * @param InlineAssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(InlineAssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $templateSubModule = $this->dtpsmProvider->getTemplatePartSubModule($_GET['builderius-template-part-type']);
        if ($templateSubModule) {
            $technology = $templateSubModule->getTechnology();
            $deliverable = $this->deliverableProvider->getDeliverable();
            $gssAll = $deliverable->getSubModules('global_settings_set', $technology);
            if (!empty($gssAll)) {
                $gssAll = reset($gssAll);
                if ($cssContent = $gssAll->getContent(BuilderiusTemplateCustomCssContentProvider::CONTENT_TYPE)) {
                    if (is_array($cssContent) && !empty($cssContent)) {
                        foreach ($cssContent as $cssContentItem) {
                            $assets[] = $this->createAsset((array)$cssContentItem, 'style', 'builderius-css');
                        }
                    }
                }
            }
        }
        $event->setAssets($assets);
    }
}