<?php

namespace BuilderiusPro\Bundle\SettingBundle\EventListener;

use Builderius\Bundle\SettingBundle\Factory\BuilderiusGlobalSettingsSetFromPostFactory;
use Builderius\Bundle\SettingBundle\Registration\BuilderiusGlobalSettingsSetPostType;
use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\InlineAssetsContainingEvent;
use BuilderiusPro\Bundle\TemplateBundle\EventListener\AbstractAssetsRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateCustomCssContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class CustomCssRegistrationInTemplatePartPreviewModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;

    /**
     * @var BuilderiusGlobalSettingsSetFromPostFactory
     */
    private $globalSettingsSetFromPostFactory;

    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    private $builderiusCache;

    /**
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     * @param BuilderiusGlobalSettingsSetFromPostFactory $globalSettingsSetFromPostFactory
     * @param \WP_Query $wpQuery
     * @param BuilderiusRuntimeObjectCache $builderiusCache
     */
    public function __construct(
        BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider,
        BuilderiusGlobalSettingsSetFromPostFactory $globalSettingsSetFromPostFactory,
        \WP_Query $wpQuery,
        BuilderiusRuntimeObjectCache $builderiusCache
    ) {
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;
        $this->globalSettingsSetFromPostFactory = $globalSettingsSetFromPostFactory;
        $this->wpQuery = $wpQuery;
        $this->builderiusCache = $builderiusCache;
    }

    /**
     * @param InlineAssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(InlineAssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $template = $this->builderiusTemplatePartProvider->getTemplatePart($_GET['builderius-template-part-type']);
        if ($template) {
            $technologyName = $template->getTechnology();
            $posts = $this->builderiusCache->get(sprintf('builderius_gss_posts_%s', $technologyName));
            if (false == $posts) {
                $queryArgs = [
                    'post_type' => BuilderiusGlobalSettingsSetPostType::POST_TYPE,
                    'post_status' => get_post_stati(),
                    'name' => $technologyName,
                    'posts_per_page' => -1,
                    'no_found_rows' => true,
                    'orderby' => 'name',
                    'order' => 'ASC'
                ];
                $posts = $this->wpQuery->query($queryArgs);
                $this->builderiusCache->set(sprintf('builderius_gss_posts_%s', $technologyName), $posts);
            }
            foreach ($posts as $post) {
                $globalSettingsSet = $this->globalSettingsSetFromPostFactory->createGlobalSettingsSet($post);
                $branch = $globalSettingsSet->getActiveBranch();
                if ($branch) {
                    if ($cssContent = $branch->getContent(BuilderiusTemplateCustomCssContentProvider::CONTENT_TYPE)) {
                        if (is_array($cssContent) && !empty($cssContent)) {
                            foreach ($cssContent as $cssContentItem) {
                                $assets[] = $this->createAsset((array)$cssContentItem, 'style', 'builderius-css');
                            }
                        }
                    } else {
                        $commit = $branch->getActiveCommit();
                        if ($commit) {
                            if ($cssContent = $commit->getContent(BuilderiusTemplateCustomCssContentProvider::CONTENT_TYPE)) {
                                if (is_array($cssContent) && !empty($cssContent)) {
                                    foreach ($cssContent as $cssContentItem) {
                                        $assets[] = $this->createAsset((array)$cssContentItem, 'style', 'builderius-css');
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        $event->setAssets($assets);
    }
}