<?php

namespace BuilderiusPro\Bundle\SettingBundle\EventListener;

use Builderius\Bundle\SettingBundle\Factory\BuilderiusGlobalSettingsSetFromPostFactory;
use Builderius\Bundle\SettingBundle\Registration\BuilderiusGlobalSettingsSetPostType;
use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplateProviderInterface;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplatesProviderInterface;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\InlineAssetsContainingEvent;
use BuilderiusPro\Bundle\TemplateBundle\EventListener\AbstractAssetsRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateJsContentProvider;

class JsRegistrationInPreviewModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusTemplateProviderInterface
     */
    private $builderiusTemplateProvider;

    /**
     * @var BuilderiusTemplatesProviderInterface
     */
    private $builderiusHookTemplatesProvider;

    /**
     * @var BuilderiusGlobalSettingsSetFromPostFactory
     */
    private $globalSettingsSetFromPostFactory;

    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @var BuilderiusRuntimeObjectCache
     */
    private $builderiusCache;

    /**
     * @param BuilderiusTemplateProviderInterface $builderiusTemplateProvider
     * @param BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider
     * @param BuilderiusGlobalSettingsSetFromPostFactory $globalSettingsSetFromPostFactory
     * @param \WP_Query $wpQuery
     * @param BuilderiusRuntimeObjectCache $builderiusCache
     */
    public function __construct(
        BuilderiusTemplateProviderInterface $builderiusTemplateProvider,
        BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider,
        BuilderiusGlobalSettingsSetFromPostFactory $globalSettingsSetFromPostFactory,
        \WP_Query $wpQuery,
        BuilderiusRuntimeObjectCache $builderiusCache
    ) {
        $this->builderiusTemplateProvider = $builderiusTemplateProvider;
        $this->builderiusHookTemplatesProvider = $builderiusHookTemplatesProvider;
        $this->globalSettingsSetFromPostFactory = $globalSettingsSetFromPostFactory;
        $this->wpQuery = $wpQuery;
        $this->builderiusCache = $builderiusCache;
    }

    /**
     * @param InlineAssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(InlineAssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $template = $this->builderiusTemplateProvider->getTemplate();
        if (!$template) {
            $hookTemplates = $this->builderiusHookTemplatesProvider->getTemplates();
            if (!empty($hookTemplates)) {
                $template = reset($hookTemplates);
            }
        }
        if ($template) {
            $technologyName = $template->getTechnology();
            $posts = $this->builderiusCache->get(sprintf('builderius_gss_posts_%s', $technologyName));
            if (false == $posts) {
                $queryArgs = [
                    'post_type' => BuilderiusGlobalSettingsSetPostType::POST_TYPE,
                    'post_status' => get_post_stati(),
                    'name' => $technologyName,
                    'posts_per_page' => -1,
                    'no_found_rows' => true,
                    'orderby' => 'name',
                    'order' => 'ASC'
                ];
                $posts = $this->wpQuery->query($queryArgs);
                $this->builderiusCache->set(sprintf('builderius_gss_posts_%s', $technologyName), $posts);
            }
            foreach ($posts as $post) {
                $globalSettingsSet = $this->globalSettingsSetFromPostFactory->createGlobalSettingsSet($post);
                $branch = $globalSettingsSet->getActiveBranch();
                if ($branch) {
                    if ($jsContent = $branch->getContent(BuilderiusTemplateJsContentProvider::CONTENT_TYPE)) {
                        if (is_array($jsContent) && !empty($jsContent)) {
                            foreach ($jsContent as $jsContentItem) {
                                $asset = $this->createAsset((array)$jsContentItem, 'script', 'builderius-js');
                                if (!in_array($asset, $assets)) {
                                    $assets[] = $asset;
                                }
                            }
                        }
                    } else {
                        $commit = $branch->getActiveCommit();
                        if ($commit) {
                            if ($jsContent = $commit->getContent(BuilderiusTemplateJsContentProvider::CONTENT_TYPE)) {
                                if (is_array($jsContent) && !empty($jsContent)) {
                                    foreach ($jsContent as $jsContentItem) {
                                        $asset = $this->createAsset((array)$jsContentItem, 'script', 'builderius-js');
                                        if (!in_array($asset, $assets)) {
                                            $assets[] = $asset;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        $event->setAssets($assets);
    }
}