<?php

namespace BuilderiusPro\Bundle\SettingBundle\EventListener;

use Builderius\Bundle\DeliverableBundle\Provider\BuilderiusDeliverableProviderInterface;
use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModuleProviderInterface;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModulesProviderInterface;
use BuilderiusPro\Bundle\TemplateBundle\EventListener\AbstractAssetsRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateStringTranslationsContentProvider;

class StringTranslationsRegistrationInFrontendModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusDeliverableProviderInterface
     */
    private $deliverableProvider;

    /**
     * @var DeliverableTemplateSubModuleProviderInterface
     */
    private $dtsmProvider;

    /**
     * @var DeliverableTemplateSubModulesProviderInterface
     */
    private $dhtsmsProvider;

    /**
     * @param BuilderiusDeliverableProviderInterface $deliverableProvider
     * @param DeliverableTemplateSubModuleProviderInterface $dtsmProvider
     * @param DeliverableTemplateSubModulesProviderInterface $dhtsmsProvider
     */
    public function __construct(
        BuilderiusDeliverableProviderInterface $deliverableProvider,
        DeliverableTemplateSubModuleProviderInterface $dtsmProvider,
        DeliverableTemplateSubModulesProviderInterface $dhtsmsProvider
    ) {
        $this->deliverableProvider = $deliverableProvider;
        $this->dtsmProvider = $dtsmProvider;
        $this->dhtsmsProvider = $dhtsmsProvider;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeStringTranslation(ConfigContainingEvent $event)
    {
        global $l10n;
        if (!isset($l10n['builderius_strings'])) {
            $templateSubModule = $this->dtsmProvider->getTemplateSubModule();
            if (!$templateSubModule) {
                $hookTemplateSubModules = $this->dhtsmsProvider->getTemplateSubModules();
                if (!empty($hookTemplateSubModules)) {
                    $templateSubModule = reset($hookTemplateSubModules);
                }
            }
            if ($templateSubModule) {
                $config = $event->getConfig();
                $locale = determine_locale();
                $technology = $templateSubModule->getTechnology();
                $deliverable = $this->deliverableProvider->getDeliverable();
                $gssAll = $deliverable->getSubModules('global_settings_set', $technology);
                if (!empty($gssAll)) {
                    $gssAll = reset($gssAll);
                    if ($stContent = $gssAll->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                        if (is_array($stContent) && isset($stContent[$locale])) {
                            if (empty($config)) {
                                $config = $stContent[$locale];
                            } else {
                                foreach ($stContent[$locale] as $origString => $translatedString) {
                                    $config[$origString] = $translatedString;
                                }
                            }
                        }
                    }
                }
                $event->setConfig($config);
            }
        }
    }
}