<?php

namespace BuilderiusPro\Bundle\SettingBundle\EventListener;

use Builderius\Bundle\DeliverableBundle\Provider\BuilderiusDeliverableProviderInterface;
use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use BuilderiusPro\Bundle\TemplateBundle\EventListener\AbstractAssetsRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateStringTranslationsContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\DeliverableTemplatePartSubModule\DeliverableTemplatePartSubModuleProviderInterface;

class StringTranslationsRegistrationInTemplatePartFrontendModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusDeliverableProviderInterface
     */
    private $deliverableProvider;

    /**
     * @var DeliverableTemplatePartSubModuleProviderInterface
     */
    private $dtpsmProvider;

    /**
     * @param BuilderiusDeliverableProviderInterface $deliverableProvider
     * @param DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
     */
    public function __construct(
        BuilderiusDeliverableProviderInterface $deliverableProvider,
        DeliverableTemplatePartSubModuleProviderInterface $dtpsmProvider
    ) {
        $this->deliverableProvider = $deliverableProvider;
        $this->dtpsmProvider = $dtpsmProvider;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeStringTranslation(ConfigContainingEvent $event)
    {
        global $l10n;
        if (!isset($l10n['builderius_strings'])) {
            $templatePartSubModule = $this->dtpsmProvider->getTemplatePartSubModule($_GET['builderius-template-part-type']);
            if ($templatePartSubModule) {
                $config = $event->getConfig();
                $locale = determine_locale();
                $technology = $templatePartSubModule->getTechnology();
                $deliverable = $this->deliverableProvider->getDeliverable();
                $gssAll = $deliverable->getSubModules('global_settings_set', $technology);
                if (!empty($gssAll)) {
                    $gssAll = reset($gssAll);
                    if ($stContent = $gssAll->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                        if (is_array($stContent) && isset($stContent[$locale])) {
                            if (empty($config)) {
                                $config = $stContent[$locale];
                            } else {
                                foreach ($stContent[$locale] as $origString => $translatedString) {
                                    $config[$origString] = $translatedString;
                                }
                            }
                        }
                    }
                }
                $event->setConfig($config);
            }
        }
    }
}