<?php

namespace BuilderiusPro\Bundle\SettingBundle\EventListener;

use Builderius\Bundle\SettingBundle\Factory\BuilderiusGlobalSettingsSetFromPostFactory;
use Builderius\Bundle\SettingBundle\Registration\BuilderiusGlobalSettingsSetPostType;
use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use BuilderiusPro\Bundle\TemplateBundle\EventListener\AbstractAssetsRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateStringTranslationsContentProvider;
use BuilderiusPro\Bundle\ComponentBundle\Provider\TemplatePart\BuilderiusTemplatePartProviderInterface;

class StringTranslationsRegistrationInTemplatePartPreviewModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusTemplatePartProviderInterface
     */
    private $builderiusTemplatePartProvider;

    /**
     * @var BuilderiusGlobalSettingsSetFromPostFactory
     */
    protected $globalSettingsSetFromPostFactory;

    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @param BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider
     * @param BuilderiusGlobalSettingsSetFromPostFactory $globalSettingsSetFromPostFactory
     * @param \WP_Query $wpQuery
     */
    public function __construct(
        BuilderiusTemplatePartProviderInterface $builderiusTemplatePartProvider,
        BuilderiusGlobalSettingsSetFromPostFactory $globalSettingsSetFromPostFactory,
        \WP_Query $wpQuery
    ) {
        $this->builderiusTemplatePartProvider = $builderiusTemplatePartProvider;
        $this->globalSettingsSetFromPostFactory = $globalSettingsSetFromPostFactory;
        $this->wpQuery = $wpQuery;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeStringTranslation(ConfigContainingEvent $event)
    {
        global $l10n;
        if (!isset($l10n['builderius_strings'])) {
            $templatePart = $this->builderiusTemplatePartProvider->getTemplatePart($_GET['builderius-template-part-type']);
            if ($templatePart) {
                $config = $event->getConfig();
                $technologyName = $templatePart->getTechnology();
                $locale = determine_locale();
                $queryArgs = [
                    'post_type' => BuilderiusGlobalSettingsSetPostType::POST_TYPE,
                    'post_status' => get_post_stati(),
                    'name' => $technologyName,
                    'orderby' => 'name',
                    'order' => 'ASC'
                ];
                $posts = $this->wpQuery->query($queryArgs);
                foreach ($posts as $post) {
                    $globalSettingsSet = $this->globalSettingsSetFromPostFactory->createGlobalSettingsSet($post);
                    $branch = $globalSettingsSet->getActiveBranch();
                    if ($branch) {
                        if ($stContent = $branch->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                            if (is_array($stContent) && isset($stContent[$locale])) {
                                if (empty($config)) {
                                    $config = $stContent[$locale];
                                } else {
                                    foreach ($stContent[$locale] as $origString => $translatedString) {
                                        $config[$origString] = $translatedString;
                                    }
                                }
                            }
                        } else {
                            $commit = $branch->getActiveCommit();
                            if ($commit) {
                                if ($stContent = $commit->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                                    if (is_array($stContent) && isset($stContent[$locale])) {
                                        if (empty($config)) {
                                            $config = $stContent[$locale];
                                        } else {
                                            foreach ($stContent[$locale] as $origString => $translatedString) {
                                                $config[$origString] = $translatedString;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                $event->setConfig($config);
            }
        }
    }
}