<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategoriesProvider;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategoriesProviderInterface;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategory;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;
use BuilderiusPro\Bundle\TemplateBundle\ApplyRule\Provider\AvailableCustomPostTypesProvider;

class BuilderiusTemplateAllCustomPostsApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @var AvailableCustomPostTypesProvider
     */
    private $customPostTypesProvider;

    /**
     * @var BuilderiusTemplateApplicantCategoriesProviderInterface|BuilderiusTemplateApplicantCategoriesProvider
     */
    private $applicantCategoriesProvider;

    /**
     * @param \WP_Query $wpQuery
     * @param AvailableCustomPostTypesProvider $customPostTypesProvider
     * @param BuilderiusTemplateApplicantCategoriesProviderInterface $applicantCategoriesProvider
     */
    public function __construct(
        \WP_Query $wpQuery,
        AvailableCustomPostTypesProvider $customPostTypesProvider,
        BuilderiusTemplateApplicantCategoriesProviderInterface $applicantCategoriesProvider
    ) {
        $this->wpQuery = $wpQuery;
        $this->customPostTypesProvider = $customPostTypesProvider;
        $this->applicantCategoriesProvider = $applicantCategoriesProvider;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $postTypes = [];
        $customPostLabels = [];
        foreach ($this->customPostTypesProvider->getArguments() as $argument) {
            if (!$this->applicantCategoriesProvider->hasCategory($argument['value'] . '_posts')) {
                $postTypeObject = get_post_type_object($argument['value']);
                $category = new BuilderiusTemplateApplicantCategory([
                    BuilderiusTemplateApplicantCategory::NAME_FIELD => $argument['value'] . '_posts',
                    BuilderiusTemplateApplicantCategory::LABEL_FIELD => $postTypeObject->labels->name,
                    BuilderiusTemplateApplicantCategory::SORT_ORDER_FIELD => 40
                ]);
                $this->applicantCategoriesProvider->addCategory($category);
            }
            $postTypes[] = $argument['value'];
            $customPostLabels[$argument['value']] = $argument['title'];
        }
        $statuses = get_post_stati();
        unset($statuses['auto-draft']);

        /** @var \WP_Post[] $posts */
        $customPosts = $this->wpQuery->query([
            'lang' => '',
            'posts_per_page' => -1,
            'post_type' => $postTypes,
            'post_status' => $statuses,
        ]);
        $applicants = [];
        foreach ($customPosts as $post) {
            $applicants[sprintf('singular.single.%s.%s', $post->post_type, $post->ID)] =
                $this->convertPostToApplicant($post, sprintf('%s: %s', $customPostLabels[$post->post_type], $post->post_title), $withData, $post->post_type . '_posts');
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        $object = $changeset->getObjectAfter();
        $postTypes = [];
        $customPostLabels = [];
        foreach ($this->customPostTypesProvider->getArguments() as $argument) {
            if (!$this->applicantCategoriesProvider->hasCategory($argument['value'] . '_posts')) {
                $postTypeObject = get_post_type_object($argument['value']);
                $category = new BuilderiusTemplateApplicantCategory([
                    BuilderiusTemplateApplicantCategory::NAME_FIELD => $argument['value'] . '_posts',
                    BuilderiusTemplateApplicantCategory::LABEL_FIELD => $postTypeObject->labels->name,
                    BuilderiusTemplateApplicantCategory::SORT_ORDER_FIELD => 40
                ]);
                $this->applicantCategoriesProvider->addCategory($category);
            }
            $postTypes[] = $argument['value'];
            $customPostLabels[$argument['value']] = $argument['title'];
        }
        if (!$object instanceof \WP_Post || !in_array($object->post_type, $postTypes)) {
            return [];
        }

        return [
            sprintf('singular.single.%s.%s', $object->post_type, $object->ID) =>
                $this->convertPostToApplicant($object, sprintf('%s: %s', $customPostLabels[$object->post_type], $object->post_title), $withData, $object->post_type . '_posts')
        ];
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if ($rule === 'singular.single.custom_post.all' && $argument === true && $operator === '==') {
            return true;
        }

        return false;
    }
}