<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;

class BuilderiusTemplateAttachmentPostWithIdApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @param \WP_Query $wpQuery
     */
    public function __construct(\WP_Query $wpQuery)
    {
        $this->wpQuery = $wpQuery;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $statuses = get_post_stati();
        unset($statuses['auto-draft']);

        $posts = $this->wpQuery->query([
            'lang' => '',
            'posts_per_page' => -1,
            'post_type' => ['attachment'],
            'p' => $argument,
            'post_status' => $statuses,
        ]);
        $applicants = [];
        foreach ($posts as $post) {
            $applicants[sprintf('singular.single.%s.%s', $post->post_type, $post->ID)] =
                $this->convertPostToApplicant($post, sprintf('%s: %s', __('Attachment post'), $post->post_title), $withData, 'attachment_posts');
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        $object = $changeset->getObjectAfter();
        if (!$object instanceof \WP_Post || $object->post_type !== 'attachment' || $object->ID !== $argument) {
            return [];
        }

        return [
            sprintf('singular.single.%s.%s', $object->post_type, $object->ID) =>
                $this->convertPostToApplicant($object, sprintf('%s: %s', __('Attachment post'), $object->post_title), $withData, 'attachment_posts')
        ];
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if (
            strpos($rule, 'singular.single.attachment_post.with_id') !== false &&
            is_integer($argument) &&
            $operator === '=='
        ) {
            return true;
        }

        return false;
    }
}