<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicant;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Converter\PostToApplicantDataConverter;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;

class BuilderiusTemplateAuthorArchiveApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @param \WP_Query $wpQuery
     */
    public function __construct(\WP_Query $wpQuery)
    {
        $this->wpQuery = $wpQuery;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $applicants = [];
        $filteredRule = str_replace(
            'archive.',
            '',
            str_replace(
                'author_archive.',
                '',
                $rule
            )
        );
        if ($filteredRule === 'all' && $argument === true && $operator === '==') {
            /** @var \WP_User $author */
            foreach (get_users() as $author) {
                $applicants[sprintf('archive.author_archive.%s', $author->ID)] =
                    $this->processSingleAuthor($author, $withData);
            }
        } elseif ($filteredRule === 'by_author' && is_int($argument) && $operator === '==') {
            $author = get_user_by('ID', $argument);
            if ($author) {
                $applicants[sprintf('archive.author_archive.%s', $author->ID)] =
                    $this->processSingleAuthor($author, $withData);
            }
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        $object = $changeset->getObjectAfter();
        if (!$object instanceof \WP_User) {
            return [];
        }
        $filteredRule = str_replace(
            'archive.',
            '',
            str_replace(
                'author_archive.',
                '',
                $rule
            )
        );
        if ($filteredRule === 'all' && $argument === true && $operator === '==') {
            return [sprintf('archive.author_archive.%s', $object->ID) =>
                $this->processSingleAuthor($object, $withData)];
        } elseif ($filteredRule === 'by_author' && is_int($argument) && $operator === '==') {
            if ($object->ID === $argument) {
                return [sprintf('archive.author_archive.%s', $object->ID) =>
                    $this->processSingleAuthor($object, $withData)];
            }
        }

        return [];
    }

    /**
     * @param \WP_User $author
     * @param bool $withData
     * @return BuilderiusTemplateApplicant
     */
    private function processSingleAuthor(\WP_User $author, $withData)
    {
        $data = [];
        if ($withData) {
            $posts = $this->wpQuery->query([
                'lang' => '',
                'author' => $author->ID,
                'posts_per_page' => -1,
            ]);
            foreach ($posts as $post) {
                $data[] = PostToApplicantDataConverter::convert($post);
            }
        }

        return new BuilderiusTemplateApplicant([
            BuilderiusTemplateApplicant::LABEL_FIELD => sprintf('%s: %s', __('Author archive'), $author->display_name),
            BuilderiusTemplateApplicant::URL_FIELD => get_author_posts_url($author->ID, $author->user_nicename),
            BuilderiusTemplateApplicant::DATA_FIELD => $data,
            BuilderiusTemplateApplicant::CATEGORY_FIELD => 'author_archives'
        ]);
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if ((strpos($rule, 'archive.author_archive') !== false && $operator === '==') || strpos($rule, 'archive.all') === 0) {
            return true;
        }

        return false;
    }
}