<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicant;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Converter\PostToApplicantDataConverter;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;

class BuilderiusTemplateBlogPostsIndexPageApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @param \WP_Query $wpQuery
     */
    public function __construct(\WP_Query $wpQuery)
    {
        $this->wpQuery = $wpQuery;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $applicants = [];
        $postsPageId = get_option('page_for_posts');
        $showOnFront = get_option('show_on_front');
        if ($showOnFront === 'posts') {
            $data = [];
            if ($withData) {
                $posts = $this->wpQuery->query([
                    'lang' => '',
                    'posts_per_page' => -1,
                    'post_type' => ['post'],
                    'post_status' => 'published',
                ]);
                foreach ($posts as $post) {
                    $data[] = PostToApplicantDataConverter::convert($post);
                }
            }
            $applicants['blog_posts_home'] =
            new BuilderiusTemplateApplicant([
                BuilderiusTemplateApplicant::LABEL_FIELD => __('Blog Posts Index Page'),
                BuilderiusTemplateApplicant::URL_FIELD => get_site_url(),
                BuilderiusTemplateApplicant::DATA_FIELD => $data,
                BuilderiusTemplateApplicant::CATEGORY_FIELD => 'special_pages',
                BuilderiusTemplateApplicant::SORT_ORDER_FIELD => 20
            ]);
        } elseif ($postsPageId) {
            $data = [];
            if ($withData) {
                $posts = $this->wpQuery->query([
                    'lang' => '',
                    'posts_per_page' => -1,
                    'post_type' => ['post'],
                    'post_status' => 'published',
                ]);
                foreach ($posts as $post) {
                    $data[] = PostToApplicantDataConverter::convert($post);
                }
            }
            $applicants['blog_posts_home'] =
                new BuilderiusTemplateApplicant([
                    BuilderiusTemplateApplicant::LABEL_FIELD => __('Blog Posts Index Page'),
                    BuilderiusTemplateApplicant::URL_FIELD => get_permalink($postsPageId),
                    BuilderiusTemplateApplicant::DATA_FIELD => $data,
                    BuilderiusTemplateApplicant::CATEGORY_FIELD => 'special_pages',
                    BuilderiusTemplateApplicant::SORT_ORDER_FIELD => 20
                ]);
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        // TODO: Implement getChangeSetApplicants() method.
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if (strpos($rule, 'home') !== false && $argument === true && $operator === '==') {
            return true;
        }

        return false;
    }
}