<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicant;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Converter\PostToApplicantDataConverter;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;

class BuilderiusTemplateCategoryArchiveApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @param \WP_Query $wpQuery
     */
    public function __construct(\WP_Query $wpQuery)
    {
        $this->wpQuery = $wpQuery;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $applicants = [];
        $filteredRule = str_replace(
            'archive.',
            '',
            str_replace(
                'category_archive.',
                '',
                $rule
            )
        );
        if ($filteredRule === 'all' && $argument === true && $operator === '==') {
            /** @var \WP_Term $category */
            foreach (get_categories() as $category) {
                $applicants[sprintf('archive.category_archive.%s', $category->term_id)] =
                    $this->processSingleCategory($category, $withData);
            }
        } elseif ($filteredRule === 'by_category' && is_int($argument) && $operator === '==') {
            /** @var \WP_Term $category */
            $category = get_category($argument);
            if ($category) {
                $applicants[sprintf('archive.category_archive.%s', $category->term_id)] =
                    $this->processSingleCategory($category, $withData);
            }
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        $object = $changeset->getObjectAfter();
        if (!$object instanceof \WP_Term || $object->taxonomy !== 'category') {
            return [];
        }
        $filteredRule = str_replace(
            'archive.',
            '',
            str_replace(
                'category_archive.',
                '',
                $rule
            )
        );
        if ($filteredRule === 'all' && $argument === true && $operator === '==') {
            return [sprintf('archive.category_archive.%s', $object->ID) =>
                $this->processSingleCategory($object, $withData)];
        } elseif ($filteredRule === 'by_category' && is_int($argument) && $operator === '==') {
            if ($object->ID === $argument) {
                return [sprintf('archive.category_archive.%s', $object->ID) =>
                    $this->processSingleCategory($object, $withData)];
            }
        }

        return [];
    }

    /**
     * @param \WP_Term $category
     * @param bool $withData
     * @return BuilderiusTemplateApplicant
     */
    private function processSingleCategory(\WP_Term $category, $withData)
    {
        $data = [];
        if ($withData) {
            $posts = $this->wpQuery->query([
                'lang' => '',
                'cat' => $category->term_id,
                'posts_per_page' => -1,
            ]);
            foreach ($posts as $post) {
                $data[] = PostToApplicantDataConverter::convert($post);
            }
        }

        return new BuilderiusTemplateApplicant([
            BuilderiusTemplateApplicant::LABEL_FIELD => sprintf('%s: %s', __('Category archive'), $category->name),
            BuilderiusTemplateApplicant::URL_FIELD => get_term_link($category, 'category'),
            BuilderiusTemplateApplicant::DATA_FIELD => $data,
            BuilderiusTemplateApplicant::CATEGORY_FIELD => 'category_archives'
        ]);
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if ((strpos($rule, 'archive.category_archive') !== false && $operator === '==') || strpos($rule, 'archive.all') === 0) {
            return true;
        }

        return false;
    }
}