<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicant;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Converter\PostToApplicantDataConverter;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;
use BuilderiusPro\Bundle\TemplateBundle\ApplyRule\Provider\AvailableCustomPostTypesProvider;

class BuilderiusTemplateCustomPostTypeIsNotArchiveApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @var AvailableCustomPostTypesProvider
     */
    private $customPostTypesProvider;

    /**
     * @param \WP_Query $wpQuery
     * @param AvailableCustomPostTypesProvider $customPostTypesProvider
     */
    public function __construct(
        \WP_Query $wpQuery,
        AvailableCustomPostTypesProvider $customPostTypesProvider
    ) {
        $this->wpQuery = $wpQuery;
        $this->customPostTypesProvider = $customPostTypesProvider;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $applicants = [];
        foreach ($this->customPostTypesProvider->getArguments() as $argumentData) {
            if ($argumentData['value'] !== $argument) {
                $postType = get_post_type_object($argumentData['value']);
                $applicants[sprintf('archive.post_type_archive.%s', $argument)] =
                    $this->processPostType($postType, $withData);
            }
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        $object = $changeset->getObjectAfter();
        // TODO: Implement getObjectApplicants() method.
    }

    /**
     * @param \WP_Post_Type $postType
     * @param string $postTypePermaStruct
     * @param bool $withData
     * @return BuilderiusTemplateApplicant
     */
    private function processPostType(\WP_Post_Type $postType, $withData)
    {
        $data = [];
        if ($withData) {
            $posts = $this->wpQuery->query([
                'lang' => '',
                'post_type' => $postType->name,
                'posts_per_page' => -1,
            ]);
            foreach ($posts as $post) {
                $data[] = PostToApplicantDataConverter::convert($post);
            }
        }

        return new BuilderiusTemplateApplicant([
            BuilderiusTemplateApplicant::LABEL_FIELD => sprintf('%s: %s', __('Custom post type archive'), $postType->label),
            BuilderiusTemplateApplicant::URL_FIELD => get_post_type_archive_link($postType->name),
            BuilderiusTemplateApplicant::DATA_FIELD => $data,
            BuilderiusTemplateApplicant::CATEGORY_FIELD => 'custom_post_type_archives'
        ]);
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if (strpos($rule, 'archive.post_type_archive.by_post_type') !== false && $operator === '!=') {
            return true;
        }

        return false;
    }
}