<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategoriesProvider;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategoriesProviderInterface;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategory;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;

class BuilderiusTemplateCustomPostWithNameApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @var BuilderiusTemplateApplicantCategoriesProviderInterface|BuilderiusTemplateApplicantCategoriesProvider
     */
    private $applicantCategoriesProvider;

    /**
     * @param \WP_Query $wpQuery
     * @param BuilderiusTemplateApplicantCategoriesProviderInterface $applicantCategoriesProvider
     */
    public function __construct(
        \WP_Query $wpQuery,
        BuilderiusTemplateApplicantCategoriesProviderInterface $applicantCategoriesProvider
    ) {
        $this->wpQuery = $wpQuery;
        $this->applicantCategoriesProvider = $applicantCategoriesProvider;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $postType = str_replace(
            'singular.single.custom_post.',
            '',
            str_replace('.with_name', '', $rule)
        );
        $postTypeObject = get_post_type_object($postType);
        if (!$this->applicantCategoriesProvider->hasCategory($postType . '_posts')) {
            $category = new BuilderiusTemplateApplicantCategory([
                BuilderiusTemplateApplicantCategory::NAME_FIELD => $postType . '_posts',
                BuilderiusTemplateApplicantCategory::LABEL_FIELD => $postTypeObject->labels->name,
                BuilderiusTemplateApplicantCategory::SORT_ORDER_FIELD => 40
            ]);
            $this->applicantCategoriesProvider->addCategory($category);
        }
        $statuses = get_post_stati();
        unset($statuses['auto-draft']);

        $posts = $this->wpQuery->query([
            'lang' => '',
            'posts_per_page' => -1,
            'post_type' => [$postType],
            'name' => $argument,
            'post_status' => $statuses,
        ]);
        $applicants = [];
        foreach ($posts as $post) {
            $applicants[sprintf('singular.single.%s.%s', $post->post_type, $post->ID)] =
                $this->convertPostToApplicant($post, sprintf('%s: %s', $postTypeObject->labels->singular_name, $post->post_title), $withData, $postType . '_posts');
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        $object = $changeset->getObjectAfter();
        $postType = str_replace(
            'singular.single.custom_post.',
            '',
            str_replace('.with_name', '', $rule)
        );
        $postTypeObject = get_post_type_object($postType);
        if (!$this->applicantCategoriesProvider->hasCategory($postType . '_posts')) {
            $category = new BuilderiusTemplateApplicantCategory([
                BuilderiusTemplateApplicantCategory::NAME_FIELD => $postType . '_posts',
                BuilderiusTemplateApplicantCategory::LABEL_FIELD => $postTypeObject->labels->name,
                BuilderiusTemplateApplicantCategory::SORT_ORDER_FIELD => 40
            ]);
            $this->applicantCategoriesProvider->addCategory($category);
        }
        if (!$object instanceof \WP_Post || $object->post_type !== $postType || $object->post_name !== $argument) {
            return [];
        }

        return [
            sprintf('singular.single.%s.%s', $object->post_type, $object->ID) =>
                $this->convertPostToApplicant($object, sprintf('%s: %s', $postTypeObject->labels->singular_name, $object->post_title), $withData, $postType . '_posts')
        ];
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        $postType = str_replace(
            'singular.single.custom_post.',
            '',
            str_replace('.with_name', '', $rule)
        );
        if (strpos($rule, 'singular.single.custom_post.') !== false &&
            strpos($rule, '.with_name') !== false &&
            strlen($postType) > 0 && is_string($argument) && strlen($argument) > 0 && $operator === '=='
        ) {
            return true;
        }

        return false;
    }
}