<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicant;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategoriesProvider;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategoriesProviderInterface;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategory;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Converter\PostToApplicantDataConverter;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;
use BuilderiusPro\Bundle\TemplateBundle\ApplyRule\Provider\AvailableCustomTaxonomiesProvider;

class BuilderiusTemplateCustomTaxonomyArchiveApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @var AvailableCustomTaxonomiesProvider
     */
    private $availableCustomTaxonomiesProvider;

    /**
     * @var BuilderiusTemplateApplicantCategoriesProviderInterface|BuilderiusTemplateApplicantCategoriesProvider
     */
    private $applicantCategoriesProvider;

    /**
     * @param \WP_Query $wpQuery
     * @param AvailableCustomTaxonomiesProvider $availableCustomTaxonomiesProvider
     * @param BuilderiusTemplateApplicantCategoriesProviderInterface $applicantCategoriesProvider
     */
    public function __construct(
        \WP_Query $wpQuery,
        AvailableCustomTaxonomiesProvider $availableCustomTaxonomiesProvider,
        BuilderiusTemplateApplicantCategoriesProviderInterface $applicantCategoriesProvider
    ) {
        $this->wpQuery = $wpQuery;
        $this->availableCustomTaxonomiesProvider = $availableCustomTaxonomiesProvider;
        $this->applicantCategoriesProvider = $applicantCategoriesProvider;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $applicants = [];
        $filteredRule = str_replace(
            'archive.',
            '',
            str_replace(
                'taxonomy_archive.',
                '',
                $rule
            )
        );
        if ($filteredRule === 'all') {
            foreach ($this->availableCustomTaxonomiesProvider->getArguments() as $taxData) {
                $applicants = $this->processSingleTaxonomyApplicants($applicants, $taxData['value'], $withData);
            }
        } else {
            $ruleParts = explode('.', $filteredRule);
            $taxonomyName = str_replace('is_tax_', '', $ruleParts[0]);
            if ($ruleParts[1] === 'all' && $argument === true && $operator === '==') {
                $applicants = $this->processSingleTaxonomyApplicants($applicants, $taxonomyName, $withData);
            } elseif ($ruleParts[1] === 'with_term' && is_string($argument) && $operator === '==') {
                $applicants = $this->processSingleTaxonomyApplicants($applicants, $taxonomyName, $withData, $argument);
            }
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        $object = $changeset->getObjectAfter();
        $filteredRule = str_replace(
            'archive.',
            '',
            str_replace(
                'taxonomy_archive.',
                '',
                $rule
            )
        );
        $ruleParts = explode('.', $filteredRule);
        $taxonomyName = str_replace('is_tax_', '', $ruleParts[0]);
        if (!$object instanceof \WP_Term || $object->taxonomy !== $taxonomyName) {
            return [];
        }
        $taxonomy = get_taxonomy($taxonomyName);
        if ($ruleParts[1] === 'all' && $argument === true && $operator === '==') {
            return [sprintf('archive.taxonomy_archive.%s.%s', $taxonomyName, $object->term_id) =>
                $this->processSingleTerm($taxonomy, $object, $withData)];
        } elseif ($ruleParts[1] === 'with_term' && is_string($argument) && $operator === '==') {
            if ($object->slug === $argument) {
                return [sprintf('archive.taxonomy_archive.%s.%s', $taxonomyName, $object->term_id) =>
                    $this->processSingleTerm($taxonomy, $object, $withData)];
            }
        }

        return [];
    }

    /**
     * @param array $applicants
     * @param string $taxonomyName
     * @param bool $withData
     * @param string $termSlug
     * @return array
     */
    private function processSingleTaxonomyApplicants(array $applicants, $taxonomyName, $withData, $termSlug = 'all')
    {
        $taxonomy = get_taxonomy($taxonomyName);
        if ($taxonomy) {
            if (!$this->applicantCategoriesProvider->hasCategory($taxonomyName . '_terms_archives')) {
                $category = new BuilderiusTemplateApplicantCategory([
                    BuilderiusTemplateApplicantCategory::NAME_FIELD => $taxonomyName . '_terms_archives',
                    BuilderiusTemplateApplicantCategory::LABEL_FIELD => $taxonomy->labels->name . ' archives',
                    BuilderiusTemplateApplicantCategory::SORT_ORDER_FIELD => 150
                ]);
                $this->applicantCategoriesProvider->addCategory($category);
            }
            if ($termSlug === 'all') {
                $terms = get_terms( [
                    'taxonomy' => $taxonomyName,
                    'hide_empty' => false,
                ] );
                /** @var \WP_Term $term */
                foreach ($terms as $term) {
                    $applicants[sprintf('archive.taxonomy_archive.%s.%s', $taxonomyName, $term->term_id)] =
                        $this->processSingleTerm($taxonomy, $term, $withData);
                }
            } else {
                /** @var \WP_Term $term */
                $term = get_term_by('slug', $termSlug, $taxonomyName);
                if ($term) {
                    $applicants[sprintf('archive.taxonomy_archive.%s.%s', $taxonomyName, $term->term_id)] =
                        $this->processSingleTerm($taxonomy, $term, $withData);
                }
            }
        }

        return $applicants;
    }

    /**
     * @param \WP_Taxonomy $taxonomy
     * @param \WP_Term $term
     * @param bool $withData
     * @return BuilderiusTemplateApplicant
     */
    private function processSingleTerm(\WP_Taxonomy $taxonomy, \WP_Term $term, bool $withData)
    {
        $data = [];
        if ($withData) {
            $posts = $this->wpQuery->query([
                'lang' => '',
                $taxonomy->name => $term->slug,
                'posts_per_page' => -1,
            ]);
            foreach ($posts as $post) {
                $data[] = PostToApplicantDataConverter::convert($post);
            }
        }

        return new BuilderiusTemplateApplicant([
            BuilderiusTemplateApplicant::LABEL_FIELD => sprintf('%s %s: %s', $taxonomy->labels->singular_name , __('archive'), $term->name),
            BuilderiusTemplateApplicant::URL_FIELD => get_term_link($term, $taxonomy->name),
            BuilderiusTemplateApplicant::DATA_FIELD => $data,
            BuilderiusTemplateApplicant::CATEGORY_FIELD => $taxonomy->name . '_terms_archives'
        ]);
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if ((strpos($rule, 'archive.taxonomy_archive') !== false || strpos($rule, 'archive.all') === 0) && $operator === '==') {
            return true;
        }

        return false;
    }
}