<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\BuilderiusTemplateRuleApplicantsProviderInterface;

class BuilderiusTemplateDateArchiveApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $yearApplicantsProvider;

    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $monthApplicantsProvider;

    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $dayApplicantsProvider;

    /**
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $yearApplicantsProvider
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $monthApplicantsProvider
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $dayApplicantsProvider
     */
    public function __construct(
        BuilderiusTemplateRuleApplicantsProviderInterface $yearApplicantsProvider,
        BuilderiusTemplateRuleApplicantsProviderInterface $monthApplicantsProvider,
        BuilderiusTemplateRuleApplicantsProviderInterface $dayApplicantsProvider
    ) {
        $this->yearApplicantsProvider = $yearApplicantsProvider;
        $this->monthApplicantsProvider = $monthApplicantsProvider;
        $this->dayApplicantsProvider = $dayApplicantsProvider;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $applicants = $this->yearApplicantsProvider->getApplicants($rule, $argument, $operator, $withData);
        $applicants = array_merge(
            $applicants,
            $this->monthApplicantsProvider->getApplicants($rule, $argument, $operator, $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->dayApplicantsProvider->getApplicants($rule, $argument, $operator, $withData)
        );

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        $applicants = $this->yearApplicantsProvider->getChangeSetApplicants($changeset, $rule, $argument, $operator, $withData);
        $applicants = array_merge(
            $applicants,
            $this->monthApplicantsProvider->getChangeSetApplicants($changeset, $rule, $argument, $operator, $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->dayApplicantsProvider->getChangeSetApplicants($changeset, $rule, $argument, $operator, $withData)
        );

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if (strpos($rule, 'archive.date_archive.all') !== false || strpos($rule, 'archive.all') === 0) {
            return true;
        }

        return false;
    }
}