<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\BuilderiusTemplateRuleApplicantsProviderInterface;
use BuilderiusPro\Bundle\TemplateBundle\ApplyRule\Provider\AvailableCustomPostTypesProvider;

class BuilderiusTemplateEntireSiteApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $singularApplicantsProvider;

    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $blogPostsIndexPageApplicantsProvider;

    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $authorArchiveApplicantsProvider;

    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $categoryArchiveApplicantsProvider;

    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $tagArchiveApplicantsProvider;

    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $customPostTypeArchiveApplicantsProvider;

    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $customTaxonomyArchiveApplicantsProvider;

    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $dateArchiveApplicantsProvider;

    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $searchResultsPageApplicantsProvider;

    /**
     * @var BuilderiusTemplateRuleApplicantsProviderInterface
     */
    private $page404ApplicantsProvider;

    /**
     * @var AvailableCustomPostTypesProvider
     */
    private $availableCustomPostTypesProvider;

    /**
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $singularApplicantsProvider
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $blogPostsIndexPageApplicantsProvider
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $authorArchiveApplicantsProvider
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $categoryArchiveApplicantsProvider
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $tagArchiveApplicantsProvider
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $customPostTypeArchiveApplicantsProvider
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $customTaxonomyArchiveApplicantsProvider
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $dateArchiveApplicantsProvider
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $searchResultsPageApplicantsProvider
     * @param BuilderiusTemplateRuleApplicantsProviderInterface $page404ApplicantsProvider
     * @param AvailableCustomPostTypesProvider $availableCustomPostTypesProvider
     */
    public function __construct(
        BuilderiusTemplateRuleApplicantsProviderInterface $singularApplicantsProvider,
        BuilderiusTemplateRuleApplicantsProviderInterface $blogPostsIndexPageApplicantsProvider,
        BuilderiusTemplateRuleApplicantsProviderInterface $authorArchiveApplicantsProvider,
        BuilderiusTemplateRuleApplicantsProviderInterface $categoryArchiveApplicantsProvider,
        BuilderiusTemplateRuleApplicantsProviderInterface $tagArchiveApplicantsProvider,
        BuilderiusTemplateRuleApplicantsProviderInterface $customPostTypeArchiveApplicantsProvider,
        BuilderiusTemplateRuleApplicantsProviderInterface $customTaxonomyArchiveApplicantsProvider,
        BuilderiusTemplateRuleApplicantsProviderInterface $dateArchiveApplicantsProvider,
        BuilderiusTemplateRuleApplicantsProviderInterface $searchResultsPageApplicantsProvider,
        BuilderiusTemplateRuleApplicantsProviderInterface $page404ApplicantsProvider,
        AvailableCustomPostTypesProvider $availableCustomPostTypesProvider
    ) {
        $this->singularApplicantsProvider = $singularApplicantsProvider;
        $this->blogPostsIndexPageApplicantsProvider = $blogPostsIndexPageApplicantsProvider;
        $this->authorArchiveApplicantsProvider = $authorArchiveApplicantsProvider;
        $this->categoryArchiveApplicantsProvider = $categoryArchiveApplicantsProvider;
        $this->tagArchiveApplicantsProvider = $tagArchiveApplicantsProvider;
        $this->customPostTypeArchiveApplicantsProvider = $customPostTypeArchiveApplicantsProvider;
        $this->customTaxonomyArchiveApplicantsProvider = $customTaxonomyArchiveApplicantsProvider;
        $this->dateArchiveApplicantsProvider = $dateArchiveApplicantsProvider;
        $this->searchResultsPageApplicantsProvider = $searchResultsPageApplicantsProvider;
        $this->page404ApplicantsProvider = $page404ApplicantsProvider;
        $this->availableCustomPostTypesProvider = $availableCustomPostTypesProvider;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $applicants = $this->singularApplicantsProvider->getApplicants('singular.all', true, '==', $withData);
        $applicants = array_merge(
            $applicants,
            $this->blogPostsIndexPageApplicantsProvider->getApplicants('home', true, '==', $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->authorArchiveApplicantsProvider->getApplicants('archive.author_archive.all', true, '==', $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->categoryArchiveApplicantsProvider->getApplicants('archive.category_archive.all', true, '==', $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->tagArchiveApplicantsProvider->getApplicants('archive.tag_archive.all', true, '==', $withData)
        );
        foreach ($this->availableCustomPostTypesProvider->getArguments() as $item) {
            $applicants = array_merge(
                $applicants,
                $this->customPostTypeArchiveApplicantsProvider->getApplicants('archive.post_type_archive.by_post_type', $item['value'], '==', $withData)
            );
        }
        $applicants = array_merge(
            $applicants,
            $this->customTaxonomyArchiveApplicantsProvider->getApplicants('archive.taxonomy_archive.all', true, '==', $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->dateArchiveApplicantsProvider->getApplicants('archive.date_archive.all', true, '==', $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->searchResultsPageApplicantsProvider->getApplicants('search', true, '==', $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->page404ApplicantsProvider->getApplicants('404', true, '==', $withData)
        );

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        $applicants = $this->singularApplicantsProvider->getApplicants('singular.all', true, '==', $withData);
        $applicants = array_merge(
            $applicants,
            $this->blogPostsIndexPageApplicantsProvider->getApplicants('home', true, '==', $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->authorArchiveApplicantsProvider->getApplicants('archive.author_archive.all', true, '==', $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->categoryArchiveApplicantsProvider->getApplicants('archive.category_archive.all', true, '==', $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->tagArchiveApplicantsProvider->getApplicants('archive.tag_archive.all', true, '==', $withData)
        );
        foreach ($this->availableCustomPostTypesProvider->getArguments() as $item) {
            $applicants = array_merge(
                $applicants,
                $this->customPostTypeArchiveApplicantsProvider->getApplicants('archive.post_type_archive.by_post_type', $item['value'], '==', $withData)
            );
        }
        $applicants = array_merge(
            $applicants,
            $this->customTaxonomyArchiveApplicantsProvider->getApplicants('archive.taxonomy_archive.all', true, '==', $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->dateArchiveApplicantsProvider->getApplicants('archive.date_archive.all', true, '==', $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->searchResultsPageApplicantsProvider->getApplicants('search', true, '==', $withData)
        );
        $applicants = array_merge(
            $applicants,
            $this->page404ApplicantsProvider->getApplicants('404', true, '==', $withData)
        );

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if ($rule === 'all_pages') {
            return true;
        }

        return false;
    }
}