<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;

class BuilderiusTemplateFrontPageApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @param \WP_Query $wpQuery
     */
    public function __construct(\WP_Query $wpQuery)
    {
        $this->wpQuery = $wpQuery;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $frontpageId = get_option('page_on_front');
        $showOnFront = get_option('show_on_front');
        if (!$frontpageId || $showOnFront !== 'page') {
            return [];
        }
        $statuses = get_post_stati();
        unset($statuses['auto-draft']);

        $posts = $this->wpQuery->query([
            'lang' => '',
            'posts_per_page' => -1,
            'post_type' => ['page'],
            'page_id' => $frontpageId,
            'post_status' => $statuses,
        ]);
        $applicants = [];
        foreach ($posts as $post) {
            $applicants[sprintf('singular.single.%s.%s', $post->post_type, $post->ID)] =
                $this->convertPostToApplicant($post, sprintf('%s: %s', __('Front Page'), $post->post_title), $withData, 'special_pages');
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        // TODO: Implement getChangeSetApplicants() method.
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if (strpos($rule, 'front_page') !== false && $argument === true && $operator === '==') {
            return true;
        }

        return false;
    }
}