<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategoriesProvider;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategoriesProviderInterface;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategory;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;

class BuilderiusTemplateHasNoTaxonomyApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @var BuilderiusTemplateApplicantCategoriesProviderInterface|BuilderiusTemplateApplicantCategoriesProvider
     */
    private $applicantCategoriesProvider;

    /**
     * @param \WP_Query $wpQuery
     * @param BuilderiusTemplateApplicantCategoriesProviderInterface $applicantCategoriesProvider
     */
    public function __construct(
        \WP_Query $wpQuery,
        BuilderiusTemplateApplicantCategoriesProviderInterface $applicantCategoriesProvider
    ) {
        $this->wpQuery = $wpQuery;
        $this->applicantCategoriesProvider = $applicantCategoriesProvider;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $filteredRule = str_replace(
            'singular.',
            '',
            str_replace(
                'single.',
                '',
                str_replace(
                    'custom_post.',
                    '',
                    $rule
                )
            )
        );
        $parts = explode('.', $filteredRule);
        $postType = $parts[0];
        if ($postType === 'post') {
            $labelPrefix = __('Blog post');
            $categoryName = 'blog_posts';
        } elseif ($postType === 'page') {
            $labelPrefix = __('Static page');
            $categoryName = 'static_pages';
        } else {
            /** @var \WP_Post_Type $postTypeObject */
            $postTypeObject = get_post_type_object($postType);
            $categoryName = $postType . '_posts';
            $labelPrefix = __($postTypeObject->labels->singular_name);
            if (!$this->applicantCategoriesProvider->hasCategory($categoryName)) {
                $category = new BuilderiusTemplateApplicantCategory([
                    BuilderiusTemplateApplicantCategory::NAME_FIELD => $categoryName,
                    BuilderiusTemplateApplicantCategory::LABEL_FIELD => $postTypeObject->labels->name,
                    BuilderiusTemplateApplicantCategory::SORT_ORDER_FIELD => 40
                ]);
                $this->applicantCategoriesProvider->addCategory($category);
            }
        }
        $taxonomy = str_replace(
            'has_',
            '',
            str_replace('_taxonomy.', '', sprintf('%s.',$parts[2]))
        );

        if ($postType === 'blog_post') {
            $postType = 'post';
        }
        $taxQueryArgs = [
            'relation' => 'AND',
            [
                'taxonomy' => $taxonomy,
                'field'    => 'slug',
            ]
        ];
        /** @var \WP_Term[] $terms */
        $terms = get_terms([
            'taxonomy' => $taxonomy,
            'hide_empty' => false,
        ]);
        $termsSlugs = [];
        foreach ($terms as $term) {
            if ($term->slug !== $argument) {
                $termsSlugs[] = $term->slug;
            }
        }
        $taxQueryArgs[0]['terms'] = $termsSlugs;

        $posts = $this->wpQuery->query([
            'lang' => '',
            'post_type' => $postType,
            'post_status' => $argument,
            'posts_per_page' => -1,
            'tax_query' => $taxQueryArgs,
        ]);
        $applicants = [];
        $frontpageId = get_option('page_on_front');
        $postsPageId = get_option('page_for_posts');
        $showOnFront = get_option('show_on_front');

        foreach ($posts as $post) {
            if ($showOnFront !== 'posts' && $postsPageId && $post->ID === (int)$postsPageId) {
                continue;
            } else {
                if ($frontpageId && $showOnFront === 'page' && (int)$post->ID == (int)$frontpageId) {
                    $applicants[sprintf('singular.single.%s.%s', $post->post_type, $post->ID)] =
                        $this->convertPostToApplicant($post, sprintf('%s: %s', __('Front page'), $post->post_title), $withData, 'special_pages');
                } else {
                    $applicants[sprintf('singular.single.%s.%s', $post->post_type, $post->ID)] =
                        $this->convertPostToApplicant($post, sprintf('%s: %s', $labelPrefix, $post->post_title), $withData, $categoryName);
                }
            }
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        // TODO: Implement getChangeSetApplicants() method.
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if (
            strpos($rule, 'singular') !== false &&
            strpos($rule, '_taxonomy') !== false &&
            strpos($rule, 'with_term') !== false &&
            $operator === '!='
        ) {
            return true;
        }

        return false;
    }
}