<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;

class BuilderiusTemplateMetaValueApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @param \WP_Query $wpQuery
     */
    public function __construct(\WP_Query $wpQuery)
    {
        $this->wpQuery = $wpQuery;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $filteredRule = str_replace(
            'singular.',
            '',
            str_replace(
                '.meta_value',
                '',
                str_replace(
                    'single.',
                    '',
                    str_replace(
                        'custom_post.',
                        '',
                        $rule
                    )
                )
            )
        );
        $parts = explode('.', $filteredRule);
        $postType = $parts[0];
        $metaKey = str_replace('is_meta_', '', $parts[2]);
        if ($postType === 'blog_post') {
            $postType = 'post';
        }
        $operator = strpos($operator, '!') !== false ? '!=' : '=';
        $posts = $this->wpQuery->query([
            'lang' => '',
            'post_type' => $postType,
            'post_status' => $argument,
            'posts_per_page' => -1,
            'meta_query' => [
                [
                    'key'     => $metaKey,
                    'value'   => $argument,
                    'compare' => $operator,
                ],
            ],
        ]);
        $applicants = [];
        foreach ($posts as $post) {
            $applicants[sprintf('singular.single.%s.%s', $post->post_type, $post->ID)] =
                $this->convertPostToApplicant($post, null, $withData);
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        // TODO: Implement getChangeSetApplicants() method.
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if (strpos($rule, 'meta_value') !== false && is_string($argument)) {
            return true;
        }

        return false;
    }
}