<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicant;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Converter\PostToApplicantDataConverter;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;

class BuilderiusTemplateMonthArchiveApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @param \WP_Query $wpQuery
     */
    public function __construct(\WP_Query $wpQuery)
    {
        $this->wpQuery = $wpQuery;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        global $wpdb;

        $dates = $wpdb->get_results(
            "SELECT YEAR(post_date) AS `year`, MONTH(post_date) AS `month` FROM {$wpdb->posts}  WHERE post_type = 'post' AND post_status = 'publish' GROUP BY YEAR(post_date), MONTH(post_date) ORDER BY post_date DESC"
        );

        $applicants = [];
        foreach ($dates as $date) {
            $applicants[sprintf('archive.date_archive.month_archive.%s.%s', $date->year, $date->month)] =
                $this->processSingleMonth($date->year, $date->month, $withData);
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        // TODO: Implement getChangeSetApplicants() method.
    }

    /**
     * @param int $year
     * @param int $month
     * @param bool $withData
     * @return BuilderiusTemplateApplicant
     */
    private function processSingleMonth($year, $month, $withData)
    {
        $data = [];
        if ($withData) {
            $posts = $this->wpQuery->query([
                'lang' => '',
                'post_type' => 'post',
                'posts_per_page' => -1,
                'date_query' => [
                    [
                        'year' => $year,
                        'month' => $month
                    ]
                ]
            ]);
            foreach ($posts as $post) {
                $data[] = PostToApplicantDataConverter::convert($post);
            }
        }

        return new BuilderiusTemplateApplicant([
            BuilderiusTemplateApplicant::LABEL_FIELD => sprintf('%s: %s/%s', __('Month archive'), $year, $month),
            BuilderiusTemplateApplicant::URL_FIELD => get_month_link($year, $month),
            BuilderiusTemplateApplicant::DATA_FIELD => $data,
            BuilderiusTemplateApplicant::CATEGORY_FIELD => 'month_archives'
        ]);
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if (strpos($rule, 'archive.date_archive.month_archive') !== false) {
            return true;
        }

        return false;
    }
}