<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;

class BuilderiusTemplatePageWithTitleApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @param \WP_Query $wpQuery
     */
    public function __construct(\WP_Query $wpQuery)
    {
        $this->wpQuery = $wpQuery;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $statuses = get_post_stati();
        unset($statuses['auto-draft']);

        $posts = $this->wpQuery->query([
            'lang' => '',
            'posts_per_page' => -1,
            'post_type' => ['page'],
            'title' => $argument,
            'post_status' => $statuses,
        ]);
        $applicants = [];
        $frontpageId = get_option('page_on_front');
        $postsPageId = get_option('page_for_posts');
        $showOnFront = get_option('show_on_front');

        foreach ($posts as $post) {
            if ($showOnFront !== 'posts' && $postsPageId && $post->ID === (int)$postsPageId) {
                continue;
            } else {
                if ($frontpageId && $showOnFront === 'page' && (int)$post->ID == (int)$frontpageId) {
                    $applicants[sprintf('singular.single.%s.%s', $post->post_type, $post->ID)] =
                        $this->convertPostToApplicant($post, sprintf('%s: %s', __('Front Page'), $post->post_title), $withData, 'special_pages');
                } else {
                    $applicants[sprintf('singular.single.%s.%s', $post->post_type, $post->ID)] =
                        $this->convertPostToApplicant($post, sprintf('%s: %s', __('Static page'), $post->post_title), $withData, 'static_pages');
                }
            }
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        // TODO: Implement getChangeSetApplicants() method.
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if (strpos($rule, 'singular.page.with_title') !== false && is_string($argument) && $operator === '==') {
            return true;
        }

        return false;
    }
}