<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicant;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Converter\PostToApplicantDataConverter;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;

class BuilderiusTemplateSearchResultsPageApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @param \WP_Query $wpQuery
     */
    public function __construct(\WP_Query $wpQuery)
    {
        $this->wpQuery = $wpQuery;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $applicants = [];
        $data = [];
        if ($withData) {
            $posts = $this->wpQuery->query([
                'lang' => '',
                'posts_per_page' => -1,
                's' => '',
            ]);
            foreach ($posts as $post) {
                $data[] = PostToApplicantDataConverter::convert($post);
            }
        }
        $applicants['search_results?s=\'\''] =
            new BuilderiusTemplateApplicant([
                BuilderiusTemplateApplicant::LABEL_FIELD => __('Search Results page'),
                BuilderiusTemplateApplicant::CATEGORY_FIELD => 'search_results',
                BuilderiusTemplateApplicant::URL_FIELD => get_search_link(' '),
                BuilderiusTemplateApplicant::DATA_FIELD => $data
            ]);

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        return [];
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if (strpos($rule, 'search') !== false && $argument === true && $operator === '==') {
            return true;
        }

        return false;
    }
}