<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategoriesProvider;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategoriesProviderInterface;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantCategory;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;
use BuilderiusPro\Bundle\TemplateBundle\ApplyRule\Provider\AvailableCustomPostTypesProvider;

class BuilderiusTemplateSingleApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @var AvailableCustomPostTypesProvider
     */
    private $customPostTypesProvider;

    /**
     * @var BuilderiusTemplateApplicantCategoriesProviderInterface|BuilderiusTemplateApplicantCategoriesProvider
     */
    private $applicantCategoriesProvider;

    /**
     * @param \WP_Query $wpQuery
     * @param AvailableCustomPostTypesProvider $customPostTypesProvider
     * @param BuilderiusTemplateApplicantCategoriesProviderInterface $applicantCategoriesProvider
     */
    public function __construct(
        \WP_Query $wpQuery,
        AvailableCustomPostTypesProvider $customPostTypesProvider,
        BuilderiusTemplateApplicantCategoriesProviderInterface $applicantCategoriesProvider
    ) {
        $this->wpQuery = $wpQuery;
        $this->customPostTypesProvider = $customPostTypesProvider;
        $this->applicantCategoriesProvider = $applicantCategoriesProvider;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $stat = get_post_stati();
        unset($stat['auto-draft']);

        $posts = $this->wpQuery->query([
            'lang' => '',
            'posts_per_page' => -1,
            'post_type' => ['post', 'attachment'],
            'post_status' => $stat,
        ]);
        $postTypes = [];
        $customPostLabels = [];
        foreach ($this->customPostTypesProvider->getArguments() as $argument) {
            if (!$this->applicantCategoriesProvider->hasCategory($argument['value'] . '_posts')) {
                $postTypeObject = get_post_type_object($argument['value']);
                $category = new BuilderiusTemplateApplicantCategory([
                    BuilderiusTemplateApplicantCategory::NAME_FIELD => $argument['value'] . '_posts',
                    BuilderiusTemplateApplicantCategory::LABEL_FIELD => $postTypeObject->labels->name,
                    BuilderiusTemplateApplicantCategory::SORT_ORDER_FIELD => 40
                ]);
                $this->applicantCategoriesProvider->addCategory($category);
            }
            $postTypes[] = $argument['value'];
            $customPostLabels[$argument['value']] = $argument['title'];
        }
        $applicants = [];
        $frontpageId = get_option('page_on_front');
        $postsPageId = get_option('page_for_posts');
        $showOnFront = get_option('show_on_front');

        /** @var \WP_Post[] $posts */
        $customPosts = $this->wpQuery->query([
            'posts_per_page' => -1,
            'post_type' => $postTypes,
            'post_status' => $stat,
        ]);
        $posts = array_merge($posts, $customPosts);
        foreach ($posts as $post) {
            if ($showOnFront !== 'posts' && $postsPageId && $post->ID === (int)$postsPageId) {
                continue;
            } else {
                if ($frontpageId && $showOnFront === 'page' && (int)$post->ID == (int)$frontpageId) {
                    $applicants[sprintf('singular.single.%s.%s', $post->post_type, $post->ID)] =
                        $this->convertPostToApplicant($post, sprintf('%s: %s', __('Front Page'), $post->post_title), $withData, 'special_pages');
                } else {
                    if ('page' === $post->post_type) {
                        $labelPrefix = __('Static page');
                        $category = 'static_pages';
                    } elseif ('post' === $post->post_type) {
                        $labelPrefix = __('Blog post');
                        $category = 'blog_posts';
                    } elseif ('attachment' === $post->post_type) {
                        $labelPrefix = __('Attachment post');
                        $category = 'attachment_posts';
                    } else {
                        $labelPrefix = __($customPostLabels[$post->post_type]);
                        $category = $post->post_type . '_posts';
                    }
                    $applicants[sprintf('singular.single.%s.%s', $post->post_type, $post->ID)] =
                        $this->convertPostToApplicant($post, sprintf('%s: %s', $labelPrefix, $post->post_title), $withData, $category);
                }
            }
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        // TODO: Implement getChangeSetApplicants() method.
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if ($rule === 'singular.single.all' && $argument === true && $operator === '==') {
            return true;
        }

        return false;
    }
}