<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Applicant\Provider;

use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicant;
use Builderius\Bundle\TemplateBundle\Applicant\BuilderiusTemplateApplicantChangeSetInterface;
use Builderius\Bundle\TemplateBundle\Applicant\Converter\PostToApplicantDataConverter;
use Builderius\Bundle\TemplateBundle\Applicant\Provider\AbstractBuilderiusTemplateSingularApplicantsProvider;

class BuilderiusTemplateTagArchiveApplicantsProvider extends AbstractBuilderiusTemplateSingularApplicantsProvider
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @param \WP_Query $wpQuery
     */
    public function __construct(\WP_Query $wpQuery)
    {
        $this->wpQuery = $wpQuery;
    }

    /**
     * @inheritDoc
     */
    public function getApplicants($rule, $argument, $operator, $withData = false)
    {
        $applicants = [];
        $filteredRule = str_replace(
            'archive.',
            '',
            str_replace(
                'tag_archive.',
                '',
                $rule
            )
        );
        if ($filteredRule === 'all' && $argument === true && $operator === '==') {
            /** @var \WP_Term $tag */
            foreach (get_tags() as $tag) {
                $applicants[sprintf('archive.tag_archive.%s', $tag->term_id)] =
                    $this->processSingleTag($tag, $withData);
            }
        } elseif ($filteredRule === 'by_tag' && is_int($argument) && $operator === '==') {
            /** @var \WP_Term $tag */
            $tag = get_tag($argument);
            if ($tag) {
                $applicants[sprintf('archive.tag_archive.%s', $tag->term_id)] =
                    $this->processSingleTag($tag, $withData);
            }
        }

        return $applicants;
    }

    /**
     * @inheritDoc
     */
    public function getChangeSetApplicants(
        BuilderiusTemplateApplicantChangeSetInterface $changeset,
        $rule,
        $argument,
        $operator,
        $withData = false
    ) {
        $object = $changeset->getObjectAfter();
        if (!$object instanceof \WP_Term || $object->taxonomy !== 'post_tag') {
            return [];
        }
        $filteredRule = str_replace(
            'archive.',
            '',
            str_replace(
                'tag_archive.',
                '',
                $rule
            )
        );
        if ($filteredRule === 'all' && $argument === true && $operator === '==') {
            return [sprintf('archive.tag_archive.%s', $object->term_id) =>
                $this->processSingleTag($object, $withData)];
        } elseif ($filteredRule === 'by_tag' && is_int($argument) && $operator === '==') {
            if ($object->term_id === $argument) {
                return [sprintf('archive.tag_archive.%s', $object->term_id) =>
                    $this->processSingleTag($object, $withData)];
            }
        }

        return [];
    }

    /**
     * @param \WP_Term $tag
     * @param bool $withData
     * @return BuilderiusTemplateApplicant
     */
    private function processSingleTag(\WP_Term $tag, $withData)
    {
        $data = [];
        if ($withData) {
            $posts = $this->wpQuery->query([
                'lang' => '',
                'tag_id' => $tag->term_id,
                'posts_per_page' => -1,
            ]);
            foreach ($posts as $post) {
                $data[] = PostToApplicantDataConverter::convert($post);
            }
        }

        return new BuilderiusTemplateApplicant([
            BuilderiusTemplateApplicant::LABEL_FIELD => sprintf('%s: %s', __('Tag archive'), $tag->name),
            BuilderiusTemplateApplicant::URL_FIELD => get_term_link($tag, 'post_tag'),
            BuilderiusTemplateApplicant::DATA_FIELD => $data,
            BuilderiusTemplateApplicant::CATEGORY_FIELD => 'tag_archives'
        ]);
    }

    /**
     * @inheritDoc
     */
    public function isApplicable($rule, $argument, $operator)
    {
        if ((strpos($rule, 'archive.tag_archive') !== false && $operator === '==') || strpos($rule, 'archive.all') === 0) {
            return true;
        }

        return false;
    }
}