<?php

namespace BuilderiusPro\Bundle\TemplateBundle\ApplyRule;

use Builderius\Bundle\TemplateBundle\ApplyRule\BuilderiusTemplateApplyRule;
use Builderius\Bundle\TemplateBundle\ApplyRule\BuilderiusTemplateApplyRuleInterface;
use Builderius\Bundle\TemplateBundle\ApplyRule\BuilderiusTemplateApplyRuleVariant;
use Builderius\Bundle\TemplateBundle\ApplyRule\Provider\ApplyRuleArgumentsProviderInterface;
use Builderius\Bundle\TemplateBundle\ApplyRule\Provider\AvailablePostsProvider;
use Builderius\Bundle\TemplateBundle\Cache\BuilderiusRuntimeObjectCache;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\EventDispatcher\EventDispatcher;
use BuilderiusPro\Bundle\TemplateBundle\Event\CustomPostApplyRuleEvent;

class BuilderiusTemplateCustomPostApplyRule extends BuilderiusTemplateApplyRule
    implements BuilderiusTemplateApplyRuleInterface
{
    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @var EventDispatcher
     */
    private $eventDispatcher;

    /**
     * @var AvailablePostsProvider
     */
    private $availablePostsProvider;

    /**
     * @var ApplyRuleArgumentsProviderInterface
     */
    private $customPostTypesProvider;

    /**
     * @var ApplyRuleArgumentsProviderInterface
     */
    private $postStatusesProvider;

    /**
     * @param \WP_Query $wpQuery
     * @return $this
     */
    public function setWpQuery(\WP_Query $wpQuery)
    {
        $this->wpQuery = $wpQuery;

        return $this;
    }

    /**
     * @param EventDispatcher $eventDispatcher
     * @return $this
     */
    public function setEventDispatcher(EventDispatcher $eventDispatcher)
    {
        $this->eventDispatcher = $eventDispatcher;

        return $this;
    }

    /**
     * @param BuilderiusRuntimeObjectCache $cache
     * @return $this
     */
    public function setCache(BuilderiusRuntimeObjectCache $cache)
    {
        $this->cache = $cache;
        
        return $this;
    }

    /**
     * @param ApplyRuleArgumentsProviderInterface $customPostTypesProvider
     * @return $this
     */
    public function setCustomPostTypesProvider(ApplyRuleArgumentsProviderInterface $customPostTypesProvider)
    {
        $this->customPostTypesProvider = $customPostTypesProvider;

        return $this;
    }

    /**
     * @param ApplyRuleArgumentsProviderInterface $postStatusesProvider
     * @return $this
     */
    public function setPostStatusesProvider(ApplyRuleArgumentsProviderInterface $postStatusesProvider)
    {
        $this->postStatusesProvider = $postStatusesProvider;

        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getExpression()
    {
        $customPostTypes = [];
        foreach ($this->customPostTypesProvider->getArguments() as $argument) {
            $customPostTypes[] = $argument['value'];
        }
        return sprintf('query.post.post_type in ["%s"]', implode('", "', $customPostTypes));
    }

    /**
     * @inheritDoc
     */
    public function getChildren()
    {
        $children = [];
        $this->availablePostsProvider = new AvailablePostsProvider();
        $this->availablePostsProvider->setWpQuery($this->wpQuery);
        foreach ($this->customPostTypesProvider->getArguments() as $argument) {
            $child = $this->createChild($argument['value'], $argument['title']);
            $children[$child->getName()] = $child;
        }

        return $children;
    }

    /**
     * @param string $name
     * @param string $label
     * @return BuilderiusTemplateApplyRuleInterface
     */
    private function createChild($name, $label)
    {
        $child = new BuilderiusTemplateApplyRule([
            BuilderiusTemplateApplyRule::NAME_FIELD => sprintf('%s', $name),
            BuilderiusTemplateApplyRule::LABEL_FIELD => $label,
            BuilderiusTemplateApplyRule::CATEGORY_FIELD => 'theme',
            BuilderiusTemplateApplyRule::EXPRESSION_FIELD => sprintf('query.post.post_type === "%s"', $name),
            BuilderiusTemplateApplyRule::SELECT_ALL_ALLOWED_FIELD => true
        ]);
        $child->setParent($this);
        $this->addMetaChild($child, $name);
        $this->addWithIdVariant($child, $name);
        $this->addWithNameVariant($child, $name);
        $this->addWithTitleVariant($child, $name);
        $this->addHasStatusVariant($child);
        $this->addHasTaxonomyChild($child, $name);

        $event = new CustomPostApplyRuleEvent($name, $child, $this->availablePostsProvider);
        $this->eventDispatcher->dispatch($event, 'builderius_custom_post_apply_rule_generation');

        return $event->getApplyRule();
    }

    /**
     * @param BuilderiusTemplateApplyRuleInterface $rule
     * @param string $postType
     */
    private function addMetaChild(BuilderiusTemplateApplyRuleInterface $rule, $postType)
    {
        $metaChild = new BuilderiusTemplateMetaValueApplyRule([
            BuilderiusTemplateApplyRule::NAME_FIELD => sprintf('%s_meta', $postType),
            BuilderiusTemplateApplyRule::LABEL_FIELD => 'Meta',
            BuilderiusTemplateApplyRule::CATEGORY_FIELD => 'theme',
            BuilderiusTemplateApplyRule::EXPRESSION_FIELD => '',
            BuilderiusTemplateApplyRule::SELECT_ALL_ALLOWED_FIELD => false,
            BuilderiusTemplateApplyRule::IGNORE_IF_NO_CHILDREN_FIELD => true,
            BuilderiusTemplateApplyRule::IGNORE_IF_NO_VARIANTS_FIELD => true

        ]);
        $rule->addChild($metaChild);
    }

    /**
     * @param BuilderiusTemplateApplyRuleInterface $rule
     * @param string $postType
     */
    private function addWithIdVariant(BuilderiusTemplateApplyRuleInterface $rule, $postType)
    {
        $this->availablePostsProvider->setQueryArguments(
            ['post_type' => $postType]
        );
        $this->availablePostsProvider->setField('ID');
        $listValues = $this->availablePostsProvider->getArguments();
        if (false !== $listValues && !empty($listValues)) {
            $variant = new BuilderiusTemplateApplyRuleVariant([
                BuilderiusTemplateApplyRuleVariant::NAME_FIELD => 'with_id',
                BuilderiusTemplateApplyRuleVariant::LABEL_FIELD => 'ID',
                BuilderiusTemplateApplyRuleVariant::POSSIBLE_ARGUMENTS_FIELD => $listValues,
                BuilderiusTemplateApplyRuleVariant::EXPRESSION_FIELD => 'query.post.ID operator argument',
                BuilderiusTemplateApplyRuleVariant::SELECT_ALL_ALLOWED_FIELD => false
            ]);
            $rule->addVariant($variant);
        }
    }

    /**
     * @param BuilderiusTemplateApplyRuleInterface $rule
     * @param string $postType
     */
    private function addWithNameVariant(BuilderiusTemplateApplyRuleInterface $rule, $postType)
    {
        $this->availablePostsProvider
            ->setQueryArguments(['post_type' => $postType])
            ->setField('post_name');
        $listValues = $this->availablePostsProvider->getArguments();
        if (false !== $listValues && !empty($listValues)) {
            $variant = new BuilderiusTemplateApplyRuleVariant([
                BuilderiusTemplateApplyRuleVariant::NAME_FIELD => 'with_name',
                BuilderiusTemplateApplyRuleVariant::LABEL_FIELD => 'Name',
                BuilderiusTemplateApplyRuleVariant::POSSIBLE_ARGUMENTS_FIELD => $listValues,
                BuilderiusTemplateApplyRuleVariant::EXPRESSION_FIELD => 'query.post.post_name operator argument',
                BuilderiusTemplateApplyRuleVariant::SELECT_ALL_ALLOWED_FIELD => false
            ]);
            $rule->addVariant($variant);
        }
    }

    /**
     * @param BuilderiusTemplateApplyRuleInterface $rule
     * @param string $postType
     */
    private function addWithTitleVariant(BuilderiusTemplateApplyRuleInterface $rule, $postType)
    {
        $this->availablePostsProvider
            ->setQueryArguments(['post_type' => $postType])
            ->setField('post_title');
        $listValues = $this->availablePostsProvider->getArguments();
        if (false !== $listValues && !empty($listValues)) {
            $variant = new BuilderiusTemplateApplyRuleVariant([
                BuilderiusTemplateApplyRuleVariant::NAME_FIELD => 'with_title',
                BuilderiusTemplateApplyRuleVariant::LABEL_FIELD => 'Title',
                BuilderiusTemplateApplyRuleVariant::POSSIBLE_ARGUMENTS_FIELD => $listValues,
                BuilderiusTemplateApplyRuleVariant::EXPRESSION_FIELD => 'query.post.post_title operator argument',
                BuilderiusTemplateApplyRuleVariant::SELECT_ALL_ALLOWED_FIELD => false
            ]);
            $rule->addVariant($variant);
        }
    }

    /**
     * @param BuilderiusTemplateApplyRuleInterface $rule
     */
    private function addHasStatusVariant(BuilderiusTemplateApplyRuleInterface $rule)
    {
        $variant = new BuilderiusTemplateApplyRuleVariant([
            BuilderiusTemplateApplyRuleVariant::NAME_FIELD => 'has_status',
            BuilderiusTemplateApplyRuleVariant::LABEL_FIELD => 'Status',
            BuilderiusTemplateApplyRuleVariant::POSSIBLE_ARGUMENTS_FIELD =>
                $this->postStatusesProvider->getArguments(),
            BuilderiusTemplateApplyRuleVariant::EXPRESSION_FIELD => 'query.post.post_status operator argument',
            BuilderiusTemplateApplyRuleVariant::SELECT_ALL_ALLOWED_FIELD => false
        ]);
        $rule->addVariant($variant);
    }

    /**
     * @param BuilderiusTemplateApplyRuleInterface $rule
     * @param string $postType
     */
    private function addHasTaxonomyChild(BuilderiusTemplateApplyRuleInterface $rule, $postType)
    {
        $taxonomies = get_object_taxonomies($postType, 'objects');
        if (!empty($taxonomies)) {
            $generalTaxonomyRule = new BuilderiusTemplateApplyRule([
                BuilderiusTemplateApplyRule::NAME_FIELD => 'has_taxonomy',
                BuilderiusTemplateApplyRule::LABEL_FIELD => 'Taxonomy',
                BuilderiusTemplateApplyRule::CATEGORY_FIELD => 'theme',
                BuilderiusTemplateApplyRule::EXPRESSION_FIELD => '',
                BuilderiusTemplateApplyRule::SELECT_ALL_ALLOWED_FIELD => false,
                BuilderiusTemplateApplyRule::IGNORE_IF_NO_CHILDREN_FIELD => true,
                BuilderiusTemplateApplyRule::IGNORE_IF_NO_VARIANTS_FIELD => true
            ]);
            foreach ($taxonomies as $taxonomy) {
                $specificTaxonomyRule = new BuilderiusTemplateApplyRule([
                    BuilderiusTemplateApplyRule::NAME_FIELD => sprintf('has_%s_taxonomy', $taxonomy->name),
                    BuilderiusTemplateApplyRule::LABEL_FIELD => $taxonomy->labels->singular_name,
                    BuilderiusTemplateApplyRule::CATEGORY_FIELD => 'theme',
                    BuilderiusTemplateApplyRule::EXPRESSION_FIELD => sprintf('has_term("", "%s", query.post)', $taxonomy->name),
                    BuilderiusTemplateApplyRule::SELECT_ALL_ALLOWED_FIELD => true,
                    BuilderiusTemplateApplyRule::SELECT_ALL_LABEL_FIELD => 'Any Term'

                ]);
                $terms = get_terms([
                    'taxonomy' => $taxonomy->name,
                    'hide_empty' => false,
                ]);
                if (!empty($terms)) {
                    $listValues = [];
                    foreach ($terms as $term) {
                        $listValues[] = ['value' => $term->slug, 'title' => $term->name];
                    }
                    $variant = new BuilderiusTemplateApplyRuleVariant([
                        BuilderiusTemplateApplyRuleVariant::NAME_FIELD => 'with_term',
                        BuilderiusTemplateApplyRuleVariant::LABEL_FIELD => 'Term',
                        BuilderiusTemplateApplyRuleVariant::POSSIBLE_ARGUMENTS_FIELD => $listValues,
                        BuilderiusTemplateApplyRuleVariant::EXPRESSION_FIELD => sprintf('has_term(argument, "%s", query.post) operator true', $taxonomy->name),
                        BuilderiusTemplateApplyRuleVariant::SELECT_ALL_ALLOWED_FIELD => false
                    ]);
                    $specificTaxonomyRule->addVariant($variant);
                }
                $generalTaxonomyRule->addChild($specificTaxonomyRule);
            }
            $rule->addChild($generalTaxonomyRule);
        }
    }
}