<?php

namespace BuilderiusPro\Bundle\TemplateBundle\ApplyRule;

use Builderius\Bundle\TemplateBundle\ApplyRule\BuilderiusTemplateApplyRule;
use Builderius\Bundle\TemplateBundle\ApplyRule\BuilderiusTemplateApplyRuleInterface;
use Builderius\Bundle\TemplateBundle\ApplyRule\BuilderiusTemplateApplyRuleVariant;

class BuilderiusTemplateCustomTaxonomyApplyRule extends BuilderiusTemplateApplyRule
    implements BuilderiusTemplateApplyRuleInterface
{
    /**
     * @var string
     */
    private $postType;

    /**
     * @param string $postType
     * @return $this
     */
    public function setPostType($postType)
    {
        $this->postType = $postType;

        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getChildren()
    {
        $children = [];
        $taxonomies = get_object_taxonomies($this->postType, 'objects');
        if (!empty($taxonomies)) {
            foreach ($taxonomies as $taxonomy) {
                if (!in_array($taxonomy->name, ['category', 'post_tag', 'post_format'])) {
                    $specificTaxonomyRule = new BuilderiusTemplateApplyRule([
                        BuilderiusTemplateApplyRule::NAME_FIELD => sprintf('has_%s_taxonomy', $taxonomy->name),
                        BuilderiusTemplateApplyRule::LABEL_FIELD => $taxonomy->labels->singular_name,
                        BuilderiusTemplateApplyRule::CATEGORY_FIELD => 'theme',
                        BuilderiusTemplateApplyRule::EXPRESSION_FIELD => sprintf('has_term("", "%s", query.post)', $taxonomy->name),
                        BuilderiusTemplateApplyRule::SELECT_ALL_ALLOWED_FIELD => true,
                        BuilderiusTemplateApplyRule::SELECT_ALL_LABEL_FIELD => 'Any Term'

                    ]);
                    $terms = get_terms([
                        'taxonomy' => $taxonomy->name,
                        'hide_empty' => false,
                    ]);
                    if (!empty($terms)) {
                        $listValues = [];
                        foreach ($terms as $term) {
                            $listValues[] = ['value' => $term->slug, 'title' => $term->name];
                        }
                        $variant = new BuilderiusTemplateApplyRuleVariant([
                            BuilderiusTemplateApplyRuleVariant::NAME_FIELD => 'with_term',
                            BuilderiusTemplateApplyRuleVariant::LABEL_FIELD => 'Term',
                            BuilderiusTemplateApplyRuleVariant::POSSIBLE_ARGUMENTS_FIELD => $listValues,
                            BuilderiusTemplateApplyRuleVariant::EXPRESSION_FIELD => sprintf('has_term(argument, "%s", query.post) operator true', $taxonomy->name),
                            BuilderiusTemplateApplyRuleVariant::SELECT_ALL_ALLOWED_FIELD => false
                        ]);
                        $specificTaxonomyRule->addVariant($variant);
                    }
                    $children[$specificTaxonomyRule->getName()] = $specificTaxonomyRule;
                }
            }
        }

        return $children;
    }
}