<?php

namespace BuilderiusPro\Bundle\TemplateBundle\ApplyRule;

use Builderius\Bundle\TemplateBundle\ApplyRule\BuilderiusTemplateApplyRule;
use Builderius\Bundle\TemplateBundle\ApplyRule\BuilderiusTemplateApplyRuleInterface;
use Builderius\Bundle\TemplateBundle\ApplyRule\BuilderiusTemplateApplyRuleVariant;
use Builderius\Bundle\TemplateBundle\ApplyRule\Provider\ApplyRuleArgumentsProviderInterface;

class BuilderiusTemplateCustomTaxonomyArchiveApplyRule extends BuilderiusTemplateApplyRule
    implements BuilderiusTemplateApplyRuleInterface
{
    /**
     * @var ApplyRuleArgumentsProviderInterface
     */
    private $customTaxonomiesProvider;

    /**
     * @param ApplyRuleArgumentsProviderInterface $customTaxonomiesProvider
     * @return $this
     */
    public function setCustomTaxonomiesProvider(ApplyRuleArgumentsProviderInterface $customTaxonomiesProvider)
    {
        $this->customTaxonomiesProvider = $customTaxonomiesProvider;

        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getChildren()
    {
        $children = [];
        foreach ($this->customTaxonomiesProvider->getArguments() as $argument) {
            $child = $this->createChild($argument['value'], $argument['title']);
            $children[$child->getName()] = $child;
        }

        return $children;
    }

    /**
     * @param string $name
     * @param string $label
     * @return BuilderiusTemplateApplyRuleInterface
     */
    private function createChild($name, $label)
    {
        $child = new BuilderiusTemplateApplyRule([
            BuilderiusTemplateApplyRule::NAME_FIELD => sprintf('is_tax_%s', $name),
            BuilderiusTemplateApplyRule::LABEL_FIELD => $label,
            BuilderiusTemplateApplyRule::CATEGORY_FIELD => 'theme',
            BuilderiusTemplateApplyRule::EXPRESSION_FIELD => sprintf('query.is_tax("%s")', $name),
            BuilderiusTemplateApplyRule::SELECT_ALL_ALLOWED_FIELD => true,
            BuilderiusTemplateApplyRule::SELECT_ALL_LABEL_FIELD => 'Any Term'
        ]);
        $child->setParent($this);
        $this->addWithTermVariant($child, $name);

        return $child;
    }

    /**
     * @param BuilderiusTemplateApplyRuleInterface $rule
     * @param string $taxonomyName
     */
    private function addWithTermVariant(BuilderiusTemplateApplyRuleInterface $rule, $taxonomyName)
    {
        $terms = get_terms([
            'taxonomy' => $taxonomyName,
            'hide_empty' => false,
        ]);
        if (!empty($terms)) {
            $listValues = [];
            foreach ($terms as $term) {
                $listValues[] = ['value' => $term->slug, 'title' => $term->name];
            }
            $variant = new BuilderiusTemplateApplyRuleVariant([
                BuilderiusTemplateApplyRuleVariant::NAME_FIELD => 'with_term',
                BuilderiusTemplateApplyRuleVariant::LABEL_FIELD => 'Term',
                BuilderiusTemplateApplyRuleVariant::POSSIBLE_ARGUMENTS_FIELD => $listValues,
                BuilderiusTemplateApplyRuleVariant::EXPRESSION_FIELD => sprintf('query.is_tax("%s", argument) operator true', $taxonomyName),
                BuilderiusTemplateApplyRuleVariant::SELECT_ALL_ALLOWED_FIELD => false
            ]);
            $rule->addVariant($variant);
        }
    }
}