<?php

namespace BuilderiusPro\Bundle\TemplateBundle\ApplyRule\Provider;

use Builderius\Bundle\TemplateBundle\ApplyRule\Provider\AbstractApplyRuleArgumentsProvider;

class AvailableCustomPostTypesProvider extends AbstractApplyRuleArgumentsProvider
{
    private $standardPostTypes = [
        'post',
        'page',
        'wp_block',
        'attachment',
        'revision',
        'nav_menu_item',
        'custom_css',
        'customize_changeset',
        'oembed_cache',
        'user_request',

    ];

    /**
     * @var bool
     */
    private $withArchiveOnly;

    /**
     * @var bool
     */
    private $excludeStandard;

    /**
     * @param bool $withArchiveOnly
     */
    public function __construct(bool $withArchiveOnly = false, $excludeStandard = true)
    {
        $this->withArchiveOnly = $withArchiveOnly;
        $this->excludeStandard = $excludeStandard;

        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getArguments()
    {
        $results = [];
        if (false === $this->withArchiveOnly) {
            $postTypes = get_post_types(['_builtin' => false, 'public' => true], 'objects');
        } else {
            $postTypes = get_post_types(['_builtin' => false, 'has_archive' => true], 'objects');
        }
        if ($this->excludeStandard === false) {
            $postTypes = array_merge($postTypes, get_post_types(['_builtin' => true, 'public' => true], 'objects'));
        }
        foreach ($postTypes as $post) {
            if (
                strpos($post->name, 'builderius') === false &&
                strpos($post->name, 'elementor') === false &&
                !in_array($post->name, ['ct_template', 'oxy_user_library']) &&
                !in_array($post->name, ['fl-builder-template', 'fl-theme-layout']) &&
                !in_array($post->name, ['et_pb_layout', 'bricks_template']) &&
                (($this->excludeStandard === true && !in_array($post->name, $this->standardPostTypes)) || $this->excludeStandard === false)
            ) {
                $results[] = ['value' => $post->name, 'title' => $post->labels->singular_name];
            }
        }
        $results = $this->sort($results);
        
        return $results;
    }
}
