<?php

namespace BuilderiusPro\Bundle\TemplateBundle\ApplyRule\Provider;

use Builderius\Bundle\TemplateBundle\ApplyRule\Provider\AbstractApplyRuleArgumentsProvider;

class AvailableCustomPostsProvider extends AbstractApplyRuleArgumentsProvider
{
    const CACHE_KEY = 'builderius_available_custom_posts_for_apply_rules';

    /**
     * @var AvailableCustomPostTypesProvider
     */
    private $availableCustomPostTypesProvider;

    /**
     * @var string
     */
    private $field;

    /**
     * @var array
     */
    private $queryArguments = [
        'posts_per_page' => -1
    ];

    /**
     * @var \WP_Query
     */
    private $wpQuery;

    /**
     * @param AvailableCustomPostTypesProvider $availableCustomPostTypesProvider
     * @param \WP_Query $wpQuery
     * @param string $field
     */
    public function __construct(
        AvailableCustomPostTypesProvider $availableCustomPostTypesProvider,
        \WP_Query $wpQuery,
        $field = 'ID'
    ) {
        $this->availableCustomPostTypesProvider = $availableCustomPostTypesProvider;
        $this->wpQuery = $wpQuery;
        $this->field = $field;
    }

    /**
     * @return array
     */
    private function getPostTypes()
    {
        $types = [];
        foreach($this->availableCustomPostTypesProvider->getArguments() as $item) {
            $types[] = $item['value'];
        }

        return $types;
    }

    /**
     * @inheritDoc
     */
    public function getArguments()
    {
        $postTypes = $this->getPostTypes();
        if (empty($postTypes)) {
            return [];
        }
        $results = wp_cache_get(self::CACHE_KEY);
        if (false === $results ||
            !isset($results[$this->field]) ||
            !isset($results[$this->field]['sorted']))
        {
            $stat = get_post_stati();
            unset($stat['auto-draft']);

            $this->queryArguments['post_status'] = $stat;
            if (!isset($results['objects'])) {
                $posts = $this->wpQuery->query([
                    'posts_per_page' => -1,
                    'post_type' => $postTypes
                ]);
            } else {
                $posts = $results['objects'];
            }
            $postObjects = [];
            if (!empty($posts)) {
                foreach ($posts as $post) {
                    if (!isset($results['objects'])) {
                        $postObjects[$post->ID] = $post;
                    }
                    if ($this->field === 'post_title') {
                        $title = $post->post_title;
                    } else {
                        $title = sprintf(
                            '%s%s (%s)',
                            $this->field === 'ID' ? '#' : '',
                            $post->{$this->field},
                            $post->post_title
                        );
                    }
                    $results[$this->field]['not_sorted'][$post->ID] = [
                        'value' => $post->{$this->field},
                        'title' => $title
                    ];
                    $results[$this->field]['sorted'][$post->ID] =
                        $results[$this->field]['not_sorted'][$post->ID];
                }
            } else {
                $postObjects = [];
                if(!is_array($results)) {
                    $results = [];
                }
                $results['objects'] = [];
                $results[$this->field]['not_sorted'] = [];
                $results[$this->field]['sorted'] = [];
            }
            if (!isset($results['objects']) && !empty($postObjects)) {
                $results['objects'] = $postObjects;
            }
            $results[$this->field]['sorted'] = $this->sort($results[$this->field]['sorted']);
            wp_cache_set(self::CACHE_KEY, $results);
        }
        return $results[$this->field]['sorted'];
    }
}
