<?php

namespace BuilderiusPro\Bundle\TemplateBundle\ApplyRule\Provider;

use Builderius\Bundle\TemplateBundle\ApplyRule\Provider\AbstractApplyRuleArgumentsProvider;

class AvailableUsersProvider extends AbstractApplyRuleArgumentsProvider
{
    const CACHE_KEY = 'builderius_available_users_for_apply_rules';
    /**
     * @var string
     */
    private $field;

    /**
     * @param $field
     */
    public function __construct($field = 'ID')
    {
        $this->field = $field;
    }

    /**
     * @inheritDoc
     */
    public function getArguments()
    {
        $results = wp_cache_get(self::CACHE_KEY);
        if (false === $results || !isset($results[$this->field]) || !isset($results[$this->field]['sorted'])) {
            if (!isset($results['objects'])) {
                $users = get_users();
            } else {
                $users = $results['objects'];
            }
            $userObjects = [];
            /** @var \WP_User $user */
            foreach ($users as $user) {
                if (!isset($results['objects'])) {
                    $userObjects[$user->ID] = $user;
                }
                if ($user->{$this->field} === $user->display_name) {
                    $title = $user->display_name;
                } else {
                    $title = sprintf(
                        '%s%s (%s)',
                        $this->field === 'ID' ? '#' : '',
                        $user->{$this->field},
                        $user->display_name
                    );
                }
                if(!is_array($results)) {
                    $results = [];
                }
                $results[$this->field]['not_sorted'][$user->ID] = [
                    'value' => $user->{$this->field},
                    'title' => $title
                ];
                $results[$this->field]['sorted'][$user->ID] =
                    $results[$this->field]['not_sorted'][$user->ID];
            }
            if (!isset($results['objects']) && !empty($userObjects)) {
                $results['objects'] = $userObjects;
            }
            $results[$this->field]['sorted'] = $this->sort($results[$this->field]['sorted']);
            wp_cache_set(self::CACHE_KEY, $results);
        }

        return $results[$this->field]['sorted'];
    }
}
