<?php

namespace BuilderiusPro\Bundle\TemplateBundle\ApplyRule\Starter;

use Builderius\Bundle\TemplateBundle\ApplyRule\Provider\AvailableCategoriesProvider;
use Builderius\Bundle\TemplateBundle\ApplyRule\Starter\BuilderiusTemplateApplyRuleStarterInterface;

class CategoryWithIdArchiveApplyRuleStarter implements BuilderiusTemplateApplyRuleStarterInterface
{
    /**
     * @var AvailableCategoriesProvider
     */
    private $availableCategoriesProvider;

    /**
     * @param AvailableCategoriesProvider $availableCategoriesProvider
     */
    public function __construct(AvailableCategoriesProvider $availableCategoriesProvider)
    {
        $this->availableCategoriesProvider = $availableCategoriesProvider;
    }

    /**
     * @inheritDoc
     */
    public function getName()
    {
        return 'category_with_id_archive';
    }

    /**
     * @inheritDoc
     */
    public function getTitle()
    {
        $availableCategories = $this->availableCategoriesProvider->getArguments();

        return sprintf("Archive for category with ID#%d", $availableCategories[0]['value']);
    }

    /**
     * @inheritDoc
     */
    public function getCategory()
    {
        return 'theme';
    }

    /**
     * @inheritDoc
     */
    public function getTemplateTypes()
    {
        return ['template', 'template_part'];
    }

    /**
     * @inheritDoc
     */
    public function getTechnologies()
    {
        return ['html'];
    }

    /**
     * @inheritDoc
     */
    public function getConfig()
    {
        $availableCategories = $this->availableCategoriesProvider->getArguments();

        return [
            "theme" => [
                "rules" => [
                    [
                        "value" => "archive",
                        "type" => "chain",
                        "name" => "archive",
                        "rules" => [
                            [
                                "value" => "category_archive",
                                "type" => "chain",
                                "name" => "category_archive",
                                "rules" => [
                                    [
                                        "type" => "select",
                                        "name" => "by_category",
                                        "value" => $availableCategories[0]['value'],
                                        "operator" => "==",
                                    ],
                                ],
                                "condition" => "and",
                            ],
                        ],
                        "condition" => "and",
                    ],
                ],
                "condition" => "and",
                "type" => "group",
            ],
        ];
    }

    /**
     * @inheritDoc
     */
    public function isValid()
    {
        return !empty($this->availableCategoriesProvider->getArguments());
    }
}