<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Converter\Version\v0_9_8_6;

use Builderius\Bundle\TemplateBundle\Converter\ConfigToHierarchicalConfigConverter;
use Builderius\Bundle\TemplateBundle\Converter\Version\BuilderiusTemplateConfigVersionConverterInterface;

class BuilderiusTemplateConfigModalModuleChildrenConverter implements BuilderiusTemplateConfigVersionConverterInterface
{
    /**
     * @inheritDoc
     */
    public function getVersion()
    {
        return '0.9.8.6';
    }

    /**
     * @inheritDoc
     */
    public function convert(array $config)
    {
        if (isset($config[ConfigToHierarchicalConfigConverter::CONFIG_MODULES_KEY]) && is_array($config[ConfigToHierarchicalConfigConverter::CONFIG_MODULES_KEY])) {
            $hierarchicalConfig = ConfigToHierarchicalConfigConverter::convert($config);
            $modulesToDelete = [];
            foreach ($hierarchicalConfig as $moduleConfig) {
                $modulesToDelete = $this->processModule($moduleConfig, $modulesToDelete);
            }
            foreach ($modulesToDelete as $moduleId) {
                unset($config[ConfigToHierarchicalConfigConverter::CONFIG_MODULES_KEY][$moduleId]);
                unset($config[ConfigToHierarchicalConfigConverter::CONFIG_INDEXES_KEY][$moduleId]);
            }

            foreach ($config[ConfigToHierarchicalConfigConverter::CONFIG_INDEXES_KEY] as $parent => $children) {
                foreach ($modulesToDelete as $moduleId) {
                    $key = array_search($moduleId, $children);
                    if (false !== $key) {
                        unset($children[$key]);
                    }
                }
                if (!empty($children)) {
                    $config[ConfigToHierarchicalConfigConverter::CONFIG_INDEXES_KEY][$parent] = array_values($children);
                } else {
                    unset($config[ConfigToHierarchicalConfigConverter::CONFIG_INDEXES_KEY][$parent]);
                }
            }
        }

        return $config;
    }
    /**
     * @param array $moduleConfig
     * @param array $modulesToDelete
     * @param bool $childOfModal
     * @return array
     * @throws \Exception
     */
    protected function processModule(array $moduleConfig, array $modulesToDelete, $childOfModal = false)
    {
        if ($childOfModal === true) {
            $modulesToDelete[] = $moduleConfig['id'];
        }
        if (isset($moduleConfig['children'])) {
            if($moduleConfig['name'] === 'Modal') {
                $childOfModal = true;
            }
            foreach ($moduleConfig['children'] as $childrenModuleConfig) {
                $modulesToDelete = $this->processModule($childrenModuleConfig, $modulesToDelete, $childOfModal);
            }
        }

        return $modulesToDelete;
    }
}