<?php

namespace BuilderiusPro\Bundle\TemplateBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplatesProviderInterface;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\InlineAssetsContainingEvent;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateCustomCssContentProvider;

class CustomCssRegistrationInHookTemplatesPreviewModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusTemplatesProviderInterface
     */
    private $builderiusHookTemplatesProvider;

    /**
     * @param BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider
     */
    public function __construct(
        BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider
    ) {
        $this->builderiusHookTemplatesProvider = $builderiusHookTemplatesProvider;
    }

    /**
     * @param InlineAssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(InlineAssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $templates = $this->builderiusHookTemplatesProvider->getTemplates();
        if (!empty($templates)) {
            foreach ($templates as $template) {
                if ($branch = $template->getActiveBranch()) {
                    if ($cssContent = $branch->getContent(BuilderiusTemplateCustomCssContentProvider::CONTENT_TYPE)) {
                        if (is_array($cssContent) && !empty($cssContent)) {
                            foreach ($cssContent as $cssContentItem) {
                                $assets[] = $this->createAsset((array)$cssContentItem, 'style', 'builderius-css');
                            }
                        }
                    } elseif ($commit = $branch->getActiveCommit()) {
                        if ($cssContent = $commit->getContent(BuilderiusTemplateCustomCssContentProvider::CONTENT_TYPE)) {
                            if (is_array($cssContent) && !empty($cssContent)) {
                                foreach ($cssContent as $cssContentItem) {
                                    $assets[] = $this->createAsset((array)$cssContentItem, 'style', 'builderius-css');
                                }
                            }
                        }
                    }
                }
            }
        }
        $event->setAssets($assets);
    }
}