<?php

namespace BuilderiusPro\Bundle\TemplateBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplateProviderInterface;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\InlineAssetsContainingEvent;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateCustomCssContentProvider;

class CustomCssRegistrationInPreviewModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusTemplateProviderInterface
     */
    private $builderiusTemplateProvider;

    /**
     * @param BuilderiusTemplateProviderInterface $builderiusTemplateProvider
     */
    public function __construct(
        BuilderiusTemplateProviderInterface $builderiusTemplateProvider
    ) {
        $this->builderiusTemplateProvider = $builderiusTemplateProvider;
    }

    /**
     * @param InlineAssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(InlineAssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $template = $this->builderiusTemplateProvider->getTemplate();
        if ($template) {
            if ($branch = $template->getActiveBranch()) {
                if ($cssContent = $branch->getContent(BuilderiusTemplateCustomCssContentProvider::CONTENT_TYPE)) {
                    if (is_array($cssContent) && !empty($cssContent)) {
                        foreach ($cssContent as $cssContentItem) {
                            $assets[] = $this->createAsset((array)$cssContentItem, 'style', 'builderius-css');
                        }
                    }
                } elseif ($commit = $branch->getActiveCommit()) {
                    if ($cssContent = $commit->getContent(BuilderiusTemplateCustomCssContentProvider::CONTENT_TYPE)) {
                        if (is_array($cssContent) && !empty($cssContent)) {
                            foreach ($cssContent as $cssContentItem) {
                                $assets[] = $this->createAsset((array)$cssContentItem, 'style', 'builderius-css');
                            }
                        }
                    }
                }
            }
        }
        $event->setAssets($assets);
    }
}