<?php

namespace BuilderiusPro\Bundle\TemplateBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Factory\BuilderiusTemplateFromPostFactory;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplateProviderInterface;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\InlineAssetsContainingEvent;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateJsContentProvider;

class JsRegistrationInPreviewModeEventListener extends AbstractAssetsRegistrationEventListener
{
    /**
     * @var BuilderiusTemplateProviderInterface
     */
    private $builderiusTemplateProvider;

    /**
     * @var BuilderiusTemplateFromPostFactory
     */
    protected $builderiusTemplateFromPostFactory;

    /**
     * @param BuilderiusTemplateProviderInterface $builderiusTemplateProvider
     * @param BuilderiusTemplateFromPostFactory $builderiusTemplateFromPostFactory
     */
    public function __construct(
        BuilderiusTemplateProviderInterface $builderiusTemplateProvider,
        BuilderiusTemplateFromPostFactory $builderiusTemplateFromPostFactory
    ) {
        $this->builderiusTemplateProvider = $builderiusTemplateProvider;
        $this->builderiusTemplateFromPostFactory = $builderiusTemplateFromPostFactory;
    }

    /**
     * @param InlineAssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(InlineAssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $template = $this->builderiusTemplateProvider->getTemplate();
        if ($template) {
            if ($branch = $template->getActiveBranch()) {
                if ($jsContent = $branch->getContent(BuilderiusTemplateJsContentProvider::CONTENT_TYPE)) {
                    if (is_array($jsContent) && !empty($jsContent)) {
                        foreach ($jsContent as $jsContentItem) {
                            $assets[] = $this->createAsset((array)$jsContentItem, 'script', 'builderius-js');
                        }
                    }
                } elseif ($commit = $branch->getActiveCommit()) {
                    if ($jsContent = $commit->getContent(BuilderiusTemplateJsContentProvider::CONTENT_TYPE)) {
                        if (is_array($jsContent) && !empty($jsContent)) {
                            foreach ($jsContent as $jsContentItem) {
                                $assets[] = $this->createAsset((array)$jsContentItem, 'script', 'builderius-js');
                            }
                        }
                    }
                }
            }
        }
        $event->setAssets($assets);
    }
}