<?php

namespace BuilderiusPro\Bundle\TemplateBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModuleProviderInterface;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\AssetsContainingEvent;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\AssetInterface;
use BuilderiusPro\Bundle\SettingBundle\EventListener\AbstractLibrariesRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateLibrariesContentProvider;

class LibrariesRegistrationInFrontendModeEventListener extends AbstractLibrariesRegistrationEventListener
{
    /**
     * @var DeliverableTemplateSubModuleProviderInterface
     */
    private $dtsmProvider;

    /**
     * @param DeliverableTemplateSubModuleProviderInterface $dtsmProvider
     */
    public function __construct(
        DeliverableTemplateSubModuleProviderInterface $dtsmProvider
    ) {
        $this->dtsmProvider = $dtsmProvider;
    }

    /**
     * @param AssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(AssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $templateSubModule = $this->dtsmProvider->getTemplateSubModule();
        if ($templateSubModule) {
            if ($libsContent = $templateSubModule->getContent(BuilderiusTemplateLibrariesContentProvider::CONTENT_TYPE)) {
                if (is_array($libsContent) && !empty($libsContent)) {
                    foreach ($libsContent as $libsContentItem) {
                        if (is_array($libsContentItem['registeredLibs'])) {
                            foreach ($libsContentItem['registeredLibs'] as $libConfig) {
                                $asset = $this->createAssetForRegisteredLibrary((array)$libConfig);
                                if ($asset instanceof AssetInterface) {
                                    $assets[] = $asset;
                                }
                            }
                        }
                        if (is_array($libsContentItem['notRegisteredLibs'])) {
                            foreach ($libsContentItem['notRegisteredLibs'] as $libConfig) {
                                $asset = $this->createAssetForNotRegisteredLibrary((array)$libConfig);
                                if ($asset instanceof AssetInterface) {
                                    $assets[] = $asset;
                                }
                            }
                        }
                    }
                }
            }
        }
        $event->setAssets($assets);
    }
}