<?php

namespace BuilderiusPro\Bundle\TemplateBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModulesProviderInterface;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\AssetsContainingEvent;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\AssetInterface;
use BuilderiusPro\Bundle\SettingBundle\EventListener\AbstractLibrariesRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateLibrariesContentProvider;

class LibrariesRegistrationInHookTemplatesFrontendModeEventListener extends AbstractLibrariesRegistrationEventListener
{
    /**
     * @var DeliverableTemplateSubModulesProviderInterface
     */
    private $dhtsmsProvider;

    /**
     * @param DeliverableTemplateSubModulesProviderInterface $dhtsmsProvider
     */
    public function __construct(
        DeliverableTemplateSubModulesProviderInterface $dhtsmsProvider
    ) {
        $this->dhtsmsProvider = $dhtsmsProvider;
    }

    /**
     * @param AssetsContainingEvent $event
     * @throws \Exception
     */
    public function beforeAssetsRegistration(AssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $templateSubModules = $this->dhtsmsProvider->getTemplateSubModules();
        if (!empty($templateSubModules)) {
            foreach ($templateSubModules as $templateSubModule) {
                if ($libsContent = $templateSubModule->getContent(BuilderiusTemplateLibrariesContentProvider::CONTENT_TYPE)) {
                    if (is_array($libsContent) && !empty($libsContent)) {
                        foreach ($libsContent as $libsContentItem) {
                            if (is_array($libsContentItem['registeredLibs'])) {
                                foreach ($libsContentItem['registeredLibs'] as $libConfig) {
                                    $asset = $this->createAssetForRegisteredLibrary((array)$libConfig);
                                    if ($asset instanceof AssetInterface && !in_array($asset, $assets)) {
                                        $assets[] = $asset;
                                    }
                                }
                            }
                            if (is_array($libsContentItem['notRegisteredLibs'])) {
                                foreach ($libsContentItem['notRegisteredLibs'] as $libConfig) {
                                    $asset = $this->createAssetForNotRegisteredLibrary((array)$libConfig);
                                    if ($asset instanceof AssetInterface && !in_array($asset, $assets)) {
                                        $assets[] = $asset;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        $event->setAssets($assets);
    }
}