<?php

namespace BuilderiusPro\Bundle\TemplateBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplatesProviderInterface;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\AssetsContainingEvent;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\AssetInterface;
use BuilderiusPro\Bundle\SettingBundle\EventListener\AbstractLibrariesRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateLibrariesContentProvider;

class LibrariesRegistrationInHookTemplatesPreviewModeEventListener extends AbstractLibrariesRegistrationEventListener
{
    /**
     * @var BuilderiusTemplatesProviderInterface
     */
    private $builderiusHookTemplatesProvider;

    /**
     * @param BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider
     */
    public function __construct(
        BuilderiusTemplatesProviderInterface $builderiusHookTemplatesProvider
    ) {
        $this->builderiusHookTemplatesProvider = $builderiusHookTemplatesProvider;
    }

    public function beforeAssetsRegistration(AssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $templates = $this->builderiusHookTemplatesProvider->getTemplates();
        if (!empty($templates)) {
            foreach ($templates as $template) {
                if ($branch = $template->getActiveBranch()) {
                    if ($libsContent = $branch->getContent(BuilderiusTemplateLibrariesContentProvider::CONTENT_TYPE)) {
                        if (is_array($libsContent) && !empty($libsContent)) {
                            foreach ($libsContent as $libsContentItem) {
                                if (is_array($libsContentItem['registeredLibs'])) {
                                    foreach ($libsContentItem['registeredLibs'] as $libConfig) {
                                        $asset = $this->createAssetForRegisteredLibrary((array)$libConfig);
                                        if ($asset instanceof AssetInterface && !in_array($asset, $assets)) {
                                            $assets[] = $asset;
                                        }
                                    }
                                }
                                if (is_array($libsContentItem['notRegisteredLibs'])) {
                                    foreach ($libsContentItem['notRegisteredLibs'] as $libConfig) {
                                        $asset = $this->createAssetForNotRegisteredLibrary((array)$libConfig);
                                        if ($asset instanceof AssetInterface && !in_array($asset, $assets)) {
                                            $assets[] = $asset;
                                        }
                                    }
                                }
                            }
                        }
                    } elseif ($commit = $branch->getActiveCommit()) {
                        if ($libsContent = $commit->getContent(BuilderiusTemplateLibrariesContentProvider::CONTENT_TYPE)) {
                            if (is_array($libsContent) && !empty($libsContent)) {
                                foreach ($libsContent as $libsContentItem) {
                                    if (is_array($libsContentItem['registeredLibs'])) {
                                        foreach ($libsContentItem['registeredLibs'] as $libConfig) {
                                            $asset = $this->createAssetForRegisteredLibrary((array)$libConfig);
                                            if ($asset instanceof AssetInterface && !in_array($asset, $assets)) {
                                                $assets[] = $asset;
                                            }
                                        }
                                    }
                                    if (is_array($libsContentItem['notRegisteredLibs'])) {
                                        foreach ($libsContentItem['notRegisteredLibs'] as $libConfig) {
                                            $asset = $this->createAssetForNotRegisteredLibrary((array)$libConfig);
                                            if ($asset instanceof AssetInterface && !in_array($asset, $assets)) {
                                                $assets[] = $asset;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        $event->setAssets($assets);
    }
}