<?php

namespace BuilderiusPro\Bundle\TemplateBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Factory\BuilderiusTemplateFromPostFactory;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplateProviderInterface;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Event\AssetsContainingEvent;
use Builderius\MooMoo\Platform\Bundle\AssetBundle\Model\AssetInterface;
use BuilderiusPro\Bundle\SettingBundle\EventListener\AbstractLibrariesRegistrationEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateLibrariesContentProvider;

class LibrariesRegistrationInPreviewModeEventListener extends AbstractLibrariesRegistrationEventListener
{
    /**
     * @var BuilderiusTemplateProviderInterface
     */
    private $builderiusTemplateProvider;

    /**
     * @var BuilderiusTemplateFromPostFactory
     */
    protected $builderiusTemplateFromPostFactory;

    /**
     * @param BuilderiusTemplateProviderInterface $builderiusTemplateProvider
     * @param BuilderiusTemplateFromPostFactory $builderiusTemplateFromPostFactory
     */
    public function __construct(
        BuilderiusTemplateProviderInterface $builderiusTemplateProvider,
        BuilderiusTemplateFromPostFactory $builderiusTemplateFromPostFactory
    ) {
        $this->builderiusTemplateProvider = $builderiusTemplateProvider;
        $this->builderiusTemplateFromPostFactory = $builderiusTemplateFromPostFactory;
    }

    public function beforeAssetsRegistration(AssetsContainingEvent $event)
    {
        $assets = $event->getAssets();
        $template = $this->builderiusTemplateProvider->getTemplate();
        if ($template) {
            if ($branch = $template->getActiveBranch()) {
                if ($libsContent = $branch->getContent(BuilderiusTemplateLibrariesContentProvider::CONTENT_TYPE)) {
                    if (is_array($libsContent) && !empty($libsContent)) {
                        foreach ($libsContent as $libsContentItem) {
                            if (is_array($libsContentItem['registeredLibs'])) {
                                foreach ($libsContentItem['registeredLibs'] as $libConfig) {
                                    $asset = $this->createAssetForRegisteredLibrary((array)$libConfig);
                                    if ($asset instanceof AssetInterface) {
                                        $assets[] = $asset;
                                    }
                                }
                            }
                            if (is_array($libsContentItem['notRegisteredLibs'])) {
                                foreach ($libsContentItem['notRegisteredLibs'] as $libConfig) {
                                    $asset = $this->createAssetForNotRegisteredLibrary((array)$libConfig);
                                    if ($asset instanceof AssetInterface) {
                                        $assets[] = $asset;
                                    }
                                }
                            }
                        }
                    }
                } elseif ($commit = $branch->getActiveCommit()) {
                    if ($libsContent = $commit->getContent(BuilderiusTemplateLibrariesContentProvider::CONTENT_TYPE)) {
                        if (is_array($libsContent) && !empty($libsContent)) {
                            foreach ($libsContent as $libsContentItem) {
                                if (is_array($libsContentItem['registeredLibs'])) {
                                    foreach ($libsContentItem['registeredLibs'] as $libConfig) {
                                        $asset = $this->createAssetForRegisteredLibrary((array)$libConfig);
                                        if ($asset instanceof AssetInterface) {
                                            $assets[] = $asset;
                                        }
                                    }
                                }
                                if (is_array($libsContentItem['notRegisteredLibs'])) {
                                    foreach ($libsContentItem['notRegisteredLibs'] as $libConfig) {
                                        $asset = $this->createAssetForNotRegisteredLibrary((array)$libConfig);
                                        if ($asset instanceof AssetInterface) {
                                            $assets[] = $asset;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        $event->setAssets($assets);
    }
}