<?php

namespace BuilderiusPro\Bundle\TemplateBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModuleProviderInterface;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\EventDispatcher\ConditionAwareEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateStringTranslationsContentProvider;

class StringTranslationsRegistrationInFrontendModeEventListener extends ConditionAwareEventListener
{
    /**
     * @var DeliverableTemplateSubModuleProviderInterface
     */
    private $dtsmProvider;

    /**
     * @param DeliverableTemplateSubModuleProviderInterface $dtsmProvider
     */
    public function __construct(
        DeliverableTemplateSubModuleProviderInterface $dtsmProvider
    ) {
        $this->dtsmProvider = $dtsmProvider;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeStringTranslation(ConfigContainingEvent $event)
    {
        global $l10n;
        if (!isset($l10n['builderius_strings'])) {
            $templateSubModule = $this->dtsmProvider->getTemplateSubModule();
            if ($templateSubModule) {
                $globalStrings = $event->getConfig();
                $locale = determine_locale();
                $mo = new \MO;
                if ($stContent = $templateSubModule->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                    if (is_array($stContent) && isset($stContent[$locale])) {
                        $templateStrings = $stContent[$locale];
                        foreach ($stContent[$locale] as $origString => $translatedString) {
                            $mo->add_entry($mo->make_entry($origString, $translatedString));
                        }
                    }
                }
                foreach ($globalStrings as $origString => $translatedString) {
                    if (!isset($templateStrings[$origString])) {
                        $mo->add_entry($mo->make_entry($origString, $translatedString));
                    }
                }
                $l10n['builderius_strings'] = $mo;
            }
        }
    }
}