<?php

namespace BuilderiusPro\Bundle\TemplateBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use Builderius\Bundle\TemplateBundle\Provider\DeliverableTemplateSubModule\DeliverableTemplateSubModulesProviderInterface;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\EventDispatcher\ConditionAwareEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateStringTranslationsContentProvider;

class StringTranslationsRegistrationInHookTemplatesFrontendModeEventListener extends ConditionAwareEventListener
{
    /**
     * @var DeliverableTemplateSubModulesProviderInterface
     */
    private $dhtsmsProvider;

    /**
     * @param DeliverableTemplateSubModulesProviderInterface $dhtsmsProvider
     */
    public function __construct(
        DeliverableTemplateSubModulesProviderInterface $dhtsmsProvider
    ) {
        $this->dhtsmsProvider = $dhtsmsProvider;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeStringTranslation(ConfigContainingEvent $event)
    {
        global $l10n;
        if (!isset($l10n['builderius_strings'])) {
            $templateSubModules = $this->dhtsmsProvider->getTemplateSubModules();
            if (!empty($templateSubModules)) {
                $globalStrings = $event->getConfig();
                $locale = determine_locale();
                $mo = new \MO;
                foreach ($templateSubModules as $templateSubModule) {
                    if ($stContent = $templateSubModule->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                        if (is_array($stContent) && isset($stContent[$locale])) {
                            $templateStrings = $stContent[$locale];
                            foreach ($stContent[$locale] as $origString => $translatedString) {
                                $mo->add_entry($mo->make_entry($origString, $translatedString));
                            }
                        }
                    }
                    foreach ($globalStrings as $origString => $translatedString) {
                        if (!isset($templateStrings[$origString])) {
                            $mo->add_entry($mo->make_entry($origString, $translatedString));
                        }
                    }
                }
                $l10n['builderius_strings'] = $mo;
            }
        }
    }
}