<?php

namespace BuilderiusPro\Bundle\TemplateBundle\EventListener;

use Builderius\Bundle\TemplateBundle\Event\ConfigContainingEvent;
use Builderius\Bundle\TemplateBundle\Factory\BuilderiusTemplateFromPostFactory;
use Builderius\Bundle\TemplateBundle\Provider\Template\BuilderiusTemplateProviderInterface;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\EventDispatcher\ConditionAwareEventListener;
use BuilderiusPro\Bundle\TemplateBundle\Provider\Content\Template\BuilderiusTemplateStringTranslationsContentProvider;

class StringTranslationsRegistrationInPreviewModeEventListener extends ConditionAwareEventListener
{
    /**
     * @var BuilderiusTemplateProviderInterface
     */
    private $builderiusTemplateProvider;

    /**
     * @var BuilderiusTemplateFromPostFactory
     */
    protected $builderiusTemplateFromPostFactory;

    /**
     * @param BuilderiusTemplateProviderInterface $builderiusTemplateProvider
     * @param BuilderiusTemplateFromPostFactory $builderiusTemplateFromPostFactory
     */
    public function __construct(
        BuilderiusTemplateProviderInterface $builderiusTemplateProvider,
        BuilderiusTemplateFromPostFactory   $builderiusTemplateFromPostFactory
    )
    {
        $this->builderiusTemplateProvider = $builderiusTemplateProvider;
        $this->builderiusTemplateFromPostFactory = $builderiusTemplateFromPostFactory;
    }

    /**
     * @param ConfigContainingEvent $event
     * @throws \Exception
     */
    public function beforeStringTranslation(ConfigContainingEvent $event)
    {
        global $l10n;
        if (!isset($l10n['builderius_strings'])) {
            $template = $this->builderiusTemplateProvider->getTemplate();
            if ($template) {
                $globalStrings = $event->getConfig();
                $locale = determine_locale();
                $mo = new \MO;
                if ($branch = $template->getActiveBranch()) {
                    if ($stContent = $branch->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                        if (is_array($stContent) && isset($stContent[$locale])) {
                            $templateStrings = $stContent[$locale];
                            foreach ($templateStrings as $origString => $translatedString) {
                                $mo->add_entry($mo->make_entry($origString, $translatedString));
                            }
                        }
                    } elseif ($commit = $branch->getActiveCommit()) {
                        if ($stContent = $commit->getContent(BuilderiusTemplateStringTranslationsContentProvider::CONTENT_TYPE)) {
                            if (is_array($stContent) && isset($stContent[$locale])) {
                                $templateStrings = $stContent[$locale];
                                foreach ($templateStrings as $origString => $translatedString) {
                                    $mo->add_entry($mo->make_entry($origString, $translatedString));
                                }
                            }
                        }
                    }
                }
                foreach ($globalStrings as $origString => $translatedString) {
                    if (!isset($templateStrings[$origString])) {
                        $mo->add_entry($mo->make_entry($origString, $translatedString));
                    }
                }
                $l10n['builderius_strings'] = $mo;
            }
        }
    }
}