<?php

namespace BuilderiusPro\Bundle\TemplateBundle\ExpressionLanguage;

use Builderius\Symfony\Component\ExpressionLanguage\ExpressionFunction;
use Builderius\Symfony\Component\ExpressionLanguage\ExpressionFunctionProviderInterface;

class SuperglobalsFunctionsProvider implements ExpressionFunctionProviderInterface
{
    /**
     * @inheritDoc
     */
    public function getFunctions()
    {
        return [
            new ExpressionFunction(
                'superglobal_has_key',
                function ($context, $alias, $argument) {
                    return sprintf('superglobal_has_key(%s, %s)', $alias, $argument);
                },
                function ($context, $alias, $argument) {
                    return array_key_exists($argument, $this->getSuperglobalByAlias($alias));
                }
            ),
            new ExpressionFunction(
                'superglobal_has_key_with_value',
                function ($context, $alias, $argument) {
                    return sprintf('superglobal_has_key_with_value(%s, %s)', $alias, $argument);
                },
                function ($context, $alias, $argument) {
                    if (!is_array($argument) ||
                        !array_key_exists('key', $argument) ||
                        !array_key_exists('value', $argument)
                    ) {
                        return false;
                    }
                    $superglobal = $this->getSuperglobalByAlias($alias);

                    return array_key_exists($argument['key'], $superglobal) && $superglobal[$argument['key']] == $argument['value'];
                }
            ),
        ];
    }

    /**
     * @param $alias
     * @return array
     */
    private function getSuperglobalByAlias($alias)
    {
        switch ($alias) {
            case '$_GET':
                return $_GET;
            case '$_POST':
                return $_POST;
            case '$_FILES':
                return $_FILES;
            case '$_COOKIE':
                return $_COOKIE;
            case '$_SESSION':
                return $_SESSION;
            case '$_REQUEST':
                return $_REQUEST;
            case '$_ENV':
                return $_ENV;
            default:
                return [];
        }
    }
}