<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Hook;

use Builderius\Bundle\TemplateBundle\Cache\BuilderiusPersistentObjectCache;
use Builderius\Bundle\TemplateBundle\Hook\AbstractApplicantDataAction;
use Builderius\Symfony\Component\Cache\CacheItem;

class ApplicantPHPDataVarProvidingHook extends AbstractApplicantDataAction
{
    /**
     * @var BuilderiusPersistentObjectCache
     */
    private $persistentCache;

    /**
     * @param BuilderiusPersistentObjectCache $persistentCache
     * @return $this
     */
    public function setPersistentCache(BuilderiusPersistentObjectCache $persistentCache)
    {
        $this->persistentCache = $persistentCache;

        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getFunction()
    {
        $user = apply_filters('builderius_get_current_user', $this->getUser());
        if (isset( $_POST['builderius-applicant-php-datavar']) && user_can($user, 'builderius-development')) {

            /** @var CacheItem $contextCacheItem */
            $contextCacheItem = $this->persistentCache->getItem('applicant_php_datavar_context');
            $builderius_vars = $contextCacheItem->get();
            $this->persistentCache->delete('applicant_php_datavar_context');

            /** @var CacheItem $codeCacheItem */
            $codeCacheItem = $this->persistentCache->getItem('applicant_php_datavar_code');
            $code = $codeCacheItem->get();
            $this->persistentCache->delete('applicant_php_datavar_code');
            try {
                add_filter('query', [$this, 'builderiusForbidDbChange']);
                ob_start();
                function php_datavar_output($builderius_vars, $body) {
                    return eval($body);
                }
                $output = php_datavar_output($builderius_vars, $code);
                ob_get_clean();
                remove_filter('query', [$this, 'builderiusForbidDbChange']);
            } catch ( \Exception $error ) {
                $output = 'Exception: ' . $error->getMessage();
            }

            /** @var CacheItem $dataCacheItem */
            $dataCacheItem = $this->persistentCache->getItem('applicant_php_datavar_data');
            $dataCacheItem->set($output);
            $this->persistentCache->save($dataCacheItem);
        }
    }

    /**
     * @param string $query
     * @throws \Exception
     */
    public function builderiusForbidDbChange($query)
    {
        if (
            strpos($query, 'INSERT INTO') !== false ||
            (
                strpos($query, 'UPDATE ') !== false &&
                strpos($query, 'SET ') !== false
            ) ||
            strpos($query, 'DELETE FROM') !== false ||
            strpos($query, 'CREATE') !== false ||
            strpos($query, 'ALTER') !== false ||
            strpos($query, 'TRUNCATE') !== false ||
            strpos($query, 'DROP') !== false
        ) {
            throw new \Exception('It is forbidden to modify the database');
        }

        return $query;
    }
}