<?php

namespace BuilderiusPro\Bundle\TemplateBundle\Migrations\v0_9_8_6;

use Builderius\Bundle\TemplateBundle\Converter\Version\BuilderiusTemplateConfigVersionConverterInterface;
use Builderius\Bundle\TemplateBundle\Converter\Version\CompositeBuilderiusTemplateConfigVersionConverter;
use Builderius\Bundle\TemplateBundle\Event\PostContainingEvent;
use Builderius\Bundle\TemplateBundle\Helper\ContentConfigHelper;
use Builderius\Bundle\VCSBundle\Model\BuilderiusBranch;
use Builderius\Bundle\VCSBundle\Model\BuilderiusCommit;
use Builderius\Bundle\VCSBundle\Registration\BuilderiusBranchPostType;
use Builderius\MooMoo\Platform\Bundle\MigrationBundle\Migration\Migration;
use Builderius\Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Builderius\Symfony\Component\DependencyInjection\ContainerAwareTrait;
use Builderius\Symfony\Component\EventDispatcher\EventDispatcherInterface;

class BuilderiusTemplateBundleMigration implements Migration, ContainerAwareInterface
{
    use ContainerAwareTrait;

    /**
     * @var EventDispatcherInterface
     */
    private $eventDispatcher;

    /**
     * @return EventDispatcherInterface
     */
    private function getEventDispatcher()
    {
        if ($this->eventDispatcher === null) {
            $this->eventDispatcher = $this->container->get('event_dispatcher');
        }

        return $this->eventDispatcher;
    }

    /**
     * @return BuilderiusTemplateConfigVersionConverterInterface[]
     */
    private function getBuilderiusTemplateConfigVersionConverters()
    {
        /** @var CompositeBuilderiusTemplateConfigVersionConverter $compositeConverter */
        $compositeConverter = $this->container->get('builderius_template.version_converter.composite');

        return $compositeConverter->getConverters('builderius-pro', '0.9.8.6');
    }

    /**
     * @inheritDoc
     */
    public function up(\wpdb $wpdb)
    {
        $posts = $wpdb->get_results(
            "SELECT ID, post_name, post_type FROM {$wpdb->posts} WHERE post_type IN ('builderius_commit') AND post_content != 'null' AND post_content IS NOT NULL");

        foreach ($posts as $post) {
            $config = ContentConfigHelper::formatConfig(
                json_decode(
                    get_post_meta( $post->ID, BuilderiusCommit::CONTENT_CONFIG_FIELD, true ),
                    true
                )
            );
            if (is_array($config) && isset($config['version']) && isset($config['version']['builderius-pro']) && version_compare($config['version']['builderius-pro'], '0.9.8.6') === -1) {
                foreach ($this->getBuilderiusTemplateConfigVersionConverters() as $versionConverter) {
                    $config = $versionConverter->convert($config);
                }
                if (!isset($config['version'])) {
                    $config['version'] = [];
                }
                $config['version']['builderius'] = '0.9.8.6';
                update_post_meta(
                    $post->ID,
                    BuilderiusCommit::CONTENT_CONFIG_FIELD,
                    wp_slash(json_encode($config))
                );
                $post = get_post($post->ID);
                $this->getEventDispatcher()->dispatch(new PostContainingEvent($post), 'builderius_commit_created');
            }
        }
    }
}