<?php

namespace BuilderiusPro\Bundle\TemplateBundle\RestApi\Endpoint;

use Builderius\Bundle\TemplateBundle\Cache\BuilderiusPersistentObjectCache;
use Builderius\MooMoo\Platform\Bundle\RestApiBundle\Endpoint\RestApiEndpointInterface;
use Builderius\Symfony\Component\Cache\CacheItem;

class BuilderiusPHPDataVarEndpoint implements RestApiEndpointInterface
{
    const NAMESPACE = 'wp/v2';
    const REST_BASE = 'builderius-php-data-var';

    /**
     * @var BuilderiusPersistentObjectCache
     */
    private $persistentCache;

    /**
     * @param BuilderiusPersistentObjectCache $persistentCache
     */
    public function __construct(
        BuilderiusPersistentObjectCache $persistentCache
    )
    {
        $this->persistentCache = $persistentCache;
    }

    /**
     * @inheritDoc
     */
    public function registerRoutes()
    {
        register_rest_route(self::NAMESPACE, '/' . self::REST_BASE, [
            [
                'methods' => \WP_REST_Server::CREATABLE,
                'callback' => [$this, 'getPHPDataVarsValues'],
                'permission_callback' => [$this, 'permissionCheck'],
                'args' => [],
            ],
            'schema' => [],
        ]);
    }

    /**
     * @return bool
     */
    public function permissionCheck() {
        $user = apply_filters('builderius_get_current_user', wp_get_current_user());

        return user_can($user, 'builderius-development');
    }

    /**
     * @param \WP_REST_Request $request
     * @return \WP_REST_Response|\WP_Error
     */
    public function getPHPDataVarsValues(\WP_REST_Request $request)
    {
        $data = $request->get_json_params();
        if (!isset($data['context']) || !isset($data['code']) || !isset($data['url'])) {
            return rest_ensure_response([]);
        }
        $url = $data['url'];
        $params = [];
        if (isset($data['url_params']) && isset($data['url_params']['GET']) && is_array($data['url_params']['GET'])) {
            $params = [];
            if (isset($data['url_params']['GET'][0])) {
                foreach ($data['url_params']['GET'] as $value) {
                    $params[$value['key']] = isset($value['value']) ? $value['value'] : 0;
                }
            } else {
                $params = $data['url_params']['GET'];
            }
        }
        $queryString = http_build_query($params);
        if (!empty($queryString)) {
            if (strpos($url, '?') === false) {
                $url = sprintf('%s?%s', $url, $queryString);
            } else {
                $url = sprintf('%s&%s', $url, $queryString);
            }
        }
        /** @var CacheItem $contextCacheItem */
        $contextCacheItem = $this->persistentCache->getItem('applicant_php_datavar_context');
        $contextCacheItem->set($data['context']);
        $this->persistentCache->save($contextCacheItem);

        /** @var CacheItem $codeCacheItem */
        $codeCacheItem = $this->persistentCache->getItem('applicant_php_datavar_code');
        $codeCacheItem->set($data['code']);
        $this->persistentCache->save($codeCacheItem);

        $wpCurl = new \WP_Http_Curl();

        $args = [
            'method' => 'POST',
            'body' => [
                'builderius-applicant-php-datavar' => true
            ],
            'timeout' => -1,
            'connection_timeout' => -1,
            'cookies' => $_COOKIE,
            'user-agent' => $_SERVER['HTTP_USER_AGENT'],
            'stream' => false,
            'filename' => false,
            'decompress' => false,
            'verify' => false,
            'verifyName' => false,
            'sslverify' => false
        ];
        $authHeader = $request->get_header('Authorization');
        if (null !== $authHeader) {
            $args['headers'] = ['Authorization' => $authHeader];
        }
        $response = $wpCurl->request(
            $url,
            $args
        );
        if ($response instanceof \WP_Error) {
            return rest_ensure_response(
                $response->get_error_message()
            );
        }

        /** @var CacheItem $dataCacheItem */
        $dataCacheItem = $this->persistentCache->getItem('applicant_php_datavar_data');
        $data = $dataCacheItem->get();
        $this->persistentCache->delete('applicant_php_datavar_data');

        return rest_ensure_response(
            $data
        );
    }
}