<?php

namespace BuilderiusPro\Bundle\UpdateBundle\Hook;

use Builderius\MooMoo\Platform\Bundle\HookBundle\Model\AbstractFilter;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\Provider\PluginsVersionsProvider;
use BuilderiusPro\Bundle\UpdateBundle\Helper\UpdateHelper;
use BuilderiusPro\Bundle\UpdateBundle\Model\Activation;

class CheckForUpdateHook extends AbstractFilter
{
    const VERSION_INFO_URL = 'https://my.builderius.io/wp-json/wp/v2/latest-version-info';

    /**
     * @var PluginsVersionsProvider
     */
    private $pluginsVersionsProvider;

    /**
     * @param PluginsVersionsProvider $pluginsVersionsProvider
     * @return $this
     */
    public function setPluginsVersionsProvider(PluginsVersionsProvider $pluginsVersionsProvider)
    {
        $this->pluginsVersionsProvider = $pluginsVersionsProvider;

        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getFunction()
    {
        global $pagenow;
        $transient = func_get_arg(0);
        if ( ! is_object( $transient ) ) {
            $transient = new \stdClass;
        }

        if ( 'plugins.php' == $pagenow && is_multisite() ) {
            return $transient;
        }

        if ( ! empty( $transient->response ) && ! empty( $transient->response['builderius-pro/builderius-pro.php'] ) ) {
            return $transient;
        }

        $versionInfo = UpdateHelper::getCachedVersionInfo();
        $builderiusProVersion = $this->pluginsVersionsProvider->getPluginVersion('builderius-pro');
        if ( false === $versionInfo ) {
            $activationCode = Activation::getKey();
            $builderiusVersion = $this->pluginsVersionsProvider->getPluginVersion('builderius');
            $hash = base64_encode(sprintf('%s-|-%s-|-%s', $activationCode, $builderiusVersion, $builderiusProVersion));
            $response = wp_remote_get(self::VERSION_INFO_URL . '?hash=' . urlencode($hash));

            if (is_wp_error($response) || 200 !== wp_remote_retrieve_response_code($response)) {
                return $transient;
            } else {
                $remoteVersionInfo = json_decode(wp_remote_retrieve_body($response), true);
                if (count($remoteVersionInfo) === 2 && isset($remoteVersionInfo['status']) && isset($remoteVersionInfo['message'])) {
                    update_option(Activation::ACTIVATION_STATUS_OPTION_FIELD, $remoteVersionInfo['status']);
                    update_option(Activation::ACTIVATION_DETAILS_OPTION_FIELD, $remoteVersionInfo);

                    return $transient;
                } else {
                    if (false !== $remoteVersionInfo && is_array($remoteVersionInfo)) {
                        $versionInfo = (object)$remoteVersionInfo;
                    }
                    UpdateHelper::setCachedVersionInfo($remoteVersionInfo);
                }
            }
        }

        if ( false !== $versionInfo && isset($versionInfo->new_version ) ) {
            $no_update = false;
            if ( version_compare($builderiusProVersion, $versionInfo->new_version, '<' ) ) {
                $transient->response['builderius-pro/builderius-pro.php'] = $versionInfo;
            } else {
                $no_update              = new \stdClass();
                $no_update->id          = '';
                $no_update->slug        = 'builderius-pro';
                $no_update->plugin      = 'builderius-pro/builderius-pro.php';
                $no_update->new_version = $versionInfo->new_version;
                $no_update->package     = $versionInfo->package;
                $no_update->icons       = $versionInfo->icons;
                $no_update->banners     = $versionInfo->banners;
            }

            $transient->last_checked = time();
            $transient->checked['builderius-pro/builderius-pro.php'] = $versionInfo->new_version;

            if ( $no_update ) {
                $transient->no_update['builderius-pro/builderius-pro.php'] = $no_update;
            }
        }

        return $transient;
    }
}