<?php

namespace BuilderiusPro\Bundle\UpdateBundle\Hook;

use Builderius\MooMoo\Platform\Bundle\HookBundle\Model\AbstractFilter;
use Builderius\MooMoo\Platform\Bundle\KernelBundle\Provider\PluginsVersionsProvider;
use BuilderiusPro\Bundle\UpdateBundle\Helper\UpdateHelper;
use BuilderiusPro\Bundle\UpdateBundle\Model\Activation;

class PluginInformationHook extends AbstractFilter
{
    /**
     * @var PluginsVersionsProvider
     */
    private $pluginsVersionsProvider;

    /**
     * @param PluginsVersionsProvider $pluginsVersionsProvider
     * @return $this
     */
    public function setPluginsVersionsProvider(PluginsVersionsProvider $pluginsVersionsProvider)
    {
        $this->pluginsVersionsProvider = $pluginsVersionsProvider;

        return $this;
    }


    /**
     * @inheritDoc
     */
    public function getFunction()
    {
        $data = func_get_arg(0);
        $action = func_get_arg(1);
        $args = func_get_arg(2);

        if ( $action != 'plugin_information' ) {
            return $data;
        }

        if ( ! isset( $args->slug ) || ( $args->slug != 'builderius-pro' ) ) {
            return $data;
        }

        $versionInfo = UpdateHelper::getCachedVersionInfo();
        $builderiusProVersion = $this->pluginsVersionsProvider->getPluginVersion('builderius-pro');
        if ( false === $versionInfo ) {
            $activationCode = Activation::getKey();
            $builderiusVersion = $this->pluginsVersionsProvider->getPluginVersion('builderius');
            $hash = base64_encode(sprintf('%s-|-%s-|-%s', $activationCode, $builderiusVersion, $builderiusProVersion));
            $response = wp_remote_get(CheckForUpdateHook::VERSION_INFO_URL . '?hash=' . urlencode($hash));

            if (is_wp_error($response) || 200 !== wp_remote_retrieve_response_code($response)) {
                return $data;
            } else {
                $remoteVersionInfo = json_decode(wp_remote_retrieve_body($response), true);
                if (count($remoteVersionInfo) === 2 && isset($remoteVersionInfo['status']) && isset($remoteVersionInfo['message'])) {
                    update_option(Activation::ACTIVATION_STATUS_OPTION_FIELD, $remoteVersionInfo['status']);
                    update_option(Activation::ACTIVATION_DETAILS_OPTION_FIELD, $remoteVersionInfo);

                    return $data;
                } else {
                    if (false !== $remoteVersionInfo && is_array($remoteVersionInfo)) {
                        $versionInfo = (object)$remoteVersionInfo;
                    }
                    UpdateHelper::setCachedVersionInfo($remoteVersionInfo);
                }
            }

        }
        if (!is_object($versionInfo)) {
            return $data;
        }
        $versionInfo->version = $versionInfo->new_version;
        $versionInfo->banners = (array)$versionInfo->banners;
        $versionInfo->sections = (array)$versionInfo->sections;
        $contributors = [];
        foreach ((array)$versionInfo->contributors as $name => $details) {
            $contributors[$name] = (array)$details;
        }
        $versionInfo->contributors = $contributors;

        return $versionInfo;
    }
}