<?php

namespace BuilderiusPro\Bundle\UpdateBundle\Model;

class Activation
{
    const DOMAIN_ACTIVATE_URL = 'https://my.builderius.io/wp-json/wp/v2/activate-domain';
    const DOMAIN_DEACTIVATE_URL = 'https://my.builderius.io/wp-json/wp/v2/deactivate-domain';

    const ACTIVATION_KEY_OPTION_FIELD = 'builderius_pro_activation_key';
    const ACTIVATION_DETAILS_OPTION_FIELD = 'builderius_pro_activation_details';
    const ACTIVATION_STATUS_OPTION_FIELD = 'builderius_pro_activation_status';

    /**
     * @return false|object
     */
    public static function getDetails()
    {
        return get_option(self::ACTIVATION_DETAILS_OPTION_FIELD);
    }

    /**
     * @return string|null
     */
    public static function getKey()
    {
        return trim(get_option(self::ACTIVATION_KEY_OPTION_FIELD));
    }

    /**
     * @return false|mixed|void
     */
    public static function getStatus()
    {
        return get_option(self::ACTIVATION_STATUS_OPTION_FIELD);
    }

    /**
     * @param string $activationKey
     * @return array|\WP_Error
     */
    public static function activate($activationKey)
    {
        $response = wp_remote_post(
            self::DOMAIN_ACTIVATE_URL,
            [
                'timeout' => 30,
                'body' => [
                    'activation_key' => $activationKey
                ],
            ]
        );

        if (is_wp_error($response)) {
            return [
                'status' => 'invalid',
                'message' => $response->get_error_message()
            ];
        } else {
            $activationDetails = json_decode(wp_remote_retrieve_body($response));
            if ('valid' !== $activationDetails->status) {
                return [
                    'status' => $activationDetails->status,
                    'message' => $activationDetails->message
                ];
            } else {
                update_option(self::ACTIVATION_STATUS_OPTION_FIELD, $activationDetails->status);
                update_option(self::ACTIVATION_KEY_OPTION_FIELD, $activationKey);
                update_option(self::ACTIVATION_DETAILS_OPTION_FIELD, $activationDetails);

                return [
                    'status' => $activationDetails->status,
                    'expires' => $activationDetails->expires,
                    'message' => __('Builderius PRO activated successfully.', 'builderius'),
                ];
            }
        }
    }

    /**
     * @return array|\WP_Error
     */
    public static function deactivate()
    {
        $response = wp_remote_post(
            self::DOMAIN_DEACTIVATE_URL,
            [
                'timeout' => 30,
                'body' => [
                    'activation_key' => self::getKey()
                ],
            ]
        );

        if (is_wp_error($response) || 200 !== wp_remote_retrieve_response_code($response)) {
            if (is_wp_error($response)) {
                $message = $response->get_error_message();
            } else {
                $message = __('An error occurred, please try again.', 'builderius');
            }

            return [
                'result' => false,
                'message' => $message,
            ];
        }

        json_decode(wp_remote_retrieve_body($response));

        delete_option(self::ACTIVATION_STATUS_OPTION_FIELD);
        delete_option(self::ACTIVATION_KEY_OPTION_FIELD);
        delete_option(self::ACTIVATION_DETAILS_OPTION_FIELD);

        return [
            'result' => true,
            'message' => __('Builderius PRO deactivated successfully.', 'builderius'),
        ];
    }
}