<?php

namespace BuilderiusPro\Bundle\UpdateBundle\RestApi\Endpoint;

use Builderius\Bundle\BuilderBundle\Condition\UserIsBuilderiusDeveloper;
use Builderius\MooMoo\Platform\Bundle\RestApiBundle\Endpoint\RestApiEndpointInterface;
use BuilderiusPro\Bundle\UpdateBundle\Model\Activation;

class RemoveActivationKeyEndpoint implements RestApiEndpointInterface
{
    const NAMESPACE = 'wp/v2';
    const REST_BASE = 'remove-activation-key';

    /**
     * @var UserIsBuilderiusDeveloper
     */
    private $userIsBuilderiusDeveloperCondition;

    /**
     * @param UserIsBuilderiusDeveloper $userIsBuilderiusDeveloperCondition
     */
    public function __construct(
        UserIsBuilderiusDeveloper $userIsBuilderiusDeveloperCondition
    )
    {
        $this->userIsBuilderiusDeveloperCondition = $userIsBuilderiusDeveloperCondition;
    }

    /**
     * @inheritDoc
     */
    public function registerRoutes()
    {
        register_rest_route(self::NAMESPACE, '/' . self::REST_BASE, [
            [
                'methods' => \WP_REST_Server::DELETABLE,
                'callback' => [$this, 'process'],
                'permission_callback' => [$this, 'permissionCheck'],
                'args' => [],
            ],
            'schema' => [],
        ]);
    }

    /**
     * @return bool
     */
    public function permissionCheck()
    {
        return $this->userIsBuilderiusDeveloperCondition->evaluate() === true;
    }

    /**
     * @param \WP_REST_Request $request
     * @return \WP_REST_Response|\WP_Error
     */
    public function process(\WP_REST_Request $request)
    {
        $deactivationDetails = Activation::deactivate();
        $response = new \WP_REST_Response;
        $response->set_data($deactivationDetails);

        return rest_ensure_response( $response );
    }
}
